searchData={"items":[{"type":"module","title":"LiveTable","doc":"A powerful Phoenix LiveView component library for building dynamic, interactive data tables with real-time updates. Perfect for admin panels, dashboards, and any application requiring advanced data presentation.","ref":"LiveTable.html"},{"type":"module","title":"✨ Features - LiveTable","doc":"- **🔍 Advanced Filtering** - Text search, range filters, select dropdowns, boolean toggles\n- **📊 Smart Sorting** - Multi-column sorting with shift-click support\n- **📄 Flexible Pagination** - Configurable page sizes with efficient querying\n- **📤 Export Capabilities** - CSV and PDF exports with background processing\n- **⚡ Real-time Updates** - Built for Phoenix LiveView with instant feedback\n- **🎨 Multiple View Modes** - Table and card layouts with custom components\n- **🔗 Custom Queries** - Support for complex joins and computed fields\n- **🚀 Performance Optimized** - Streams-based rendering for large datasets\n\n![LiveTable Demo](../demo.gif)\n\n**[Live Demo with 1M+ records →](https://livetable.gurujada.com)**\n\n**[Advanced Demo with custom queries, & transformer usage →](https://josaa.gurujada.com)**\n\n**[Advanced Demo Git Url →](https://github.com/ChivukulaVirinchi/college-app)**","ref":"LiveTable.html#module-features"},{"type":"module","title":"🚀 Quick Start - LiveTable","doc":"","ref":"LiveTable.html#module-quick-start"},{"type":"module","title":"1. Installation - LiveTable","doc":"Add to your `mix.exs`:\n\n```elixir\ndef deps do\n[\n  {:live_table, \"~> 0.3.0\"},\n  {:oban, \"~> 2.19\"}  # Required for exports\n]\nend\n```","ref":"LiveTable.html#module-1-installation"},{"type":"module","title":"2. Basic Configuration - LiveTable","doc":"In your `config/config.exs`:\n\n```elixir\nconfig :live_table,\nrepo: YourApp.Repo,\npubsub: YourApp.PubSub\n\n# Configure Oban for exports\nconfig :your_app, Oban,\nrepo: YourApp.Repo,\nqueues: [exports: 10]\n```","ref":"LiveTable.html#module-2-basic-configuration"},{"type":"module","title":"3. Setup Assets - LiveTable","doc":"Add to `assets/js/app.js`:\n\n```javascript\nimport hooks_default from \"../../deps/live_table/priv/static/live-table.js\";\n\nconst liveSocket = new LiveSocket(\"/live\", Socket, {\nlongPollFallbackMs: 2500,\nparams: { _csrf_token: csrfToken },\nhooks: hooks_default,\n});\n```\n\nAdd to `assets/css/app.css`:\n\n```css\n@source \"../../deps/live_table/lib\";\n\n@import \"../../deps/live_table/priv/static/live-table.css\";\n```","ref":"LiveTable.html#module-3-setup-assets"},{"type":"module","title":"4. Create Your First Table - LiveTable","doc":"LiveTable requires field & filter definitions to build a table. Additional configuration options can be defined per table under `table_options`.\n\n```elixir\n# lib/your_app_web/live/product_live/index.ex\ndefmodule YourAppWeb.ProductLive.Index do\nuse YourAppWeb, :live_view\nuse LiveTable.LiveResource, schema: YourApp.Product\n\ndef fields do\n  [\n    id: %{label: \"ID\", sortable: true},\n    name: %{label: \"Product Name\", sortable: true, searchable: true},\n    price: %{label: \"Price\", sortable: true},\n    stock_quantity: %{label: \"Stock\", sortable: true}\n  ]\nend\n\ndef filters do\n  [\n    in_stock: Boolean.new(:stock_quantity, \"in_stock\", %{\n      label: \"In Stock Only\",\n      condition: dynamic([p], p.stock_quantity > 0)\n    }),\n\n    price_range: Range.new(:price, \"price_range\", %{\n      type: :number,\n      label: \"Price Range\",\n      min: 0,\n      max: 1000\n    })\n  ]\nend\nend\n```","ref":"LiveTable.html#module-4-create-your-first-table"},{"type":"module","title":"5. Add to Your Template - LiveTable","doc":"```elixir\n# lib/your_app_web/live/product_live/index.html.heex\n<.live_table\nfields={fields()}\nfilters={filters()}\noptions={@options}\nstreams={@streams}\n/>\n```\n\nThat's it! You now have a fully functional data table with sorting, filtering, pagination, and search.","ref":"LiveTable.html#module-5-add-to-your-template"},{"type":"module","title":"🏗 Usage Patterns - LiveTable","doc":"","ref":"LiveTable.html#module-usage-patterns"},{"type":"module","title":"Simple Tables (Single Schema) - LiveTable","doc":"For basic tables querying a single schema, use the `schema:` parameter. The field keys must match the schema field names exactly:\n\n```elixir\ndefmodule YourAppWeb.UserLive.Index do\nuse YourAppWeb, :live_view\nuse LiveTable.LiveResource, schema: YourApp.User\n\ndef fields do\n  [\n    id: %{label: \"ID\", sortable: true},        # Must match User.id field\n    email: %{label: \"Email\", sortable: true, searchable: true},   # Must match User.email field\n    name: %{label: \"Name\", sortable: true, searchable: true}      # Must match User.name field\n  ]\nend\nend\n```","ref":"LiveTable.html#module-simple-tables-single-schema"},{"type":"module","title":"Complex Tables (Custom Queries) - LiveTable","doc":"For tables with joins, computed fields, or complex logic, you must define a custom data provider. The field keys must match the keys in your query's `select` clause:\n\n```elixir\ndefmodule YourAppWeb.OrderReportLive.Index do\nuse YourAppWeb, :live_view\nuse LiveTable.LiveResource\n\ndef mount(_params, _session, socket) do\n  # Assign your custom data provider\n  socket = assign(socket, :data_provider, {YourApp.Orders, :list_with_details, []})\n  {:ok, socket}\nend\n\ndef fields do\n  [\n    order_id: %{label: \"Order #\", sortable: true},        # Must match select key\n    customer_name: %{label: \"Customer\", sortable: true, searchable: true},  # Must match select key\n    total_amount: %{label: \"Total\", sortable: true},      # Must match select key\n    # For sorting by joined fields, specify the alias used in your query\n    product_name: %{\n      label: \"Product\",\n      sortable: true,\n      assoc: {:order_items, :name}    # Must match query alias and field\n    }\n  ]\nend\n\ndef filters do\n  [\n    status: Select.new({:orders, :status}, \"status\", %{\n      label: \"Order Status\",\n      options: [\n        %{label: \"Pending\", value: [\"pending\"]},\n        %{label: \"Completed\", value: [\"completed\"]}\n      ]\n    })\n  ]\nend\nend\n```\n\n```elixir\n# In your context\ndefmodule YourApp.Orders do\ndef list_with_details do\n  from o in Order,\n    join: c in Customer, on: o.customer_id == c.id,\n    join: oi in OrderItem, on: oi.order_id == o.id, as: :order_items,\n    select: %{\n      order_id: o.id,               # Field key must match this\n      customer_name: c.name,        # Field key must match this\n      total_amount: o.total_amount, # Field key must match this\n      product_name: oi.product_name # Field key must match this\n    }\nend\nend\n```","ref":"LiveTable.html#module-complex-tables-custom-queries"},{"type":"module","title":"📚 Documentation - LiveTable","doc":"- **[Installation & Setup](docs/installation.md)** - Complete setup guide\n- **[Quick Start Guide](docs/quick-start.md)** - Get up and running in 5 minutes\n- **[Configuration](docs/configuration.md)** - Customize table behavior\n- **[API Reference](docs/api/fields.md)** - Complete API documentation\n- **[Examples](docs/examples/simple-table.md)** - Real-world usage examples\n- **[Troubleshooting](docs/troubleshooting.md)** - Common issues and solutions","ref":"LiveTable.html#module-documentation"},{"type":"module","title":"🎯 Use Cases - LiveTable","doc":"LiveTable is perfect for:\n\n- **Admin Dashboards** - Manage users, orders, products with advanced filtering\n- **E-commerce Catalogs** - Product listings with search, filters, and sorting\n- **Data Analytics** - Present large datasets with exports and real-time updates\n- **CRM Systems** - Customer and lead management with custom views\n- **Inventory Management** - Track stock with complex filtering and reporting","ref":"LiveTable.html#module-use-cases"},{"type":"module","title":"📄 License - LiveTable","doc":"MIT License. See LICENSE for details.","ref":"LiveTable.html#module-license"},{"type":"module","title":"🤝 Contributing - LiveTable","doc":"1. Fork the repository\n2. Create your feature branch (`git checkout -b feature/amazing-feature`)\n3. Commit your changes (`git commit -m 'Add amazing feature'`)\n4. Push to the branch (`git push origin feature/amazing-feature`)\n5. Open a Pull Request","ref":"LiveTable.html#module-contributing"},{"type":"module","title":"📞 Support - LiveTable","doc":"- **Issues**: [GitHub Issues](https://github.com/gurujada/live_table/issues)\n- **Discussions**: [GitHub Discussions](https://github.com/gurujada/live_table/discussions)\n- **Documentation**: [API Docs](https://hexdocs.pm/live_table)\n\n---\n\nBuilt with ❤️ for the Phoenix LiveView community.","ref":"LiveTable.html#module-support"},{"type":"module","title":"LiveTable.Boolean","doc":"A module for handling boolean (checkbox) filters in LiveTable.\n\n  This module provides functionality for creating and managing boolean filters implemented\n  as checkboxes in the LiveTable interface. It's designed to handle simple true/false and even complex\n  filtering scenarios with customizable options and conditions.\n\n  Similar to all filters in LiveTable, Boolean filters can be created using the `new/3` function,\n  which takes the `key`, `field`, and an `options` map.\n  The key will be used to reference the filter in the URL params.","ref":"LiveTable.Boolean.html"},{"type":"module","title":"Options map - LiveTable.Boolean","doc":"The boolean filter accepts the following arguments in the options map:\n- `:label` - The text label displayed next to the checkbox\n- `:condition` - The Ecto query condition to be applied when the checkbox is checked (a dynamic query)\n- `:class` - Optional CSS classes for the checkbox","ref":"LiveTable.Boolean.html#module-options-map"},{"type":"module","title":"Examples - LiveTable.Boolean","doc":"```elixir\n# Creating a basic boolean filter for active status\nBoolean.new(:active, \"active_filter\", %{\n  label: \"Show Active Only\",\n  condition: dynamic([p], p.active == true)\n})\n\n# Creating a boolean filter with a complex condition\nBoolean.new(:premium, \"premium_filter\", %{\n  label: \"Premium Users\",\n  condition: dynamic([p], p.subscription_level == \"premium\" and p.active == true)\n})\n\n# Creating a boolean filter for any toggleable condition\nBoolean.new(:premium, \"premium_filter\", %{\n  label: \"Greater than 50$\",\n  condition: dynamic([p], p.price > 50)\n})\n```\nSince the condition is a dynamic query, a condition using joined fields can be given.\n```elixir\nBoolean.new(\n  :supplier_email,\n  \"supplier\",\n  %{\n    label: \"Email\",\n    condition: dynamic([p, s], s.contact_info == \"procurement@autopartsdirect.com\")\n  }\n)\n```\n> **Note**: Remember to use aliases in the same order in which they were defined in `fields()`.\n\n\nIf an associated field is not defined in the `fields()` function, and a boolean filter needs to be applied,\na tuple containing `{:table_name, field}` can be passed as the field and can be\naliased using the `table_name` in `dynamic`\n\n```elixir\nBoolean.new(\n  {:suppliers, :email},\n  \"supplier\",\n  %{\n    label: \"Email\",\n    condition: dynamic([p, s], s.contact_info == \"procurement@autopartsdirect.com\")\n  }\n)\n```","ref":"LiveTable.Boolean.html#module-examples"},{"type":"module","title":"LiveTable.Range","doc":"A module for handling range-based filters in LiveTable.\n\nThis module provides functionality for creating and managing range filters that can handle\nnumeric values, dates, and datetimes. It supports creating range sliders with customizable\noptions and appearances.\n\nMakes use of nouislider under the hood for creating the range slider.\nSee the [noUiSlider documentation](https://refreshless.com/nouislider/) for more details\non customizing the slider behavior and appearance.\n\n> **Note**: Requires the TableHooks to be imported in your app.js","ref":"LiveTable.Range.html"},{"type":"module","title":"Options - LiveTable.Range","doc":"The module accepts the following options:\n- `:type` - The type of range filter (`:number`, `:date`, or `:datetime`)\n- `:label` - The label text for the range filter\n- `:unit` - The unit to display after the label (optional)\n- `:css_classes` - CSS classes for the main container\n- `:slider_classes` - CSS classes for the slider element\n- `:label_classes` - CSS classes for the label element\n\nFor default values, see: [LiveTable.Range source code](https://github.com/gurujada/live_table/blob/master/lib/live_table/range.ex)","ref":"LiveTable.Range.html#module-options"},{"type":"module","title":"Types Support - LiveTable.Range","doc":"The module supports three types of range filters:\n- `:number` - For numeric ranges\n- `:date` - For date ranges\n- `:datetime` - For datetime ranges\n\n\n### :number\nFor numeric ranges with configurable min, max and step values:\n\n```elixir\nRange.new(:price, \"price_range\", %{\n  type: :number,\n  label: \"Price\",\n  default_min: 0,\n  default_max: 1000,\n  step: 10\n})\n# Renders a range filter with from 0 to 1000 with a step of 10.\n```\n\n### :date\nFor date ranges with customizable date boundaries:\n\n```elixir\nRange.new(:created_at, \"date_range\", %{\n  type: :date,\n  label: \"Creation Date\",\n  default_min: ~D[2024-01-01],\n  default_max: ~D[2024-12-31]\n})\n# Renders a date range filter with dates from 2024-01-01 to 2024-12-31.\n```\n\n### :datetime\nFor datetime ranges with configurable datetime boundaries:\n\n```elixir\nRange.new(:updated_at, \"datetime_range\", %{\n  type: :datetime,\n  label: \"Last Updated\",\n  default_min: ~N[2024-01-01 00:00:00],\n  default_max: ~N[2024-12-31 23:59:59],\n  step: 3600 # Step in seconds\n})\n# Renders a datetime range filter with dates from 2024-01-01 00:00:00 to 2024-12-31 23:59:59.\n# Remember to pass the datetime values in `NaiveDateTime` format.\n```","ref":"LiveTable.Range.html#module-types-support"},{"type":"module","title":"Examples - LiveTable.Range","doc":"```elixir\n# Creating a numeric range filter\nRange.new(:price, \"price_range\", %{\n  type: :number,\n  label: \"Price Range\",\n  unit: \"$\",\n  min: 0,\n  max: 1000,\n  step: 10\n})\n\n# Creating a date range filter\nRange.new(:created_at, \"date_range\", %{\n  type: :date,\n  label: \"Date Range\",\n  min: ~D[2024-01-01],\n  max: ~D[2024-12-31]\n})\n```\nIf you want to use the range filter with a joined schema, you can pass the field as a tuple, with the table name and the field-\n```elixir\nRange.new({:suppliers, :created_at}, \"created_at\", %{\n  type: :date,\n  label: \"Supplier Creation\",\n  min: ~D[2024-01-01],\n  max: ~D[2024-12-31],\n})\n```","ref":"LiveTable.Range.html#module-examples"},{"type":"module","title":"LiveTable.Select","doc":"A module for handling select-based filters in LiveTable.\n\n  This module provides functionality for creating and managing select filters that can handle\n  single or multiple selections. It supports both static options and dynamic option loading,\n  with customizable appearances and templates.","ref":"LiveTable.Select.html"},{"type":"module","title":"Options - LiveTable.Select","doc":"The module accepts the following options:\n  - `:label` - The label text for the select filter\n  - `:options` - Static list of options for the select\n  - `:options_source` - Function or module for dynamic option loading\n  - `:option_template` - Custom template for rendering options\n  - `:selected` - List of pre-selected values\n  - `:loading_text` - Text to display while loading options\n  - `:prompt` - Prompt text for the select\n  - `:placeholder` - Placeholder text for the select\n  - `:css_classes` - CSS classes for the main container\n  - `:label_classes` - CSS classes for the label element\n  - `:select_classes` - CSS classes for the select element\n\n  For default values, see: [LiveTable.Select source code](https://github.com/gurujada/live_table/blob/master/lib/live_table/select.ex)","ref":"LiveTable.Select.html#module-options"},{"type":"module","title":"Working with Options - LiveTable.Select","doc":"There are two ways to configure and display options in the select filter:","ref":"LiveTable.Select.html#module-working-with-options"},{"type":"module","title":"1. Static Options - LiveTable.Select","doc":"The simplest approach using a predefined list of options:\n\n  ```elixir\n  Select.new(:status, \"status_select\", %{\n    label: \"Status\",\n    options: [\n      %{label: \"Active\", value: [1, \"Currently active\"]},\n      %{label: \"Pending\", value: [2, \"Awaiting processing\"]},\n      %{label: \"Archived\", value: [3, \"No longer active\"]}\n    ]\n  })\n  ```","ref":"LiveTable.Select.html#module-1-static-options"},{"type":"module","title":"2. Dynamic Options via `options_source` - LiveTable.Select","doc":"Load options dynamically using a function or module. Used for fetching new options based on typed input.\nUses `apply/3` under the hood to apply the function. Uses [`live-select-change`](https://github.com/gurujada/live_table/blob/master/lib/live_table/liveview_helpers.ex#L109) event to update the options.\n\n  ```elixir\n    # Point to your custom function\n    Select.new({:suppliers, :name}, \"supplier_name\", %{\n      label: \"Supplier\",\n      options_source: {Demo.Catalog, :search_suppliers, []} # Same as you'd use for `apply/3`\n    })\n\n    # in your context module\n    def search_suppliers(text) do\n      Supplier\n      |> where([c], ilike(c.name, ^\"%#{text}%\"))\n      |> select([c], {c.name, [c.id, c.contact_info]})\n      |> Repo.all()\n    end\n  ```\n\n  You could write your function to have other args passed to it as well. Just make sure the first arg is the text.\n  It must return a tuple, with the first element being the label and the second being the value(or a list of fields).","ref":"LiveTable.Select.html#module-2-dynamic-options-via-options_source"},{"type":"module","title":"Option Templates - LiveTable.Select","doc":"You can provide custom templates for rendering options in two ways:\n  1. Using the default template format for options with label and value pairs\n  2. Providing a custom template function through the `:option_template` option","ref":"LiveTable.Select.html#module-option-templates"},{"type":"module","title":"Default Template - LiveTable.Select","doc":"The default template expects options in the format:\n  ```elixir\n  %{label: label, value: [id, description]}\n  ```\n  The default template can be seen at [git link](https://github.com/gurujada/live_table/blob/master/lib/live_table/select.ex#L211)","ref":"LiveTable.Select.html#module-default-template"},{"type":"module","title":"Custom Template - LiveTable.Select","doc":"Custom templates can be provided as functions that take an option map and return rendered HTML:\n\n  ```elixir\n  def custom_template(option) do\n    assigns = %{option: option}\n    ~H\"\"\"\n     \n       <%= @option.label %>  inas\n       <%= @option.value |> Enum.at(0) %> \n     \n    \"\"\"\n  end\n\n  # in your filter definition\n  Select.new({:suppliers, :name}, \"supplier_name\", %{\n    label: \"Supplier\",\n    placeholder: \"Search for suppliers...\",\n    options_source: {Demo.Catalog, :search_suppliers, []}\n    option_template: &custom_template/1\n  })\n  ```\n\n  Each method can be combined with others - for example, you could use dynamic or static options with\n  custom templates.","ref":"LiveTable.Select.html#module-custom-template"},{"type":"module","title":"Examples - LiveTable.Select","doc":"If the field you want to use is part of the base schema(given to `LiveResource`), you can simply pass the field name as an atom.\n  ```elixir\n  # Creating a basic select filter\n  Select.new(:category, \"category_select\", %{\n    label: \"Category\",\n    options: [\n      %{label: \"Electronics\", value: [1, \"Electronics\"]},\n      %{label: \"Books\", value: [2, \"Books\"]}\n    ]\n  })\n  ```\n\n  If its part of a joined schema, you can pass it as a tuple, with the table name(aliased in the query) and field name as shown-\n  ```elixir\n  # Creating a select filter with options loaded from database\n  Select.new({:suppliers, :name}, \"supplier_name\", %{\n      label: \"Supplier\",\n      options_source: {Demo.Catalog, :search_suppliers, []}\n    })\n  ```\n\n  ```elixir\n  # Advanced example with all options\n  Select.new({:category, :name}, \"category_name\", %{\n    label: \"Category\",\n    placeholder: \"Search for categories...\",\n    options_source: {Demo.Catalog, :search_categories, [optional args]}\n    option_template: &custom_template/1,\n  })\n  ```\n\n  Currently, nested relations are not supported.","ref":"LiveTable.Select.html#module-examples"},{"type":"module","title":"LiveTable.Transformer","doc":"","ref":"LiveTable.Transformer.html"},{"type":"function","title":"LiveTable.Transformer.apply/2","doc":"","ref":"LiveTable.Transformer.html#apply/2"},{"type":"function","title":"LiveTable.Transformer.new/2","doc":"","ref":"LiveTable.Transformer.html#new/2"},{"type":"extras","title":"LiveTable","doc":"# LiveTable\n\nA powerful Phoenix LiveView component library for building dynamic, interactive data tables with real-time updates. Perfect for admin panels, dashboards, and any application requiring advanced data presentation.","ref":"readme.html"},{"type":"extras","title":"✨ Features - LiveTable","doc":"- **🔍 Advanced Filtering** - Text search, range filters, select dropdowns, boolean toggles\n- **📊 Smart Sorting** - Multi-column sorting with shift-click support\n- **📄 Flexible Pagination** - Configurable page sizes with efficient querying\n- **📤 Export Capabilities** - CSV and PDF exports with background processing\n- **⚡ Real-time Updates** - Built for Phoenix LiveView with instant feedback\n- **🎨 Multiple View Modes** - Table and card layouts with custom components\n- **🔗 Custom Queries** - Support for complex joins and computed fields\n- **🚀 Performance Optimized** - Streams-based rendering for large datasets\n\n![LiveTable Demo](https://github.com/gurujada/live_table/blob/master/demo.gif?raw=true)\n\n**[Live Demo with 1M+ records →](https://livetable.gurujada.com)**\n\n**[Advanced Demo with custom queries, & transformer usage →](https://josaa.gurujada.com)**\n\n**[Advanced Demo Git Url →](https://github.com/ChivukulaVirinchi/college-app)**","ref":"readme.html#features"},{"type":"extras","title":"🚀 Quick Start - LiveTable","doc":"","ref":"readme.html#quick-start"},{"type":"extras","title":"1. Installation - LiveTable","doc":"Add to your `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:live_table, \"~> 0.3.0\"},\n    {:oban, \"~> 2.19\"}  # Required for exports\n  ]\nend\n```","ref":"readme.html#1-installation"},{"type":"extras","title":"2. Basic Configuration - LiveTable","doc":"In your `config/config.exs`:\n\n```elixir\nconfig :live_table,\n  repo: YourApp.Repo,\n  pubsub: YourApp.PubSub\n\n# Configure Oban for exports\nconfig :your_app, Oban,\n  repo: YourApp.Repo,\n  queues: [exports: 10]\n```","ref":"readme.html#2-basic-configuration"},{"type":"extras","title":"3. Setup Assets - LiveTable","doc":"Add to `assets/js/app.js`:\n\n```javascript\nimport hooks_default from \"../../deps/live_table/priv/static/live-table.js\";\n\nconst liveSocket = new LiveSocket(\"/live\", Socket, {\n  longPollFallbackMs: 2500,\n  params: { _csrf_token: csrfToken },\n  hooks: hooks_default,\n});\n```\n\nAdd to `assets/css/app.css`:\n\n```css\n@source \"../../deps/live_table/lib\";\n\n@import \"../../deps/live_table/priv/static/live-table.css\";\n```","ref":"readme.html#3-setup-assets"},{"type":"extras","title":"4. Create Your First Table - LiveTable","doc":"LiveTable requires field & filter definitions to build a table. Additional configuration options can be defined per table under `table_options`.\n\n```elixir\n# lib/your_app_web/live/product_live/index.ex\ndefmodule YourAppWeb.ProductLive.Index do\n  use YourAppWeb, :live_view\n  use LiveTable.LiveResource, schema: YourApp.Product\n\n  def fields do\n    [\n      id: %{label: \"ID\", sortable: true},\n      name: %{label: \"Product Name\", sortable: true, searchable: true},\n      price: %{label: \"Price\", sortable: true},\n      stock_quantity: %{label: \"Stock\", sortable: true}\n    ]\n  end\n\n  def filters do\n    [\n      in_stock: Boolean.new(:stock_quantity, \"in_stock\", %{\n        label: \"In Stock Only\",\n        condition: dynamic([p], p.stock_quantity > 0)\n      }),\n\n      price_range: Range.new(:price, \"price_range\", %{\n        type: :number,\n        label: \"Price Range\",\n        min: 0,\n        max: 1000\n      })\n    ]\n  end\nend\n```","ref":"readme.html#4-create-your-first-table"},{"type":"extras","title":"5. Add to Your Template - LiveTable","doc":"```elixir\n# lib/your_app_web/live/product_live/index.html.heex\n<.live_table\n  fields={fields()}\n  filters={filters()}\n  options={@options}\n  streams={@streams}\n/>\n```\n\nThat's it! You now have a fully functional data table with sorting, filtering, pagination, and search.","ref":"readme.html#5-add-to-your-template"},{"type":"extras","title":"🏗 Usage Patterns - LiveTable","doc":"","ref":"readme.html#usage-patterns"},{"type":"extras","title":"Simple Tables (Single Schema) - LiveTable","doc":"For basic tables querying a single schema, use the `schema:` parameter. The field keys must match the schema field names exactly:\n\n```elixir\ndefmodule YourAppWeb.UserLive.Index do\n  use YourAppWeb, :live_view\n  use LiveTable.LiveResource, schema: YourApp.User\n\n  def fields do\n    [\n      id: %{label: \"ID\", sortable: true},        # Must match User.id field\n      email: %{label: \"Email\", sortable: true, searchable: true},   # Must match User.email field\n      name: %{label: \"Name\", sortable: true, searchable: true}      # Must match User.name field\n    ]\n  end\nend\n```","ref":"readme.html#simple-tables-single-schema"},{"type":"extras","title":"Complex Tables (Custom Queries) - LiveTable","doc":"For tables with joins, computed fields, or complex logic, you must define a custom data provider. The field keys must match the keys in your query's `select` clause:\n\n```elixir\ndefmodule YourAppWeb.OrderReportLive.Index do\n  use YourAppWeb, :live_view\n  use LiveTable.LiveResource\n\n  def mount(_params, _session, socket) do\n    # Assign your custom data provider\n    socket = assign(socket, :data_provider, {YourApp.Orders, :list_with_details, []})\n    {:ok, socket}\n  end\n\n  def fields do\n    [\n      order_id: %{label: \"Order #\", sortable: true},        # Must match select key\n      customer_name: %{label: \"Customer\", sortable: true, searchable: true},  # Must match select key\n      total_amount: %{label: \"Total\", sortable: true},      # Must match select key\n      # For sorting by joined fields, specify the alias used in your query\n      product_name: %{\n        label: \"Product\",\n        sortable: true,\n        assoc: {:order_items, :name}    # Must match query alias and field\n      }\n    ]\n  end\n\n  def filters do\n    [\n      status: Select.new({:orders, :status}, \"status\", %{\n        label: \"Order Status\",\n        options: [\n          %{label: \"Pending\", value: [\"pending\"]},\n          %{label: \"Completed\", value: [\"completed\"]}\n        ]\n      })\n    ]\n  end\nend\n```\n\n```elixir\n# In your context\ndefmodule YourApp.Orders do\n  def list_with_details do\n    from o in Order,\n      join: c in Customer, on: o.customer_id == c.id,\n      join: oi in OrderItem, on: oi.order_id == o.id, as: :order_items,\n      select: %{\n        order_id: o.id,               # Field key must match this\n        customer_name: c.name,        # Field key must match this\n        total_amount: o.total_amount, # Field key must match this\n        product_name: oi.product_name # Field key must match this\n      }\n  end\nend\n```","ref":"readme.html#complex-tables-custom-queries"},{"type":"extras","title":"📚 Documentation - LiveTable","doc":"- **[Installation & Setup](docs/installation.md)** - Complete setup guide\n- **[Quick Start Guide](docs/quick-start.md)** - Get up and running in 5 minutes\n- **[Configuration](docs/configuration.md)** - Customize table behavior\n- **[API Reference](docs/api/fields.md)** - Complete API documentation\n- **[Examples](docs/examples/simple-table.md)** - Real-world usage examples\n- **[Troubleshooting](docs/troubleshooting.md)** - Common issues and solutions","ref":"readme.html#documentation"},{"type":"extras","title":"🎯 Use Cases - LiveTable","doc":"LiveTable is perfect for:\n\n- **Admin Dashboards** - Manage users, orders, products with advanced filtering\n- **E-commerce Catalogs** - Product listings with search, filters, and sorting\n- **Data Analytics** - Present large datasets with exports and real-time updates\n- **CRM Systems** - Customer and lead management with custom views\n- **Inventory Management** - Track stock with complex filtering and reporting","ref":"readme.html#use-cases"},{"type":"extras","title":"📄 License - LiveTable","doc":"MIT License. See LICENSE for details.","ref":"readme.html#license"},{"type":"extras","title":"🤝 Contributing - LiveTable","doc":"1. Fork the repository\n2. Create your feature branch (`git checkout -b feature/amazing-feature`)\n3. Commit your changes (`git commit -m 'Add amazing feature'`)\n4. Push to the branch (`git push origin feature/amazing-feature`)\n5. Open a Pull Request","ref":"readme.html#contributing"},{"type":"extras","title":"📞 Support - LiveTable","doc":"- **Issues**: [GitHub Issues](https://github.com/gurujada/live_table/issues)\n- **Discussions**: [GitHub Discussions](https://github.com/gurujada/live_table/discussions)\n- **Documentation**: [API Docs](https://hexdocs.pm/live_table)\n\n---\n\nBuilt with ❤️ for the Phoenix LiveView community.","ref":"readme.html#support"},{"type":"extras","title":"LiveTable Overview","doc":"# LiveTable Overview\n\nLiveTable is a powerful Phoenix LiveView component library that transforms data presentation in your web applications. It provides everything you need to build sophisticated, interactive data tables with minimal code.","ref":"overview.html"},{"type":"extras","title":"What is LiveTable? - LiveTable Overview","doc":"LiveTable is designed for developers who need to present complex data in an intuitive, interactive way. Whether you're building admin dashboards, e-commerce catalogs, or analytics platforms, LiveTable provides the tools to create rich data experiences without the complexity.","ref":"overview.html#what-is-livetable"},{"type":"extras","title":"Key Benefits - LiveTable Overview","doc":"- **Developer-Friendly** - Minimal boilerplate, maximum functionality\n- **Performance-First** - Optimized for large datasets with streaming and pagination\n- **Highly Customizable** - Adapt to any design system or use case\n- **Real-time Ready** - Built for Phoenix LiveView's real-time capabilities\n- **Production-Tested** - Handles millions of records efficiently","ref":"overview.html#key-benefits"},{"type":"extras","title":"Core Features - LiveTable Overview","doc":"","ref":"overview.html#core-features"},{"type":"extras","title":"🔍 Smart Filtering - LiveTable Overview","doc":"Create powerful filter interfaces with minimal configuration:\n- **Boolean filters** for toggle states (active/inactive, featured/regular)\n- **Range filters** for numeric values, dates, and times\n- **Select filters** with static options or dynamic database queries\n- **Transformers** for complex custom filters that can modify entire queries\n- **Full-text search** across multiple fields\n- **URL persistence** so users can bookmark and share filtered views","ref":"overview.html#smart-filtering"},{"type":"extras","title":"📊 Advanced Sorting - LiveTable Overview","doc":"Flexible sorting options that scale:\n- **Single-column sorting** with visual indicators\n- **Multi-column sorting** with shift+click\n- **Custom sort logic** for computed fields\n- **Database-optimized** queries for performance","ref":"overview.html#advanced-sorting"},{"type":"extras","title":"📄 Intelligent Pagination - LiveTable Overview","doc":"Handle datasets of any size:\n- **Configurable page sizes** (10, 25, 50, 100+)\n- **Efficient querying** with LIMIT/OFFSET optimization\n- **User preferences** maintained across sessions\n- **Performance monitoring** for large datasets","ref":"overview.html#intelligent-pagination"},{"type":"extras","title":"📤 Professional Exports - LiveTable Overview","doc":"Generate reports without blocking your UI:\n- **CSV exports** for spreadsheet analysis\n- **PDF reports** with professional formatting using Typst\n- **Background processing** via Oban for large datasets\n- **Custom formatting** and filtering in exports","ref":"overview.html#professional-exports"},{"type":"extras","title":"🎨 Flexible Display Modes - LiveTable Overview","doc":"Present data the way your users need it:\n- **Table mode** for traditional row/column layouts\n- **Card mode** for rich, visual presentations\n- **Custom components** for specialized displays\n- **Complete UI customization** with custom headers, content, and footers\n- **Responsive design** that works on all devices","ref":"overview.html#flexible-display-modes"},{"type":"extras","title":"⚡ Real-time Updates - LiveTable Overview","doc":"Built for modern web applications:\n- **LiveView integration** with automatic updates\n- **Streams or assigns support** for flexible data handling\n- **Event handling** for user interactions\n- **State management** with URL synchronization","ref":"overview.html#real-time-updates"},{"type":"extras","title":"Architecture Overview - LiveTable Overview","doc":"LiveTable uses a three-tier architecture that separates concerns and maximizes flexibility:","ref":"overview.html#architecture-overview"},{"type":"extras","title":"1. Data Layer - LiveTable Overview","doc":"- **Schema Integration** - Direct Ecto schema support for simple tables\n- **Custom Queries** - Full control with complex joins and computed fields\n- **Data Providers** - Flexible functions that return queryable data","ref":"overview.html#1-data-layer"},{"type":"extras","title":"2. Configuration Layer - LiveTable Overview","doc":"- **Fields** - Define what data to display and how\n- **Filters** - Configure filtering options and behavior\n- **Options** - Control pagination, sorting, exports, and display modes","ref":"overview.html#2-configuration-layer"},{"type":"extras","title":"3. Presentation Layer - LiveTable Overview","doc":"- **Components** - Customizable UI components for every element\n- **Templates** - Override default layouts with your own\n- **Styling** - Full Tailwind CSS integration with custom themes","ref":"overview.html#3-presentation-layer"},{"type":"extras","title":"Usage Patterns - LiveTable Overview","doc":"","ref":"overview.html#usage-patterns"},{"type":"extras","title":"Simple Tables - LiveTable Overview","doc":"Perfect for basic CRUD operations and data browsing:\n\n```elixir\ndefmodule YourAppWeb.ProductLive.Index do\n  use YourAppWeb, :live_view\n  use LiveTable.LiveResource, schema: YourApp.Product\n\n  def fields do\n    [\n      name: %{label: \"Product Name\", sortable: true, searchable: true},\n      price: %{label: \"Price\", sortable: true},\n      stock: %{label: \"Stock\", sortable: true}\n    ]\n  end\nend\n```","ref":"overview.html#simple-tables"},{"type":"extras","title":"Complex Analytics - LiveTable Overview","doc":"For advanced reporting and data analysis:\n\n```elixir\ndefmodule YourAppWeb.SalesReportLive.Index do\n  use YourAppWeb, :live_view\n  use LiveTable.LiveResource\n\n  def mount(_params, _session, socket) do\n    socket = assign(socket, :data_provider, {Reports, :sales_summary, []})\n    {:ok, socket}\n  end\n\n  def fields do\n    [\n      product_name: %{label: \"Product\", sortable: true, searchable: true},\n      revenue: %{label: \"Revenue\", sortable: true, renderer: &format_currency/1},\n      units_sold: %{label: \"Units\", sortable: true},\n      avg_price: %{label: \"Avg Price\", sortable: true, renderer: &format_currency/1}\n    ]\n  end\nend\n```","ref":"overview.html#complex-analytics"},{"type":"extras","title":"When to Use LiveTable - LiveTable Overview","doc":"","ref":"overview.html#when-to-use-livetable"},{"type":"extras","title":"✅ Perfect For - LiveTable Overview","doc":"- **Admin interfaces** with user, order, and product management\n- **E-commerce catalogs** with filtering and search\n- **Analytics dashboards** with large datasets and custom metrics\n- **CRM systems** with customer and lead tracking\n- **Inventory management** with stock monitoring\n- **Financial reports** with complex calculations and transformations\n- **Any data-heavy application** requiring advanced filtering and customization","ref":"overview.html#perfect-for"},{"type":"extras","title":"⚠️ Consider Alternatives For - LiveTable Overview","doc":"- **Simple lists** without sorting or filtering needs\n- **Static data** that doesn't change frequently\n- **Non-tabular data** better suited for other presentations\n- **Real-time streaming** data that updates continuously","ref":"overview.html#consider-alternatives-for"},{"type":"extras","title":"Performance Characteristics - LiveTable Overview","doc":"LiveTable is designed to handle real-world application demands:","ref":"overview.html#performance-characteristics"},{"type":"extras","title":"Scalability - LiveTable Overview","doc":"- **Large datasets** - Tested with 1M+ records\n- **Complex queries** - Efficient handling of joins and aggregations\n- **Concurrent users** - Optimized for multi-user environments\n- **Memory efficiency** - Streaming exports for large files","ref":"overview.html#scalability"},{"type":"extras","title":"Optimization Strategies - LiveTable Overview","doc":"- **Query optimization** - Automatic LIMIT/OFFSET for pagination\n- **Index-aware** - Designed to work with database indexes\n- **Lazy loading** - Components render only when needed\n- **Background processing** - Exports don't block the UI","ref":"overview.html#optimization-strategies"},{"type":"extras","title":"Integration Requirements - LiveTable Overview","doc":"","ref":"overview.html#integration-requirements"},{"type":"extras","title":"Dependencies - LiveTable Overview","doc":"- **Phoenix LiveView 1.0+** - Core framework dependency\n- **Ecto 3.10+** - Database query interface\n- **Oban 2.19+** - Background job processing for exports\n- **Tailwind CSS** - Styling framework","ref":"overview.html#dependencies"},{"type":"extras","title":"Optional Dependencies - LiveTable Overview","doc":"- **Typst** - PDF generation engine for reports\n- **Phoenix PubSub** - Real-time updates across users\n- **Custom CSS frameworks** - Alternative to Tailwind","ref":"overview.html#optional-dependencies"},{"type":"extras","title":"Learning Path - LiveTable Overview","doc":"","ref":"overview.html#learning-path"},{"type":"extras","title":"1. Start Here - LiveTable Overview","doc":"- [Installation Guide](installation.md) - Get LiveTable running\n- [Quick Start](quick-start.md) - Build your first table in 5 minutes\n- [Simple Examples](examples/simple-table.md) - Basic patterns and usage","ref":"overview.html#1-start-here"},{"type":"extras","title":"2. Core Concepts - LiveTable Overview","doc":"- [Fields API](api/fields.md) - Configure data display\n- [Filters API](api/filters.md) - Add filtering capabilities\n- [Table Options](api/table-options.md) - Control behavior and appearance","ref":"overview.html#2-core-concepts"},{"type":"extras","title":"3. Advanced Features - LiveTable Overview","doc":"- [Complex Queries](examples/complex-queries.md) - Joins and computed fields\n- [Transformers API](api/transformers.md) - Ultimate filtering power\n- [Exports API](api/exports.md) - PDF and CSV generation\n- [Custom Components](configuration.md#custom-components) - Complete UI control","ref":"overview.html#3-advanced-features"},{"type":"extras","title":"4. Production Ready - LiveTable Overview","doc":"- [Performance Guide](troubleshooting.md#performance-issues) - Optimize for scale\n- [Troubleshooting](troubleshooting.md) - Solve common issues\n- [Configuration Reference](configuration.md) - Complete options guide","ref":"overview.html#4-production-ready"},{"type":"extras","title":"Real-World Examples - LiveTable Overview","doc":"","ref":"overview.html#real-world-examples"},{"type":"extras","title":"E-commerce Product Catalog - LiveTable Overview","doc":"```elixir\n# Feature-rich product listing with category filtering,\n# price ranges, stock status, and bulk actions\ndef fields do\n  [\n    image: %{label: \"Image\", renderer: &product_image/1},\n    name: %{label: \"Product\", sortable: true, searchable: true},\n    category: %{label: \"Category\", sortable: true},\n    price: %{label: \"Price\", sortable: true, renderer: &currency/1},\n    stock: %{label: \"Stock\", sortable: true, renderer: &stock_status/2},\n    actions: %{label: \"Actions\", renderer: &product_actions/2}\n  ]\nend\n```","ref":"overview.html#e-commerce-product-catalog"},{"type":"extras","title":"Customer Analytics Dashboard - LiveTable Overview","doc":"```elixir\n# Customer lifetime value analysis with cohort filtering,\n# revenue tracking, and churn prediction\ndef fields do\n  [\n    customer_name: %{label: \"Customer\", sortable: true, searchable: true},\n    total_orders: %{label: \"Orders\", sortable: true},\n    lifetime_value: %{label: \"LTV\", sortable: true, renderer: &currency/1},\n    last_order: %{label: \"Last Order\", sortable: true, renderer: &date/1},\n    churn_risk: %{label: \"Churn Risk\", sortable: true, renderer: &risk_score/1}\n  ]\nend\n```","ref":"overview.html#customer-analytics-dashboard"},{"type":"extras","title":"Inventory Management System - LiveTable Overview","doc":"```elixir\n# Stock monitoring with reorder alerts, supplier tracking,\n# and automated inventory actions\ndef fields do\n  [\n    sku: %{label: \"SKU\", sortable: true, searchable: true},\n    product_name: %{label: \"Product\", sortable: true, searchable: true},\n    current_stock: %{label: \"Stock\", sortable: true, renderer: &stock_level/2},\n    reorder_point: %{label: \"Reorder At\", sortable: true},\n    supplier: %{label: \"Supplier\", sortable: true, searchable: true},\n    last_restock: %{label: \"Last Restock\", sortable: true, renderer: &date/1}\n  ]\nend\n```","ref":"overview.html#inventory-management-system"},{"type":"extras","title":"Community and Support - LiveTable Overview","doc":"","ref":"overview.html#community-and-support"},{"type":"extras","title":"Getting Help - LiveTable Overview","doc":"- **Documentation** - Comprehensive guides and API reference\n- **GitHub Issues** - Bug reports and feature requests\n- **GitHub Discussions** - Community Q&A and tips\n- **Examples Repository** - Real-world implementation patterns","ref":"overview.html#getting-help"},{"type":"extras","title":"Key Documentation - LiveTable Overview","doc":"- **[Transformers API](api/transformers.md)** - Most powerful filtering feature\n- **[Configuration Guide](configuration.md)** - Streams vs assigns, custom components\n- **[Field & Filter APIs](api/fields.md)** - Complete reference for all filter types","ref":"overview.html#key-documentation"},{"type":"extras","title":"Contributing - LiveTable Overview","doc":"LiveTable is open source and welcomes contributions:\n- **Bug fixes** and performance improvements\n- **Feature development** for new capabilities\n- **Documentation** improvements and examples\n- **Testing** across different environments","ref":"overview.html#contributing"},{"type":"extras","title":"Roadmap - LiveTable Overview","doc":"- **Enhanced filtering** with date ranges and custom operators\n- **Advanced exports** with custom templates and scheduling\n- **Real-time collaboration** features\n- **Mobile-optimized** responsive layouts\n- **Accessibility improvements** for better WCAG compliance\n\n---\n\nLiveTable empowers Phoenix developers to build sophisticated data interfaces without sacrificing simplicity or performance. Whether you're creating admin panels, customer dashboards, or analytics platforms, LiveTable provides the foundation for exceptional user experiences.\n\nReady to get started? Check out the [Installation Guide](installation.md) and build your first table in minutes.","ref":"overview.html#roadmap"},{"type":"extras","title":"Installation","doc":"# Installation\n\nThis guide will walk you through setting up LiveTable in your Phoenix application.","ref":"installation.html"},{"type":"extras","title":"Prerequisites - Installation","doc":"Before installing LiveTable, ensure you have:\n\n- **Elixir** 1.14 or later\n- **Phoenix** 1.7+ with LiveView 1.0+\n- **Ecto** 3.10+","ref":"installation.html#prerequisites"},{"type":"extras","title":"Step 1: Add Dependencies - Installation","doc":"Add LiveTable to your `mix.exs` dependencies:\n\n```elixir\ndef deps do\n  [\n    {:live_table, \"~> 0.3.0\"},\n    {:oban, \"~> 2.19\"}           # Required for background exports\n  ]\nend\n```\n\nRun the dependency installation:\n\n```bash\nmix deps.get\n```","ref":"installation.html#step-1-add-dependencies"},{"type":"extras","title":"Step 2: Application Configuration - Installation","doc":"Configure LiveTable in your `config/config.exs`:\n\n```elixir\nconfig :live_table,\n  repo: YourApp.Repo,\n  pubsub: YourApp.PubSub\n```","ref":"installation.html#step-2-application-configuration"},{"type":"extras","title":"Oban Configuration (Required for Exports) - Installation","doc":"Add Oban configuration for background job processing:\n\n```elixir\n# config/config.exs\nconfig :your_app, Oban,\n  repo: YourApp.Repo,\n  engine: Oban.Engines.Basic,\n  notifier: Oban.Notifiers.Postgres,\n  plugins: [Oban.Plugins.Pruner],\n  queues: [exports: 10]\n```\n\nAdd Oban to your supervision tree in `lib/your_app/application.ex`:\n\n```elixir\ndef start(_type, _args) do\n  children = [\n    # ... your existing children\n    {Oban, Application.fetch_env!(:your_app, Oban)}\n  ]\n  \n  opts = [strategy: :one_for_one, name: YourApp.Supervisor]\n  Supervisor.start_link(children, opts)\nend\n```","ref":"installation.html#oban-configuration-required-for-exports"},{"type":"extras","title":"Step 3: Asset Setup - Installation","doc":"","ref":"installation.html#step-3-asset-setup"},{"type":"extras","title":"Configure Tailwind CSS - Installation","doc":"Update your `assets/tailwind.config.js`:\n\n```javascript\nmodule.exports = {\n  content: [\n    \"./js/**/*.js\",\n    \"../lib/*_web.ex\",\n    \"../lib/*_web/**/*.*ex\",\n    // Add these LiveTable paths\n    \"../deps/live_table/priv/static/*.js\",\n    \"../deps/live_table/**/*.*ex\"\n  ],\n  theme: {\n    extend: {},\n  },\n  plugins: [\n    require(\"@tailwindcss/forms\")\n  ]\n}\n```","ref":"installation.html#configure-tailwind-css"},{"type":"extras","title":"Add JavaScript Hooks - Installation","doc":"Add LiveTable hooks to your `assets/js/app.js`:\n\n```javascript\n// Import LiveTable hooks\nimport { TableHooks } from \"../../deps/live_table/priv/static/live-table.js\"\n\n// Your existing imports\nimport {Socket} from \"phoenix\"\nimport {LiveSocket} from \"phoenix_live_view\"\nimport topbar from \"../vendor/topbar\"\n\nlet csrfToken = document.querySelector(\"meta[name='csrf-token']\").getAttribute(\"content\")\n\n// Add TableHooks to your LiveSocket\nlet liveSocket = new LiveSocket(\"/live\", Socket, {\n  params: {_csrf_token: csrfToken},\n  hooks: TableHooks  // Add this line\n})\n\n// Connect if there are any LiveViews on the page\nliveSocket.connect()\n\n// Show progress bar on live navigation and form submits\ntopbar.config({barColors: {0: \"#29d\"}, shadowColor: \"rgba(0, 0, 0, .3)\"})\nwindow.addEventListener(\"phx:page-loading-start\", info => topbar.show())\nwindow.addEventListener(\"phx:page-loading-stop\", info => topbar.hide())\n\n// Make liveSocket available for debugging\nwindow.liveSocket = liveSocket\n```","ref":"installation.html#add-javascript-hooks"},{"type":"extras","title":"Add CSS Styles - Installation","doc":"Add LiveTable styles to your `assets/css/app.css`:\n\n```css\n@import \"tailwindcss/base\";\n@import \"tailwindcss/components\";\n@import \"tailwindcss/utilities\";\n\n/* Add LiveTable styles */\n@import \"../../deps/live_table/priv/static/live-table.css\";\n\n/* Your existing styles */\n```","ref":"installation.html#add-css-styles"},{"type":"extras","title":"Step 4: Database Setup - Installation","doc":"Run migrations to set up Oban tables:\n\n```bash\nmix ecto.create_migration add_oban_jobs_table\n```\n\nUse the Oban migration generator:\n\n```bash\nmix oban.install\nmix ecto.migrate\n```","ref":"installation.html#step-4-database-setup"},{"type":"extras","title":"Step 5: Static File Configuration - Installation","doc":"Add `exports` to your allowed static paths in `lib/your_app_web.ex`:\n\n```elixir\ndef static_paths, do: ~w(assets fonts images favicon.ico robots.txt exports)\n```\n\nThis allows LiveTable to serve generated export files.","ref":"installation.html#step-5-static-file-configuration"},{"type":"extras","title":"Step 6: PDF Export Setup (Optional) - Installation","doc":"For PDF exports, install Typst on your system:","ref":"installation.html#step-6-pdf-export-setup-optional"},{"type":"extras","title":"macOS - Installation","doc":"```bash\nbrew install typst\n```","ref":"installation.html#macos"},{"type":"extras","title":"Ubuntu/Debian - Installation","doc":"```bash\n# Download latest release from GitHub\nwget https://github.com/typst/typst/releases/latest/download/typst-x86_64-unknown-linux-musl.tar.xz\ntar -xf typst-x86_64-unknown-linux-musl.tar.xz\nsudo mv typst-x86_64-unknown-linux-musl/typst /usr/local/bin/\n```","ref":"installation.html#ubuntu-debian"},{"type":"extras","title":"Windows - Installation","doc":"Download from [Typst Releases](https://github.com/typst/typst/releases) and add to PATH.","ref":"installation.html#windows"},{"type":"extras","title":"Verify Installation - Installation","doc":"```bash\ntypst --version\n```","ref":"installation.html#verify-installation"},{"type":"extras","title":"Step 7: Verification - Installation","doc":"Create a simple test to verify everything is working:\n\n```elixir\n# lib/your_app_web/live/test_table_live.ex\ndefmodule YourAppWeb.TestTableLive do\n  use YourAppWeb, :live_view\n  use LiveTable.LiveResource, schema: YourApp.User\n\n  def fields do\n    [\n      id: %{label: \"ID\", sortable: true},\n      email: %{label: \"Email\", sortable: true, searchable: true}\n    ]\n  end\n\n  def filters, do: []\nend\n```\n\nAdd a route in `router.ex`:\n\n```elixir\nscope \"/\", YourAppWeb do\n  pipe_through :browser\n  \n  live \"/test-table\", TestTableLive\nend\n```\n\nVisit `/test-table` to see your table in action!","ref":"installation.html#step-7-verification"},{"type":"extras","title":"Troubleshooting - Installation","doc":"","ref":"installation.html#troubleshooting"},{"type":"extras","title":"Common Issues - Installation","doc":"**Hooks not working**: Verify TableHooks are properly imported and added to LiveSocket.\n\n**Styling issues**: Make sure Tailwind CSS is properly configured and processing LiveTable classes.\n\n**Export errors**: Check that Oban is running and the exports queue is configured.","ref":"installation.html#common-issues"},{"type":"extras","title":"Development vs Production - Installation","doc":"**Development**: Assets are compiled automatically with `mix phx.server`\n\n**Production**: Run `mix assets.deploy` to compile assets, or use your deployment pipeline.","ref":"installation.html#development-vs-production"},{"type":"extras","title":"Next Steps - Installation","doc":"- Read the [Quick Start Guide](quick-start.md) to build your first table\n- Explore [Configuration Options](configuration.md) to customize behavior\n- Check out [Examples](examples/simple-table.md) for real-world usage patterns","ref":"installation.html#next-steps"},{"type":"extras","title":"Quick Start Guide","doc":"# Quick Start Guide\n\nGet up and running with LiveTable in 5 minutes! This guide assumes you've already completed the [installation](installation.md).","ref":"quick-start.html"},{"type":"extras","title":"What We'll Build - Quick Start Guide","doc":"We'll create a product catalog table for an e-commerce application with:\n- Product listing with sorting and search\n- Price range filtering\n- Stock status filtering\n- Export functionality","ref":"quick-start.html#what-we-ll-build"},{"type":"extras","title":"Step 1: Create Your Schema - Quick Start Guide","doc":"First, let's define a simple Product schema:\n\n```elixir\n# lib/your_app/catalog/product.ex\ndefmodule YourApp.Catalog.Product do\n  use Ecto.Schema\n  \n  schema \"products\" do\n    field :name, :string\n    field :description, :string\n    field :price, :decimal\n    field :stock_quantity, :integer\n    field :active, :boolean, default: true\n    field :sku, :string\n    \n    timestamps()\n  end\nend\n```","ref":"quick-start.html#step-1-create-your-schema"},{"type":"extras","title":"Step 2: Create a Simple LiveView - Quick Start Guide","doc":"Create your products listing LiveView for a single table:\n\n```elixir\n# lib/your_app_web/live/product_live/index.ex\ndefmodule YourAppWeb.ProductLive.Index do\n  use YourAppWeb, :live_view\n  use LiveTable.LiveResource, schema: YourApp.Catalog.Product\n  \n  # Define your table columns\n  def fields do\n    [\n      id: %{\n        label: \"ID\",\n        sortable: true\n      },\n      name: %{\n        label: \"Product Name\",\n        sortable: true,\n        searchable: true\n      },\n      sku: %{\n        label: \"SKU\",\n        sortable: true,\n        searchable: true\n      },\n      price: %{\n        label: \"Price\",\n        sortable: true,\n        renderer: &format_price/1\n      },\n      stock_quantity: %{\n        label: \"Stock\",\n        sortable: true\n      },\n      active: %{\n        label: \"Status\",\n        sortable: true,\n        renderer: &format_status/1\n      }\n    ]\n  end\n  \n  # Define your filters\n  def filters do\n    [\n      # Boolean filter for active products\n      active_only: Boolean.new(:active, \"active\", %{\n        label: \"Active Products Only\",\n        condition: dynamic([p], p.active == true)\n      }),\n      \n      # Range filter for price\n      price_range: Range.new(:price, \"price_range\", %{\n        type: :number,\n        label: \"Price Range\",\n        unit: \"$\",\n        min: 0,\n        max: 1000,\n        step: 10\n      }),\n      \n      # Boolean filter for low stock\n      low_stock: Boolean.new(:stock_quantity, \"low_stock\", %{\n        label: \"Low Stock (< 10)\",\n        condition: dynamic([p], p.stock_quantity < 10)\n      })\n    ]\n  end\n  \n  # Custom renderers for better display\n  defp format_price(price) do\n    assigns = %{price: price}\n    ~H\"\"\"\n     \n      $<%= :erlang.float_to_binary(@price, decimals: 2) %>\n     \n    \"\"\"\n  end\n  \n  defp format_status(active) do\n    assigns = %{active: active}\n    ~H\"\"\"\n     \n      <%= if @active, do: \"Active\", else: \"Inactive\" %>\n     \n    \"\"\"\n  end\nend\n```","ref":"quick-start.html#step-2-create-a-simple-liveview"},{"type":"extras","title":"Step 3: Create the Template - Quick Start Guide","doc":"Create the template file:\n\n```elixir\n# lib/your_app_web/live/product_live/index.html.heex\n \n   \n     Product Catalog \n     \n      Manage your product inventory with advanced filtering and search.\n     \n   \n\n  <.live_table\n    fields={fields()}\n    filters={filters()}\n    options={@options}\n    streams={@streams}\n  />\n \n```","ref":"quick-start.html#step-3-create-the-template"},{"type":"extras","title":"Step 4: Add Routes - Quick Start Guide","doc":"Add the route to your router:\n\n```elixir\n# lib/your_app_web/router.ex\nscope \"/\", YourAppWeb do\n  pipe_through :browser\n  \n  live \"/products\", ProductLive.Index, :index\nend\n```","ref":"quick-start.html#step-4-add-routes"},{"type":"extras","title":"Step 5: Seed Some Data - Quick Start Guide","doc":"Create some sample data to test with:\n\n```elixir\n# priv/repo/seeds.exs\nalias YourApp.Repo\nalias YourApp.Catalog.Product\n\n# Create products\nproducts = [\n  %Product{\n    name: \"iPhone 15 Pro\",\n    description: \"Latest Apple smartphone\",\n    price: Decimal.new(\"999.99\"),\n    stock_quantity: 25,\n    active: true,\n    sku: \"IPHONE15PRO\"\n  },\n  %Product{\n    name: \"MacBook Air M2\",\n    description: \"Apple laptop with M2 chip\",\n    price: Decimal.new(\"1199.99\"),\n    stock_quantity: 8,\n    active: true,\n    sku: \"MACBOOKAIR\"\n  },\n  %Product{\n    name: \"Wireless Mouse\",\n    description: \"Ergonomic wireless mouse\",\n    price: Decimal.new(\"45.99\"),\n    stock_quantity: 50,\n    active: true,\n    sku: \"WMOUSE01\"\n  },\n  %Product{\n    name: \"USB Cable\",\n    description: \"High-quality USB-C cable\",\n    price: Decimal.new(\"24.99\"),\n    stock_quantity: 3,\n    active: true,\n    sku: \"USBCABLE\"\n  },\n  %Product{\n    name: \"Discontinued Phone\",\n    description: \"No longer available\",\n    price: Decimal.new(\"299.99\"),\n    stock_quantity: 0,\n    active: false,\n    sku: \"OLDPHONE\"\n  }\n]\n\nEnum.each(products, &Repo.insert!/1)\n```\n\nRun the seeds:\n\n```bash\nmix run priv/repo/seeds.exs\n```","ref":"quick-start.html#step-5-seed-some-data"},{"type":"extras","title":"Step 6: Test Your Table - Quick Start Guide","doc":"Start your server and visit your new table:\n\n```bash\nmix phx.server\n```\n\nNavigate to `http://localhost:4000/products` and you should see:\n\n✅ **A fully functional data table** with your products\n✅ **Sortable columns** - click any column header to sort\n✅ **Multi-column sorting** - hold Shift and click multiple headers\n✅ **Search functionality** - type in the search box to filter products\n✅ **Advanced filters** - toggle active products, adjust price range\n✅ **Pagination** - if you have many products\n✅ **Export buttons** - download CSV or PDF reports","ref":"quick-start.html#step-6-test-your-table"},{"type":"extras","title":"Advanced Example: Custom Query with Joins - Quick Start Guide","doc":"For more complex scenarios, you can use custom queries:\n\n```elixir\n# lib/your_app_web/live/order_report_live/index.ex\ndefmodule YourAppWeb.OrderReportLive.Index do\n  use YourAppWeb, :live_view\n  use LiveTable.LiveResource\n\n  def mount(_params, _session, socket) do\n    # Assign your custom data provider function\n    socket = assign(socket, :data_provider, {YourApp.Orders, :list_with_products, []})\n    {:ok, socket}\n  end\n\n  def fields do\n    [\n      order_id: %{label: \"Order #\", sortable: true},\n      customer_email: %{label: \"Customer\", sortable: true, searchable: true},\n      total_amount: %{label: \"Total\", sortable: true},\n      # Reference the alias used in your custom query\n      product_name: %{\n        label: \"Product\", \n        sortable: true, \n        searchable: true,\n        assoc: {:order_items, :product_name}\n      },\n      order_date: %{label: \"Date\", sortable: true}\n    ]\n  end\n\n  def filters do\n    [\n      status: Select.new({:orders, :status}, \"status\", %{\n        label: \"Order Status\",\n        options: [\n          %{label: \"Pending\", value: [\"pending\"]},\n          %{label: \"Shipped\", value: [\"shipped\"]},\n          %{label: \"Delivered\", value: [\"delivered\"]}\n        ]\n      }),\n      \n      amount_range: Range.new(:total_amount, \"amount_range\", %{\n        type: :number,\n        label: \"Order Amount\",\n        min: 0,\n        max: 5000\n      })\n    ]\n  end\nend\n```\n\nAnd the corresponding context function:\n\n```elixir\n# lib/your_app/orders.ex\ndefmodule YourApp.Orders do\n  import Ecto.Query\n  alias YourApp.Repo\n\n  def list_with_products do\n    from o in YourApp.Order,\n      join: c in YourApp.Customer, on: o.customer_id == c.id,\n      join: oi in YourApp.OrderItem, on: oi.order_id == o.id, as: :order_items,\n      join: p in YourApp.Product, on: oi.product_id == p.id,\n      select: %{\n        order_id: o.id,\n        customer_email: c.email,\n        total_amount: o.total_amount,\n        product_name: p.name,  # This field key must match your fields definition\n        order_date: o.inserted_at\n      }\n  end\nend\n```","ref":"quick-start.html#advanced-example-custom-query-with-joins"},{"type":"extras","title":"What's Next? - Quick Start Guide","doc":"Congratulations! You now have a fully functional LiveTable. Here are some next steps:","ref":"quick-start.html#what-s-next"},{"type":"extras","title":"Customize the Appearance - Quick Start Guide","doc":"```elixir\ndef table_options do\n  %{\n    pagination: %{\n      sizes: [5, 10, 25, 50]\n    },\n    sorting: %{\n      default_sort: [name: :asc]  # Only works for single table queries - field must exist in schema\n    }\n  }\nend\n```\n\n**Note:** Default sort only works with simple schema fields for single table queries. For custom queries with joins, LiveTable doesn't currently support default sorting on joined fields.","ref":"quick-start.html#customize-the-appearance"},{"type":"extras","title":"Add Custom Actions - Quick Start Guide","doc":"```elixir\n# In your fields definition\nactions: %{\n  label: \"Actions\",\n  sortable: false,\n  renderer: &render_actions/1\n}\n\ndefp render_actions(product) do\n  assigns = %{product: product}\n  ~H\"\"\"\n   \n    <.link \n      navigate={~p\"/products/#{@product.id}\"} \n      class=\"text-blue-600 hover:text-blue-800 text-sm\"\n    >\n      Edit\n    </.link>\n     \n      Delete\n     \n   \n  \"\"\"\nend\n```","ref":"quick-start.html#add-custom-actions"},{"type":"extras","title":"Enable Card View - Quick Start Guide","doc":"```elixir\ndef table_options do\n  %{\n    mode: :card,\n    card_component: &product_card/1\n  }\nend\n\ndefp product_card(assigns) do\n  ~H\"\"\"\n   \n     <%= @record.name %> \n     <%= @record.description %> \n     \n       \n        $<%= @record.price %>\n       \n       \n        Stock: <%= @record.stock_quantity %>\n       \n     \n   \n  \"\"\"\nend\n```","ref":"quick-start.html#enable-card-view"},{"type":"extras","title":"Troubleshooting - Quick Start Guide","doc":"**Table not showing data?**\n- Check that your schema is correct\n- Verify you have data in your database\n- Ensure the LiveView is properly mounted\n\n**Filters not working?**\n- Verify filter field names match your schema\n- Check dynamic query syntax in filter conditions\n- Ensure Boolean/Range/Select are properly imported\n\n**Sorting not working on custom queries?**\n- Make sure field keys in `fields/0` match the select keys in your query\n- For joined fields, use `assoc: {:alias_name, :field}` where alias_name matches your query alias\n\n**Styling looks wrong?**\n- Ensure Tailwind CSS is properly configured\n- Check that LiveTable CSS is imported\n- Verify Tailwind is processing LiveTable classes","ref":"quick-start.html#troubleshooting"},{"type":"extras","title":"Learn More - Quick Start Guide","doc":"- [Field Configuration](api/fields.md) - Learn about all field options\n- [Filter Types](api/filters.md) - Explore Boolean, Range, and Select filters\n- [Table Configuration](api/table-options.md) - Customize table behavior\n- [Advanced Examples](examples/simple-table.md) - See more complex use cases","ref":"quick-start.html#learn-more"},{"type":"extras","title":"Configuration","doc":"# Configuration\n\nLiveTable provides flexible configuration options that can be set at three different levels, with each level taking precedence over the previous one.","ref":"configuration.html"},{"type":"extras","title":"Configuration Hierarchy - Configuration","doc":"1. **LiveTable Defaults** - Built-in sensible defaults\n2. **Application-wide Configuration** - Set in your `config.exs`\n3. **Per-table Configuration** - Override specific tables via `table_options/0`","ref":"configuration.html#configuration-hierarchy"},{"type":"extras","title":"Application-wide Configuration - Configuration","doc":"Configure LiveTable globally in your `config/config.exs`:\n\n```elixir\nconfig :live_table,\n  repo: YourApp.Repo,\n  pubsub: YourApp.PubSub,\n  components: YourApp.CustomComponents,  # Optional\n  defaults: %{\n    pagination: %{\n      enabled: true,\n      sizes: [10, 25, 50]\n    },\n    sorting: %{\n      enabled: true,\n      default_sort: [id: :asc]\n    },\n    exports: %{\n      enabled: true,\n      formats: [:csv, :pdf]\n    },\n    search: %{\n      enabled: true,\n      debounce: 300,\n      placeholder: \"Search...\"\n    },\n    mode: :table,\n    use_streams: true\n  }\n```","ref":"configuration.html#application-wide-configuration"},{"type":"extras","title":"Required Configuration - Configuration","doc":"- **`repo`** - Your application's Ecto repository\n- **`pubsub`** - Your application's PubSub module for real-time updates","ref":"configuration.html#required-configuration"},{"type":"extras","title":"Optional Configuration - Configuration","doc":"- **`components`** - Custom component module (defaults to built-in components)\n- **`defaults`** - Override default table options","ref":"configuration.html#optional-configuration"},{"type":"extras","title":"Per-table Configuration - Configuration","doc":"Override settings for specific tables by implementing `table_options/0`:\n\n```elixir\ndefmodule YourAppWeb.ProductLive.Index do\n  use LiveTable.LiveResource, schema: Product\n  \n  def table_options do\n    %{\n      pagination: %{\n        enabled: true,\n        sizes: [5, 15, 30]\n      },\n      sorting: %{\n        default_sort: [name: :asc]\n      },\n      exports: %{\n        enabled: false  # Disable exports for this table\n      },\n      mode: :table\n    }\n  end\nend\n```","ref":"configuration.html#per-table-configuration"},{"type":"extras","title":"Configuration Options - Configuration","doc":"","ref":"configuration.html#configuration-options"},{"type":"extras","title":"Pagination Options - Configuration","doc":"Control how pagination behaves in your tables:\n\n```elixir\npagination: %{\n  enabled: true,           # Enable/disable pagination\n  sizes: [10, 25, 50]     # Available page size options\n}\n```\n\n**Options:**\n- `enabled` (boolean) - Enable or disable pagination entirely\n- `sizes` (list) - Available page size options\n\n**Examples:**\n\n```elixir\n# Disable pagination (show all records)\npagination: %{enabled: false}\n\n# Custom page sizes\npagination: %{\n  enabled: true,\n  sizes: [5, 15, 30]\n}\n```","ref":"configuration.html#pagination-options"},{"type":"extras","title":"Sorting Options - Configuration","doc":"Configure sorting behavior and defaults:\n\n```elixir\nsorting: %{\n  enabled: true,                    # Enable/disable sorting\n  default_sort: [id: :asc]         # Default sort order\n}\n```\n\n**Options:**\n- `enabled` (boolean) - Enable or disable sorting\n- `default_sort` (keyword list) - Default sort columns and directions\n\n**Examples:**\n\n```elixir\n# Default sort order\nsorting: %{\n  default_sort: [name: :asc]\n}\n\n# Disable sorting entirely\nsorting: %{\n  enabled: false\n}\n```","ref":"configuration.html#sorting-options"},{"type":"extras","title":"Export Options - Configuration","doc":"Control export functionality:\n\n```elixir\nexports: %{\n  enabled: true,                        # Enable/disable exports\n  formats: [:csv, :pdf]                # Available export formats\n}\n```\n\n**Options:**\n- `enabled` (boolean) - Enable or disable exports\n- `formats` (list) - Available formats (`:csv`, `:pdf`, or both)\n\n**Examples:**\n\n```elixir\n# CSV only\nexports: %{\n  enabled: true,\n  formats: [:csv]\n}\n\n# Disable exports\nexports: %{\n  enabled: false\n}\n```","ref":"configuration.html#export-options"},{"type":"extras","title":"Search Options - Configuration","doc":"Configure the global search behavior:\n\n```elixir\nsearch: %{\n  enabled: true,                   # Enable/disable search\n  debounce: 300,                  # Debounce time in milliseconds\n  placeholder: \"Search...\"        # Search input placeholder text\n}\n```\n\n**Options:**\n- `enabled` (boolean) - Enable or disable global search\n- `debounce` (integer) - Delay before search executes (milliseconds)\n- `placeholder` (string) - Search input placeholder text","ref":"configuration.html#search-options"},{"type":"extras","title":"View Mode Options - Configuration","doc":"Configure table display modes:\n\n```elixir\nmode: :table  # or :card\n```\n\n**Table Mode:**\n```elixir\nmode: :table\n```\n\n**Card Mode:**\n```elixir\nmode: :card,\ncard_component: &product_card/1\n```","ref":"configuration.html#view-mode-options"},{"type":"extras","title":"Streams vs Assigns - Configuration","doc":"LiveTable supports two data handling modes controlled by `use_streams`:\n\n```elixir\n# Default: Use Phoenix LiveView streams (recommended)\nuse_streams: true   # Default\n\n# Alternative: Use traditional assigns\nuse_streams: false\n```\n\n**When `use_streams: true` (default):**\n- Uses Phoenix LiveView streams for efficient DOM updates\n- Pass `streams={@streams}` to the live_table component\n- Better performance with large datasets and real-time updates\n\n**When `use_streams: false`:**\n- Uses traditional assigns for data\n- Pass `streams={@resources}` to the live_table component\n- Simpler for small datasets or when streams aren't needed\n\n**Template usage:**\n```elixir\n# With streams (default)\n<.live_table\n  fields={fields()}\n  filters={filters()}\n  options={@options}\n  streams={@streams}\n/>\n\n# With assigns\n<.live_table\n  fields={fields()}\n  filters={filters()}\n  options={@options}\n  streams={@resources}\n/>\n```","ref":"configuration.html#streams-vs-assigns"},{"type":"extras","title":"Custom Components - Configuration","doc":"LiveTable allows complete UI customization through custom components. You can replace any section of the table interface with your own implementation while still benefiting from LiveTable's data handling, filtering, and state management.\n\n```elixir\n# Override entire sections\ndef table_options do\n  %{\n    custom_header: {MyApp.CustomComponents, :header},\n    custom_content: {MyApp.CustomComponents, :content}, \n    custom_footer: {MyApp.CustomComponents, :footer}\n  }\nend\n```\n\n**Available customizations:**\n- `custom_header` - Replace the entire header section (search, filters, controls)\n- `custom_content` - Replace the table/card content area\n- `custom_footer` - Replace the footer section (pagination, exports)","ref":"configuration.html#custom-components"},{"type":"extras","title":"Real-World Example: College Counselling Interface - Configuration","doc":"This example shows a sophisticated custom header that integrates transformers with a beautiful UI:\n\n```elixir\n# table_options with custom header\ndef table_options do\n  %{\n    mode: :card,\n    card_component: &CounsellingWeb.CollegeComponent.college_component/1,\n    custom_header: {CounsellingWeb.CollegeLive.CustomHeader, :custom_header}\n  }\nend\n\n# Custom header component\ndefmodule CounsellingWeb.CollegeLive.CustomHeader do\n  use Phoenix.Component\n\n  def custom_header(assigns) do\n    ~H\"\"\"\n     \n       \n         \n          <!-- Hero section with transformer input -->\n           \n             \n                \n               \n                Find Colleges for Your Rank\n               \n             \n            \n             \n              <!-- Transformer input for rank filtering -->\n               \n                <.form for={%{}} phx-debounce={get_in(@table_options, [:search, :debounce])} phx-change=\"sort\">\n                   \n                    Your JEE Rank  (Saved automatically) \n                   \n                   \n                </.form>\n               \n              \n              <!-- Category selector -->\n               \n                 \n                  Category\n                 \n                 \n                   General \n                   OBC-NCL \n                   SC \n                   ST \n                   EWS \n                 \n               \n             \n           \n\n          <!-- Advanced filters with transformers -->\n           \n            <!-- Search -->\n             \n              <.form for={%{}} phx-change=\"sort\">\n                 \n                  Search Colleges\n                 \n                 \n                   \n                   \n                      \n                   \n                 \n              </.form>\n             \n\n            <!-- NIRF Ranking transformer -->\n             \n              <.form for={%{}} phx-change=\"sort\">\n                 \n                  NIRF Ranking\n                 \n                 \n                   All Rankings \n                   Top 10 \n                   Top 25 \n                 \n              </.form>\n             \n\n            <!-- Sort transformer -->\n             \n              <.form for={%{}} phx-change=\"sort\">\n                 \n                  Sort By\n                 \n                 \n                   NIRF Ranking \n                   Name (A-Z) \n                   Name (Z-A) \n                 \n              </.form>\n             \n           \n\n          <!-- Boolean filters for institution types -->\n           \n            <.form for={%{}} phx-change=\"sort\">\n               \n                Institution Type\n               \n               \n                <.input\n                  :for={{id, %Boolean{field: :class, options: %{label: label}}} <- @filters}\n                  type=\"checkbox\"\n                  name={\"filters[#{id}]\"}\n                  label={label}\n                  checked={Map.has_key?(@options[\"filters\"], id)}\n                />\n               \n            </.form>\n           \n\n          <!-- Clear filters -->\n           \n            <.link\n              :if={@options[\"filters\"] != %{\"search\" => \"\"}}\n              phx-click=\"sort\"\n              phx-value-clear_filters=\"true\"\n              class=\"text-sm text-gray-500 hover:text-gray-700 transition-colors\"\n            >\n              Clear All Filters\n            </.link>\n           \n         \n       \n     \n    \"\"\"\n  end\nend\n```","ref":"configuration.html#real-world-example-college-counselling-interface"},{"type":"extras","title":"Complete UI Control - Configuration","doc":"With custom components, you have complete control over:\n\n- **Layout and styling** - Use any CSS framework or design system\n- **User interactions** - Implement custom form controls and behaviors\n- **Data presentation** - Create sophisticated filtering interfaces\n- **Integration** - Combine LiveTable data with your own UI components\n- **Responsive design** - Build mobile-optimized interfaces\n\n**Key Benefits:**\n- Keep LiveTable's powerful data handling and state management\n- Replace only the UI parts you want to customize\n- Maintain URL persistence and real-time updates\n- Integrate seamlessly with your existing design system\n\nThe college counselling example shows how you can create a completely custom interface that still benefits from LiveTable's transformer system, URL persistence, and efficient data handling.","ref":"configuration.html#complete-ui-control"},{"type":"extras","title":"Environment-specific Configuration - Configuration","doc":"Configure different settings per environment:\n\n```elixir\n# config/dev.exs\nconfig :live_table,\n  defaults: %{\n    pagination: %{sizes: [5, 10]},  # Smaller pages in development\n    exports: %{enabled: false}       # Disable exports in dev\n  }\n\n# config/prod.exs  \nconfig :live_table,\n  defaults: %{\n    pagination: %{sizes: [25, 50, 100]},\n    exports: %{enabled: true}\n  }\n```","ref":"configuration.html#environment-specific-configuration"},{"type":"extras","title":"Oban Configuration - Configuration","doc":"Configure background job processing for exports:\n\n```elixir\n# config/config.exs\nconfig :your_app, Oban,\n  repo: YourApp.Repo,\n  engine: Oban.Engines.Basic,\n  notifier: Oban.Notifiers.Postgres,\n  plugins: [\n    Oban.Plugins.Pruner,\n    {Oban.Plugins.Cron, \n     crontab: [\n       # Clean up old export files daily at 2 AM\n       {\"0 2 * * *\", YourApp.Workers.CleanupExports}\n     ]}\n  ],\n  queues: [\n    exports: 10,        # Export processing queue\n    default: 5          # Other background jobs\n  ]\n```","ref":"configuration.html#oban-configuration"},{"type":"extras","title":"Oban Web (Optional) - Configuration","doc":"Monitor export jobs with Oban Web:\n\n```elixir\n# lib/your_app_web/router.ex\nimport Oban.Web.Router\n\nscope \"/admin\" do\n  pipe_through [:browser, :admin_required]\n  oban_dashboard \"/oban\"\nend\n```","ref":"configuration.html#oban-web-optional"},{"type":"extras","title":"Custom Components - Configuration","doc":"Replace LiveTable's default components with your own:\n\n```elixir\n# config/config.exs\nconfig :live_table,\n  components: YourApp.CustomComponents\n```\n\nCreate your custom components module:\n\n```elixir\n# lib/your_app/custom_components.ex\ndefmodule YourApp.CustomComponents do\n  use Phoenix.Component\n  \n  # Override the default input component\n  def input(assigns) do\n    ~H\"\"\"\n     \n       \n       \n         \n            \n           \n              \n           \n         \n         <%= @label %> \n       \n     \n    \"\"\"\n  end\n  \n  # Override the default icon component\n  def icon(%{name: \"sort-asc\"} = assigns) do\n    ~H\"\"\"\n     \n        \n     \n    \"\"\"\n  end\nend\n```","ref":"configuration.html#custom-components"},{"type":"extras","title":"Complete Example - Configuration","doc":"Here's a comprehensive configuration example:\n\n```elixir\n# config/config.exs\nconfig :live_table,\n  repo: YourApp.Repo,\n  pubsub: YourApp.PubSub,\n  defaults: %{\n    pagination: %{\n      enabled: true,\n      sizes: [10, 25, 50]\n    },\n    sorting: %{\n      enabled: true,\n      default_sort: [id: :asc]\n    },\n    exports: %{\n      enabled: true,\n      formats: [:csv, :pdf]\n    },\n    search: %{\n      enabled: true,\n      debounce: 300,\n      placeholder: \"Search...\"\n    },\n    mode: :table,\n    use_streams: true\n  }\n\n# Per-table override\ndefmodule YourAppWeb.ProductLive.Index do\n  def table_options do\n    %{\n      pagination: %{sizes: [5, 15, 30]},\n      sorting: %{default_sort: [name: :asc]},\n      exports: %{formats: [:csv]},\n      mode: :card,\n      card_component: &product_card/1\n    }\n  end\nend\n```","ref":"configuration.html#complete-example"},{"type":"extras","title":"Complete Configuration Reference - Configuration","doc":"Based on the actual table configuration, here are all available options:\n\n```elixir\n# All available configuration options\nconfig :live_table,\n  defaults: %{\n    pagination: %{\n      enabled: true,\n      sizes: [10, 25, 50]\n    },\n    sorting: %{\n      enabled: true,\n      default_sort: [id: :asc]\n    },\n    exports: %{\n      enabled: true,\n      formats: [:csv, :pdf]\n    },\n    search: %{\n      enabled: true,\n      debounce: 300,\n      placeholder: \"Search...\"\n    },\n    mode: :table,\n    use_streams: true\n  }\n```","ref":"configuration.html#complete-configuration-reference"},{"type":"extras","title":"Fields API Reference","doc":"# Fields API Reference\n\nFields define the columns displayed in your LiveTable. They control what data is shown, how it's formatted, and whether columns can be sorted or searched.","ref":"fields.html"},{"type":"extras","title":"Overview - Fields API Reference","doc":"Fields are defined in the `fields/0` function as a keyword list where each key represents a column and maps to configuration options.\n\n```elixir\ndef fields do\n  [\n    id: %{label: \"ID\", sortable: true},\n    name: %{label: \"Product Name\", sortable: true, searchable: true},\n    price: %{label: \"Price\", sortable: true, renderer: &format_price/1}\n  ]\nend\n```","ref":"fields.html#overview"},{"type":"extras","title":"Field Configuration Options - Fields API Reference","doc":"","ref":"fields.html#field-configuration-options"},{"type":"extras","title":"Required Options - Fields API Reference","doc":"#### `label` (string)\nThe display name for the column header and exports.\n\n```elixir\nname: %{label: \"Product Name\"}\n```\n\n#### `sortable` (boolean)\nWhether the column can be sorted. Adds clickable sort controls to the header.\n\n```elixir\nprice: %{label: \"Price\", sortable: true}\n```\n\n#### `searchable` (boolean)\nWhether the column is included in global text search using ILIKE matching.\n\n```elixir\nname: %{label: \"Name\", searchable: true}\n```","ref":"fields.html#required-options"},{"type":"extras","title":"Optional Options - Fields API Reference","doc":"#### `renderer` (function)\nCustom function component for formatting cell display. You can use either:\n\n- **`function/1`** - Receives only the cell value\n- **`function/2`** - Receives the cell value and the entire record\n\n```elixir\n# Using function/1 - only gets the cell value\nstatus: %{\n  label: \"Status\", \n  renderer: &format_status/1\n}\n\ndefp format_status(status) do\n  assigns = %{status: status}\n  ~H\"\"\"\n   \n    <%= String.capitalize(@status) %>\n   \n  \"\"\"\nend\n\n# Using function/2 - gets cell value AND entire record\npriority: %{\n  label: \"Priority\",\n  renderer: &format_priority_with_context/2\n}\n\ndefp format_priority_with_context(priority, record) do\n  assigns = %{priority: priority, record: record}\n  ~H\"\"\"\n   \n     \n      <%= String.upcase(@priority) %>\n     \n    <%= if @record.is_urgent do %>\n       URGENT \n    <% end %>\n   \n  \"\"\"\nend\n```\n\n#### `computed` (dynamic query)\nDefine calculated fields using Ecto dynamic expressions.\n\n```elixir\ntotal_value: %{\n  label: \"Total Value\",\n  sortable: true,\n  computed: dynamic([r], fragment(\"? * ?\", r.price, r.quantity))\n}\n```\n\n#### `assoc` (tuple) - For Custom Queries Only\nWhen using custom queries with joins, specify the table alias used in your query.\n\n```elixir\n# Only needed when you have a custom query with joins\ncustomer_name: %{\n  label: \"Customer\",\n  sortable: true,\n  searchable: true,\n  assoc: {:customers, :name}  # :customers must match your query alias\n}\n```","ref":"fields.html#optional-options"},{"type":"extras","title":"Usage Patterns - Fields API Reference","doc":"","ref":"fields.html#usage-patterns"},{"type":"extras","title":"Simple Tables (Single Schema) - Fields API Reference","doc":"For basic tables querying a single schema, just reference the schema fields directly:\n\n```elixir\ndefmodule YourAppWeb.ProductLive.Index do\n  use YourAppWeb, :live_view\n  use LiveTable.LiveResource, schema: YourApp.Product\n\n  def fields do\n    [\n      id: %{label: \"ID\", sortable: true},\n      name: %{label: \"Product Name\", sortable: true, searchable: true},\n      price: %{label: \"Price\", sortable: true},\n      stock_quantity: %{label: \"Stock\", sortable: true},\n      active: %{label: \"Active\", sortable: true, renderer: &render_boolean/1}\n    ]\n  end\nend\n```","ref":"fields.html#simple-tables-single-schema"},{"type":"extras","title":"Custom Queries with Joins - Fields API Reference","doc":"For complex scenarios with joins, provide a custom data provider and reference aliases:\n\n```elixir\ndefmodule YourAppWeb.OrderReportLive.Index do\n  use YourAppWeb, :live_view\n  use LiveTable.LiveResource\n\n  def mount(_params, _session, socket) do\n    socket = assign(socket, :data_provider, {YourApp.Orders, :list_with_details, []})\n    {:ok, socket}\n  end\n\n  def fields do\n    [\n      order_id: %{label: \"Order #\", sortable: true},\n      customer_email: %{label: \"Customer\", sortable: true, searchable: true},\n      total_amount: %{label: \"Total\", sortable: true},\n      # Reference the alias from your custom query\n      product_name: %{\n        label: \"Product\", \n        sortable: true, \n        searchable: true,\n        assoc: {:order_items, :product_name}\n      }\n    ]\n  end\nend\n```\n\nThe corresponding context function must use matching aliases:\n\n```elixir\ndefmodule YourApp.Orders do\n  def list_with_details do\n    from o in Order,\n      join: c in Customer, on: o.customer_id == c.id,\n      join: oi in OrderItem, on: oi.order_id == o.id, as: :order_items,\n      join: p in Product, on: oi.product_id == p.id,\n      select: %{\n        order_id: o.id,\n        customer_email: c.email,\n        total_amount: o.total_amount,\n        product_name: p.name  # Field key must match\n      }\n  end\nend\n```","ref":"fields.html#custom-queries-with-joins"},{"type":"extras","title":"Computed Fields - Fields API Reference","doc":"Create calculated fields using database functions:\n\n```elixir\ndef fields do\n  [\n    # Simple calculation\n    total_value: %{\n      label: \"Total Value\",\n      sortable: true,\n      computed: dynamic([r], fragment(\"? * ?\", r.price, r.stock_quantity))\n    },\n    \n    # Conditional logic\n    stock_status: %{\n      label: \"Stock Status\",\n      sortable: true,\n      computed: dynamic([r], \n        fragment(\"CASE WHEN ? > 50 THEN 'High' WHEN ? > 10 THEN 'Medium' ELSE 'Low' END\", \n                 r.stock_quantity, r.stock_quantity)\n      )\n    },\n    \n    # Using joined tables (for custom queries)\n    category_product_count: %{\n      label: \"Products in Category\",\n      sortable: true,\n      assoc: {:categories, :name},  # Must match your query alias\n      computed: dynamic([r, categories: c], \n        fragment(\"(SELECT COUNT(*) FROM products WHERE category_id = ?)\", c.id)\n      )\n    }\n  ]\nend\n```","ref":"fields.html#computed-fields"},{"type":"extras","title":"Custom Renderers - Fields API Reference","doc":"Transform how data appears in your table cells:","ref":"fields.html#custom-renderers"},{"type":"extras","title":"Simple Formatting (function/1) - Fields API Reference","doc":"```elixir\ndef fields do\n  [\n    price: %{\n      label: \"Price\",\n      sortable: true,\n      renderer: &format_currency/1\n    },\n    created_at: %{\n      label: \"Created\",\n      sortable: true,\n      renderer: &format_date/1\n    }\n  ]\nend\n\ndefp format_currency(amount) do\n  assigns = %{amount: amount}\n  ~H\"\"\"\n   \n    $<%= :erlang.float_to_binary(@amount, decimals: 2) %>\n   \n  \"\"\"\nend\n\ndefp format_date(datetime) do\n  assigns = %{datetime: datetime}\n  ~H\"\"\"\n   \n    <%= Calendar.strftime(@datetime, \"%b %d, %Y\") %>\n   \n  \"\"\"\nend\n```","ref":"fields.html#simple-formatting-function-1"},{"type":"extras","title":"Status Indicators - Fields API Reference","doc":"```elixir\ndef fields do\n  [\n    status: %{\n      label: \"Order Status\",\n      renderer: &render_order_status/1\n    },\n    priority: %{\n      label: \"Priority\",\n      renderer: &render_priority_badge/1  \n    }\n  ]\nend\n\ndefp render_order_status(status) do\n  assigns = %{status: status}\n  ~H\"\"\"\n   \n      \"bg-yellow-400\"\n        \"processing\" -> \"bg-blue-400\" \n        \"shipped\" -> \"bg-green-400\"\n        \"delivered\" -> \"bg-green-600\"\n        \"cancelled\" -> \"bg-red-400\"\n      end\n    ]}> \n     <%= @status %> \n   \n  \"\"\"\nend\n\ndefp render_priority_badge(priority) do\n  assigns = %{priority: priority}\n  ~H\"\"\"\n    \"bg-red-100 text-red-700\"\n      \"medium\" -> \"bg-yellow-100 text-yellow-700\"  \n      \"low\" -> \"bg-green-100 text-green-700\"\n    end\n  ]}>\n    <%= String.upcase(@priority) %>\n   \n  \"\"\"\nend\n```","ref":"fields.html#status-indicators"},{"type":"extras","title":"Interactive Elements (function/2) - Fields API Reference","doc":"```elixir\ndef fields do\n  [\n    actions: %{\n      label: \"Actions\",\n      sortable: false,\n      renderer: &render_actions/2  # function/2 to access full record\n    }\n  ]\nend\n\ndefp render_actions(_value, record) do\n  assigns = %{record: record}\n  ~H\"\"\"\n   \n    <.link \n      navigate={~p\"/products/#{@record.id}\"} \n      class=\"text-blue-600 hover:text-blue-800 text-sm font-medium\"\n    >\n      Edit\n    </.link>\n     \n      <%= if @record.active, do: \"Deactivate\", else: \"Activate\" %>\n     \n     \n      Delete\n     \n   \n  \"\"\"\nend\n```","ref":"fields.html#interactive-elements-function-2"},{"type":"extras","title":"Conditional Rendering with Context (function/2) - Fields API Reference","doc":"```elixir\ndef fields do\n  [\n    stock_status: %{\n      label: \"Stock\",\n      sortable: true,\n      renderer: &render_stock_with_alerts/2\n    }\n  ]\nend\n\ndefp render_stock_with_alerts(stock_quantity, record) do\n  assigns = %{stock: stock_quantity, record: record}\n  ~H\"\"\"\n   \n      50 -> \"text-green-600\"\n        @stock > 10 -> \"text-yellow-600\"\n        @stock > 0 -> \"text-orange-600\"\n        true -> \"text-red-600\"\n      end\n    ]}>\n      <%= @stock %> in stock\n     \n    \n    <%= if @record.reorder_point && @stock <= @record.reorder_point do %>\n       \n        Reorder needed\n       \n    <% end %>\n    \n    <%= if @record.category == \"perishable\" && @stock > 0 do %>\n       \n        Expires: <%= @record.expiry_date %>\n       \n    <% end %>\n   \n  \"\"\"\nend\n```","ref":"fields.html#conditional-rendering-with-context-function-2"},{"type":"extras","title":"Common Patterns - Fields API Reference","doc":"","ref":"fields.html#common-patterns"},{"type":"extras","title":"E-commerce Product Table - Fields API Reference","doc":"```elixir\ndef fields do\n  [\n    image: %{\n      label: \"Image\",\n      sortable: false,\n      renderer: &render_product_image/1\n    },\n    name: %{\n      label: \"Product\",\n      sortable: true,\n      searchable: true\n    },\n    sku: %{\n      label: \"SKU\",\n      sortable: true,\n      searchable: true\n    },\n    price: %{\n      label: \"Price\",\n      sortable: true,\n      renderer: &format_currency/1\n    },\n    stock: %{\n      label: \"Stock\",\n      sortable: true,\n      renderer: &render_stock_status/1\n    },\n    status: %{\n      label: \"Status\",\n      sortable: true,\n      renderer: &render_product_status/1\n    }\n  ]\nend\n\ndefp render_product_image(image_url) do\n  assigns = %{image_url: image_url}\n  ~H\"\"\"\n   \n  \"\"\"\nend\n\ndefp render_stock_status(quantity) do\n  assigns = %{quantity: quantity}\n  ~H\"\"\"\n    50 -> \"text-green-600\"\n      @quantity > 10 -> \"text-yellow-600\"\n      @quantity > 0 -> \"text-orange-600\"\n      true -> \"text-red-600\"\n    end\n  ]}>\n    <%= @quantity %> in stock\n   \n  \"\"\"\nend\n```","ref":"fields.html#e-commerce-product-table"},{"type":"extras","title":"User Management Table - Fields API Reference","doc":"```elixir\ndef fields do\n  [\n    avatar: %{\n      label: \"\",\n      sortable: false,\n      renderer: &render_avatar/1\n    },\n    name: %{\n      label: \"Name\",\n      sortable: true,\n      searchable: true\n    },\n    email: %{\n      label: \"Email\",\n      sortable: true,\n      searchable: true\n    },\n    role: %{\n      label: \"Role\",\n      sortable: true,\n      renderer: &render_role_badge/1\n    },\n    last_sign_in: %{\n      label: \"Last Active\",\n      sortable: true,\n      renderer: &format_relative_time/1\n    },\n    active: %{\n      label: \"Status\",\n      sortable: true,\n      renderer: &render_user_status/1\n    }\n  ]\nend\n\ndefp render_avatar(user) do\n  assigns = %{user: user}\n  ~H\"\"\"\n   \n     \n   \n  \"\"\"\nend\n\ndefp render_role_badge(role) do\n  assigns = %{role: role}\n  ~H\"\"\"\n    \"bg-purple-100 text-purple-700\"\n      \"manager\" -> \"bg-blue-100 text-blue-700\"\n      \"user\" -> \"bg-gray-100 text-gray-700\"\n    end\n  ]}>\n    <%= String.capitalize(@role) %>\n   \n  \"\"\"\nend\n```","ref":"fields.html#user-management-table"},{"type":"extras","title":"Key Rules - Fields API Reference","doc":"","ref":"fields.html#key-rules"},{"type":"extras","title":"For Simple Tables (Single Schema) - Fields API Reference","doc":"1. Use the LiveResource with `schema: YourSchema`\n2. Field keys must match your schema attributes\n3. No `assoc:` needed - LiveTable handles everything","ref":"fields.html#for-simple-tables-single-schema"},{"type":"extras","title":"For Custom Queries - Fields API Reference","doc":"1. Use the LiveResource (no schema)\n2. Assign `:data_provider` in mount/handle_params\n3. Field keys must match your query's select keys\n4. Use `assoc: {:alias_name, :field}` only for sorting joined fields\n5. The alias_name must match your query's `as:` alias","ref":"fields.html#for-custom-queries"},{"type":"extras","title":"Troubleshooting - Fields API Reference","doc":"**Field not displaying?**\n- For simple tables: ensure field key matches schema attribute\n- For custom queries: ensure field key matches select key in your query\n\n**Sorting not working?**\n- Confirm `sortable: true` is set\n- For custom queries with joins: use `assoc: {:alias, :field}` where alias matches your query\n\n**Search not finding results?**\n- Verify `searchable: true` is set\n- Search only works on text/string fields\n- For custom queries: searchable fields must be in your select clause","ref":"fields.html#troubleshooting"},{"type":"extras","title":"Filters API Reference","doc":"# Filters API Reference\n\nFilters provide interactive filtering capabilities for your LiveTable. They allow users to narrow down displayed data using various input types like checkboxes, dropdowns, and range sliders.","ref":"filters.html"},{"type":"extras","title":"Overview - Filters API Reference","doc":"Filters are defined in the `filters/0` function as a keyword list where each key represents a filter and maps to a filter struct (Boolean, Range, or Select).\n\n```elixir\ndef filters do\n  [\n    active: Boolean.new(:active, \"active\", %{\n      label: \"Active Only\",\n      condition: dynamic([p], p.active == true)\n    }),\n    \n    price_range: Range.new(:price, \"price_range\", %{\n      type: :number,\n      label: \"Price Range\",\n      min: 0,\n      max: 1000\n    }),\n    \n    category: Select.new({:category, :name}, \"category\", %{\n      label: \"Category\",\n      options: [\n        %{label: \"Electronics\", value: [\"electronics\"]},\n        %{label: \"Books\", value: [\"books\"]}\n      ]\n    })\n  ]\nend\n```","ref":"filters.html#overview"},{"type":"extras","title":"Filter Types - Filters API Reference","doc":"LiveTable supports three types of filters:\n\n- **Boolean** - Checkbox filters for true/false conditions\n- **Range** - Slider filters for numeric, date, or datetime ranges\n- **Select** - Dropdown filters with static or dynamic options","ref":"filters.html#filter-types"},{"type":"extras","title":"Common Filter Patterns - Filters API Reference","doc":"","ref":"filters.html#common-filter-patterns"},{"type":"extras","title":"Filter Constructor Arguments - Filters API Reference","doc":"All filters use the same constructor pattern:\n\n```elixir\nFilterType.new(field, key, options)\n```\n\n- **`field`** - The database field to filter on (atom for simple fields, `{:table_alias, :field}` for joined fields)\n- **`key`** - URL parameter key (string, used for persistence and routing)\n- **`options`** - Map of filter-specific configuration options","ref":"filters.html#filter-constructor-arguments"},{"type":"extras","title":"Field References - Filters API Reference","doc":"#### Simple Schema Fields\nFor single-table queries, reference schema fields directly:\n\n```elixir\n# Filter on Product.active field\nactive_filter: Boolean.new(:active, \"active\", %{...})\n\n# Filter on Product.price field  \nprice_filter: Range.new(:price, \"price_range\", %{...})\n```\n\n#### Joined Fields (Custom Queries Only)\nFor custom queries with joins, reference the table alias:\n\n```elixir\n# Filter on joined suppliers table\nsupplier_filter: Select.new({:suppliers, :name}, \"supplier\", %{...})\n\n# Filter on joined categories table\ncategory_filter: Boolean.new({:categories, :active}, \"category_active\", %{...})\n```","ref":"filters.html#field-references"},{"type":"extras","title":"Boolean Filters - Filters API Reference","doc":"Boolean filters render as checkboxes and apply dynamic query conditions when checked.","ref":"filters.html#boolean-filters"},{"type":"extras","title":"Basic Usage - Filters API Reference","doc":"```elixir\ndef filters do\n  [\n    active: Boolean.new(:active, \"active\", %{\n      label: \"Active Products Only\",\n      condition: dynamic([p], p.active == true)\n    }),\n    \n    in_stock: Boolean.new(:stock_quantity, \"in_stock\", %{\n      label: \"In Stock\",\n      condition: dynamic([p], p.stock_quantity > 0)\n    })\n  ]\nend\n```","ref":"filters.html#basic-usage"},{"type":"extras","title":"Advanced Conditions - Filters API Reference","doc":"```elixir\ndef filters do\n  [\n    # Complex condition with multiple criteria\n    premium_products: Boolean.new(:price, \"premium\", %{\n      label: \"Premium Products (>$500 & Featured)\",\n      condition: dynamic([p], p.price > 500 and p.featured == true)\n    }),\n    \n    # Condition using joined tables (for custom queries)\n    verified_suppliers: Boolean.new({:suppliers, :verified}, \"verified\", %{\n      label: \"Verified Suppliers Only\",\n      condition: dynamic([p, suppliers: s], s.verified == true and s.active == true)\n    })\n  ]\nend\n```","ref":"filters.html#advanced-conditions"},{"type":"extras","title":"Boolean Options - Filters API Reference","doc":"```elixir\nBoolean.new(field, key, %{\n  label: \"Filter Label\",           # Required: Display text\n  condition: dynamic_expression,   # Required: Ecto dynamic query\n  default: false,                  # Optional: Default checked state\n  class: \"custom-css-class\"        # Optional: CSS classes\n})\n```","ref":"filters.html#boolean-options"},{"type":"extras","title":"Range Filters - Filters API Reference","doc":"Range filters provide sliders for filtering numeric values, dates, or datetimes within specified ranges.","ref":"filters.html#range-filters"},{"type":"extras","title":"Numeric Ranges - Filters API Reference","doc":"```elixir\ndef filters do\n  [\n    price_range: Range.new(:price, \"price_range\", %{\n      type: :number,\n      label: \"Price Range\",\n      unit: \"$\",\n      min: 0,\n      max: 1000,\n      step: 10,\n      default_min: 0,\n      default_max: 1000\n    }),\n    \n    age_range: Range.new(:age, \"age_range\", %{\n      type: :number,\n      label: \"Age\",\n      min: 18,\n      max: 65,\n      step: 1\n    })\n  ]\nend\n```","ref":"filters.html#numeric-ranges"},{"type":"extras","title":"Date Ranges - Filters API Reference","doc":"```elixir\ndef filters do\n  [\n    created_date: Range.new(:inserted_at, \"created_range\", %{\n      type: :date,\n      label: \"Created Date\",\n      min: ~D[2024-01-01],\n      max: ~D[2024-12-31],\n      default_min: ~D[2024-01-01],\n      default_max: ~D[2024-12-31]\n    })\n  ]\nend\n```","ref":"filters.html#date-ranges"},{"type":"extras","title":"DateTime Ranges - Filters API Reference","doc":"```elixir\ndef filters do\n  [\n    updated_time: Range.new(:updated_at, \"updated_range\", %{\n      type: :datetime,\n      label: \"Last Updated\",\n      min: ~N[2024-01-01 00:00:00],\n      max: ~N[2024-12-31 23:59:59],\n      step: 3600  # Step in seconds (1 hour)\n    })\n  ]\nend\n```","ref":"filters.html#datetime-ranges"},{"type":"extras","title":"Range Options - Filters API Reference","doc":"```elixir\nRange.new(field, key, %{\n  type: :number,                    # Required: :number, :date, or :datetime\n  label: \"Range Label\",             # Required: Display text\n  min: 0,                          # Required: Minimum value\n  max: 100,                        # Required: Maximum value\n  step: 1,                         # Optional: Step increment\n  default_min: 0,                  # Optional: Default minimum\n  default_max: 100,                # Optional: Default maximum\n  unit: \"$\",                       # Optional: Unit display\n  css_classes: \"custom-wrapper\",   # Optional: CSS classes\n  slider_classes: \"custom-slider\", # Optional: Slider CSS classes\n  pips: true,                      # Optional: Show slider markers\n  event_type: \"change\"             # Optional: JS event type\n})\n```","ref":"filters.html#range-options"},{"type":"extras","title":"Select Filters - Filters API Reference","doc":"Select filters provide dropdown selection with static or dynamic options.","ref":"filters.html#select-filters"},{"type":"extras","title":"Static Options - Filters API Reference","doc":"```elixir\ndef filters do\n  [\n    status: Select.new(:status, \"status\", %{\n      label: \"Order Status\",\n      options: [\n        %{label: \"Pending\", value: [\"pending\"]},\n        %{label: \"Processing\", value: [\"processing\"]},\n        %{label: \"Shipped\", value: [\"shipped\"]},\n        %{label: \"Delivered\", value: [\"delivered\"]}\n      ]\n    }),\n    \n    priority: Select.new(:priority, \"priority\", %{\n      label: \"Priority Level\",\n      options: [\n        %{label: \"Low\", value: [\"low\"]},\n        %{label: \"Medium\", value: [\"medium\"]},\n        %{label: \"High\", value: [\"high\"]},\n        %{label: \"Critical\", value: [\"critical\"]}\n      ]\n    })\n  ]\nend\n```","ref":"filters.html#static-options"},{"type":"extras","title":"Dynamic Options - Filters API Reference","doc":"For options loaded from the database:\n\n```elixir\ndef filters do\n  [\n    category: Select.new({:categories, :name}, \"category\", %{\n      label: \"Product Category\",\n      options_source: {YourApp.Catalog, :search_categories, []}\n    }),\n    \n    supplier: Select.new({:suppliers, :name}, \"supplier\", %{\n      label: \"Supplier\",\n      placeholder: \"Search suppliers...\",\n      options_source: {YourApp.Suppliers, :search_suppliers, []}\n    })\n  ]\nend\n```\n\nThe corresponding context functions must return tuples:\n\n```elixir\n# lib/your_app/catalog.ex\ndefmodule YourApp.Catalog do\n  def search_categories(search_text \\\\ \"\") do\n    Category\n    |> where([c], ilike(c.name, ^\"%#{search_text}%\"))\n    |> select([c], {c.name, [c.id, c.description]})\n    |> Repo.all()\n  end\nend\n\n# lib/your_app/suppliers.ex  \ndefmodule YourApp.Suppliers do\n  def search_suppliers(search_text \\\\ \"\") do\n    Supplier\n    |> where([s], ilike(s.name, ^\"%#{search_text}%\"))\n    |> select([s], {s.name, [s.id, s.contact_email]})\n    |> Repo.all()\n  end\nend\n```","ref":"filters.html#dynamic-options"},{"type":"extras","title":"Custom Option Templates - Filters API Reference","doc":"```elixir\ndef filters do\n  [\n    supplier: Select.new({:suppliers, :name}, \"supplier\", %{\n      label: \"Supplier\",\n      options_source: {YourApp.Suppliers, :search_suppliers, []},\n      option_template: &custom_supplier_template/1\n    })\n  ]\nend\n\ndefp custom_supplier_template(option) do\n  assigns = %{option: option}\n  ~H\"\"\"\n   \n     <%= @option.label %> \n     ID: <%= @option.value |> Enum.at(0) %> \n     <%= @option.value |> Enum.at(1) %> \n   \n  \"\"\"\nend\n```","ref":"filters.html#custom-option-templates"},{"type":"extras","title":"Select Options - Filters API Reference","doc":"```elixir\nSelect.new(field, key, %{\n  label: \"Select Label\",                    # Required: Display text\n  options: [],                             # Static options list\n  options_source: {Module, :function, []}, # Dynamic options source\n  option_template: &template_function/1,   # Custom option template\n  placeholder: \"Choose an option...\",      # Placeholder text\n  loading_text: \"Loading...\",              # Loading state text\n  css_classes: \"custom-wrapper\",           # CSS classes\n  label_classes: \"custom-label\",           # Label CSS classes\n  select_classes: \"custom-select\"          # Select input CSS classes\n})\n```","ref":"filters.html#select-options"},{"type":"extras","title":"Real-World Examples - Filters API Reference","doc":"","ref":"filters.html#real-world-examples"},{"type":"extras","title":"E-commerce Product Filters - Filters API Reference","doc":"```elixir\ndef filters do\n  [\n    # Active products toggle\n    active: Boolean.new(:active, \"active\", %{\n      label: \"Show Active Products Only\",\n      condition: dynamic([p], p.active == true)\n    }),\n    \n    # Price range slider\n    price_range: Range.new(:price, \"price_range\", %{\n      type: :number,\n      label: \"Price Range\",\n      unit: \"$\",\n      min: 0,\n      max: 2000,\n      step: 25,\n      default_min: 0,\n      default_max: 500\n    }),\n    \n    # Category dropdown with search\n    category: Select.new({:categories, :name}, \"category\", %{\n      label: \"Category\",\n      placeholder: \"Search categories...\",\n      options_source: {Shop.Catalog, :search_categories, []}\n    }),\n    \n    # Stock level toggle\n    low_stock: Boolean.new(:stock_quantity, \"low_stock\", %{\n      label: \"Low Stock Alert (< 10 items)\",\n      condition: dynamic([p], p.stock_quantity < 10 and p.stock_quantity > 0)\n    }),\n    \n    # Recently added toggle\n    new_arrivals: Boolean.new(:inserted_at, \"new_arrivals\", %{\n      label: \"New Arrivals (Last 30 Days)\",\n      condition: dynamic([p], p.inserted_at >= ago(30, \"day\"))\n    })\n  ]\nend\n```","ref":"filters.html#e-commerce-product-filters"},{"type":"extras","title":"User Management Filters - Filters API Reference","doc":"```elixir\ndef filters do\n  [\n    # Active users\n    active_users: Boolean.new(:active, \"active\", %{\n      label: \"Active Users Only\",\n      condition: dynamic([u], u.active == true)\n    }),\n    \n    # Registration date range\n    signup_date: Range.new(:inserted_at, \"signup_range\", %{\n      type: :date,\n      label: \"Registration Date\",\n      min: ~D[2020-01-01],\n      max: Date.utc_today()\n    }),\n    \n    # Role selection\n    role: Select.new(:role, \"role\", %{\n      label: \"User Role\",\n      options: [\n        %{label: \"Admin\", value: [\"admin\"]},\n        %{label: \"Manager\", value: [\"manager\"]},\n        %{label: \"User\", value: [\"user\"]},\n        %{label: \"Guest\", value: [\"guest\"]}\n      ]\n    }),\n    \n    # Email verified toggle\n    verified: Boolean.new(:email_verified_at, \"verified\", %{\n      label: \"Email Verified\",\n      condition: dynamic([u], not is_nil(u.email_verified_at))\n    })\n  ]\nend\n```","ref":"filters.html#user-management-filters"},{"type":"extras","title":"Order Analytics Filters - Filters API Reference","doc":"```elixir\ndef filters do\n  [\n    # Order status\n    status: Select.new(:status, \"status\", %{\n      label: \"Order Status\",\n      options: [\n        %{label: \"Pending\", value: [\"pending\"]},\n        %{label: \"Processing\", value: [\"processing\"]},\n        %{label: \"Shipped\", value: [\"shipped\"]},\n        %{label: \"Delivered\", value: [\"delivered\"]},\n        %{label: \"Cancelled\", value: [\"cancelled\"]}\n      ]\n    }),\n    \n    # Order value range\n    order_total: Range.new(:total_amount, \"total_range\", %{\n      type: :number,\n      label: \"Order Total\",\n      unit: \"$\",\n      min: 0,\n      max: 10000,\n      step: 100\n    }),\n    \n    # Date range for order placement\n    order_date: Range.new(:inserted_at, \"order_date_range\", %{\n      type: :date,\n      label: \"Order Date\",\n      min: Date.add(Date.utc_today(), -365),\n      max: Date.utc_today()\n    }),\n    \n    # High priority orders\n    priority_orders: Boolean.new(:priority, \"priority\", %{\n      label: \"Priority Orders Only\",\n      condition: dynamic([o], o.priority in [\"high\", \"urgent\"])\n    }),\n    \n    # Customer type (for custom queries with joins)\n    customer_type: Select.new({:customers, :type}, \"customer_type\", %{\n      label: \"Customer Type\",\n      options: [\n        %{label: \"Individual\", value: [\"individual\"]},\n        %{label: \"Business\", value: [\"business\"]},\n        %{label: \"Enterprise\", value: [\"enterprise\"]}\n      ]\n    })\n  ]\nend\n```","ref":"filters.html#order-analytics-filters"},{"type":"extras","title":"Filter Persistence - Filters API Reference","doc":"Filters automatically persist their state in URL parameters, allowing users to:\n\n- **Bookmark filtered views**\n- **Share filtered URLs**\n- **Navigate back/forward** with filter state intact\n- **Refresh the page** without losing filters\n\nThe `key` parameter in each filter constructor determines the URL parameter name:\n\n```elixir\n# Creates URL parameter: ?active=true\nactive: Boolean.new(:active, \"active\", %{...})\n\n# Creates URL parameter: ?price_range[min]=100&price_range[max]=500  \nprice_range: Range.new(:price, \"price_range\", %{...})\n\n# Creates URL parameter: ?category[]=electronics&category[]=books\ncategory: Select.new(:category, \"category\", %{...})\n```","ref":"filters.html#filter-persistence"},{"type":"extras","title":"Performance Tips - Filters API Reference","doc":"","ref":"filters.html#performance-tips"},{"type":"extras","title":"Optimize Filter Queries - Filters API Reference","doc":"```elixir\n# ✅ Good: Simple field conditions\ncondition: dynamic([p], p.active == true)\ncondition: dynamic([p], p.price > 100)\n\n# ⚠️ Be careful: Complex subqueries\ncondition: dynamic([p], fragment(\"(SELECT COUNT(*) FROM orders WHERE product_id = ?) > 5\", p.id))\n\n# ✅ Better: Use database views or computed fields for complex logic\n```","ref":"filters.html#optimize-filter-queries"},{"type":"extras","title":"Index Filtered Fields - Filters API Reference","doc":"Make sure frequently filtered fields have database indexes:\n\n```sql\n-- Add indexes for better filter performance\nCREATE INDEX idx_products_active ON products(active);\nCREATE INDEX idx_products_price ON products(price);\nCREATE INDEX idx_products_category_id ON products(category_id);\nCREATE INDEX idx_orders_status ON orders(status);\nCREATE INDEX idx_orders_inserted_at ON orders(inserted_at);\n```","ref":"filters.html#index-filtered-fields"},{"type":"extras","title":"Troubleshooting - Filters API Reference","doc":"","ref":"filters.html#troubleshooting"},{"type":"extras","title":"Filter Not Appearing - Filters API Reference","doc":"- Check that the filter is included in your `filters/0` function\n- Verify the filter syntax and required options\n- Ensure Boolean/Range/Select are properly imported","ref":"filters.html#filter-not-appearing"},{"type":"extras","title":"Filter Not Working - Filters API Reference","doc":"- For simple queries: ensure field exists in your schema\n- For custom queries: ensure field/alias exists in your query\n- Check dynamic query syntax in Boolean filter conditions\n- Verify options_source functions return correct tuple format for Select filters","ref":"filters.html#filter-not-working"},{"type":"extras","title":"Options Not Loading (Select Filters) - Filters API Reference","doc":"- Check that your context function is accessible\n- Verify the function returns `{label, [value1, value2]}` tuples\n- Ensure the module/function exists and is properly imported\n- Check for any errors in your context function","ref":"filters.html#options-not-loading-select-filters"},{"type":"extras","title":"URL Parameters Not Persisting - Filters API Reference","doc":"- Verify each filter has a unique `key` parameter\n- Check that keys don't conflict with other URL parameters\n- Ensure LiveView handle_params is properly set up","ref":"filters.html#url-parameters-not-persisting"},{"type":"extras","title":"Transformers API Reference","doc":"# Transformers API Reference\n\nTransformers are the most powerful feature of LiveTable, allowing you to implement custom filters that can modify the entire query and maintain state. Unlike simple filters that add WHERE conditions, transformers receive the full query and can perform any operations including joins, aggregations, subqueries, and complex transformations.","ref":"transformers.html"},{"type":"extras","title":"Overview - Transformers API Reference","doc":"Transformers provide complete control over query modification while maintaining filter state and URL persistence. They are perfect for complex filtering scenarios that go beyond simple field comparisons.\n\n```elixir\ndef filters do\n  [\n    # Regular filters\n    active: Boolean.new(:active, \"active\", %{\n      label: \"Active Only\",\n      condition: dynamic([p], p.active == true)\n    }),\n    \n    # Transformer - can modify entire query\n    sales_performance: Transformer.new(\"sales_performance\", %{\n      query_transformer: &apply_sales_filter/2\n    })\n  ]\nend\n\ndefp apply_sales_filter(query, filter_data) do\n  case filter_data do\n    %{\"period\" => period, \"min_sales\" => min_sales} ->\n      from p in query,\n        join: s in Sales, on: s.product_id == p.id,\n        where: s.period == ^period and s.total >= ^min_sales,\n        group_by: p.id\n        \n    _ -> \n      query\n  end\nend\n```","ref":"transformers.html#overview"},{"type":"extras","title":"Basic Usage - Transformers API Reference","doc":"","ref":"transformers.html#basic-usage"},{"type":"extras","title":"Creating a Transformer - Transformers API Reference","doc":"```elixir\n# Using a function reference\nsales_filter: Transformer.new(\"sales_filter\", %{\n  query_transformer: &transform_sales_query/2\n})\n\n# Using a module and function tuple\nanalytics_filter: Transformer.new(\"analytics\", %{\n  query_transformer: {MyApp.Analytics, :transform_query}\n})\n\n# Using an anonymous function\ncustom_filter: Transformer.new(\"custom\", %{\n  query_transformer: fn query, data ->\n    # Transform query based on data\n    query\n  end\n})\n```","ref":"transformers.html#creating-a-transformer"},{"type":"extras","title":"Transformer Function Signature - Transformers API Reference","doc":"Transformer functions receive two arguments:\n\n```elixir\ndef transform_query(query, filter_data) do\n  # query: The current Ecto query\n  # filter_data: Map of applied filter data from URL/form\n  \n  # Return modified query\n  query\nend\n```","ref":"transformers.html#transformer-function-signature"},{"type":"extras","title":"Real-World Examples - Transformers API Reference","doc":"","ref":"transformers.html#real-world-examples"},{"type":"extras","title":"Sales Performance Filter - Transformers API Reference","doc":"```elixir\ndef filters do\n  [\n    sales_performance: Transformer.new(\"sales_performance\", %{\n      query_transformer: &apply_sales_performance/2\n    })\n  ]\nend\n\ndefp apply_sales_performance(query, filter_data) do\n  case filter_data do\n    %{\"period\" => period, \"threshold\" => threshold} when period != \"\" and threshold != \"\" ->\n      {threshold, _} = Integer.parse(threshold)\n      \n      from p in query,\n        join: s in Sale, on: s.product_id == p.id,\n        join: pe in SalePeriod, on: pe.sale_id == s.id,\n        where: pe.period == ^period,\n        group_by: [p.id, p.name, p.price],\n        having: sum(s.amount) >= ^threshold,\n        select_merge: %{\n          total_sales: sum(s.amount),\n          sale_count: count(s.id)\n        }\n        \n    _ ->\n      query\n  end\nend\n```","ref":"transformers.html#sales-performance-filter"},{"type":"extras","title":"Geographic Region Filter - Transformers API Reference","doc":"```elixir\ndef filters do\n  [\n    region_analysis: Transformer.new(\"region_analysis\", %{\n      query_transformer: {MyApp.Geography, :filter_by_region}\n    })\n  ]\nend\n\n# In your context module\ndefmodule MyApp.Geography do\n  def filter_by_region(query, %{\"region\" => region, \"include_nearby\" => \"true\"}) do\n    from p in query,\n      join: l in Location, on: l.id == p.location_id,\n      join: r in Region, on: r.id == l.region_id,\n      left_join: nr in Region, on: nr.parent_id == r.id,\n      where: r.name == ^region or nr.name == ^region,\n      preload: [location: [region: :parent]]\n  end\n  \n  def filter_by_region(query, %{\"region\" => region}) do\n    from p in query,\n      join: l in Location, on: l.id == p.location_id,\n      join: r in Region, on: r.id == l.region_id,\n      where: r.name == ^region\n  end\n  \n  def filter_by_region(query, _), do: query\nend\n```","ref":"transformers.html#geographic-region-filter"},{"type":"extras","title":"Date Range with Aggregations - Transformers API Reference","doc":"```elixir\ndef filters do\n  [\n    date_metrics: Transformer.new(\"date_metrics\", %{\n      query_transformer: &apply_date_metrics/2\n    })\n  ]\nend\n\ndefp apply_date_metrics(query, filter_data) do\n  case filter_data do\n    %{\"start_date\" => start_date, \"end_date\" => end_date, \"metric\" => metric} \n    when start_date != \"\" and end_date != \"\" ->\n      \n      {:ok, start_date} = Date.from_iso8601(start_date)\n      {:ok, end_date} = Date.from_iso8601(end_date)\n      \n      case metric do\n        \"revenue\" ->\n          from p in query,\n            join: o in Order, on: o.product_id == p.id,\n            where: o.inserted_at >= ^start_date and o.inserted_at <= ^end_date,\n            group_by: [p.id, p.name],\n            select_merge: %{\n              period_revenue: sum(o.total),\n              order_count: count(o.id),\n              avg_order_value: avg(o.total)\n            }\n            \n        \"popularity\" ->\n          from p in query,\n            join: oi in OrderItem, on: oi.product_id == p.id,\n            join: o in Order, on: o.id == oi.order_id,\n            where: o.inserted_at >= ^start_date and o.inserted_at <= ^end_date,\n            group_by: [p.id, p.name],\n            order_by: [desc: count(oi.id)],\n            select_merge: %{\n              times_ordered: count(oi.id),\n              total_quantity: sum(oi.quantity)\n            }\n            \n        _ ->\n          query\n      end\n      \n    _ ->\n      query\n  end\nend\n```","ref":"transformers.html#date-range-with-aggregations"},{"type":"extras","title":"College Counselling - Rank-based Filtering - Transformers API Reference","doc":"This real-world example shows how to implement complex rank-based filtering for college admissions:\n\n```elixir\ndefmodule CounsellingWeb.CollegeLive.Index do\n  use CounsellingWeb, :live_view\n  alias Counselling.Colleges\n  alias CounsellingWeb.CollegeLive.CustomHeader\n  use LiveTable.LiveResource\n  import Ecto.Query\n\n  @impl true\n  def mount(_params, _session, socket) do\n    socket =\n      socket\n      |> assign(:page_title, \"Colleges\")\n      |> assign(:data_provider, {Colleges, :list_colleges, []})\n\n    {:ok, socket}\n  end\n\n  def fields do\n    [\n      id: %{label: \"ID\", sortable: true},\n      name: %{label: \"College Name\", sortable: true, searchable: true},\n      location: %{label: \"Location\", sortable: true, searchable: true}\n    ]\n  end\n\n  def table_options do\n    %{\n      mode: :card,\n      card_component: &CounsellingWeb.CollegeComponent.college_component/1,\n      custom_header: {CustomHeader, :custom_header}\n    }\n  end\n\n  def filters do\n    [\n      # Boolean filters for institution types\n      iit: Boolean.new(:class, \"iit\", %{\n        label: \"IIT\",\n        condition: dynamic([c], c.class == :IIT)\n      }),\n      nit: Boolean.new(:class, \"nit\", %{\n        label: \"NIT\", \n        condition: dynamic([c], c.class == :NIT)\n      }),\n      \n      # Complex rank-based transformer\n      rank: Transformer.new(\"rank\", %{\n        query_transformer: &rank_filter/2\n      }),\n      \n      # Dynamic sorting transformer\n      sort_mode: Transformer.new(\"sort_mode\", %{\n        query_transformer: &sort_query/2\n      }),\n      \n      # NIRF ranking limitations\n      limit_results: Transformer.new(\"limit_results\", %{\n        query_transformer: &limit_results/2\n      })\n    ]\n  end\n\n  # Complex rank filtering with institution-specific adjustments\n  def rank_filter(query, %{\"value\" => \"\"}) do\n    query\n  end\n\n  def rank_filter(query, %{\"value\" => rank}) do\n    number = String.to_integer(rank)\n\n    query\n    |> where(\n      [c, _, rc],\n      rc.closing_rank >= ^number or\n        rc.closing_rank +\n          fragment(\n            \"\"\"\n            CASE ?\n              WHEN 'IIT' THEN ? * 0.05 *\n                CASE WHEN ? <= 1000 THEN 0.8\n                     WHEN ? <= 5000 THEN 1.0\n                     WHEN ? <= 20000 THEN 1.2\n                     ELSE 1.5 END\n              WHEN 'NIT' THEN ? * 0.08 *\n                CASE WHEN ? <= 1000 THEN 0.8\n                     WHEN ? <= 5000 THEN 1.0\n                     WHEN ? <= 20000 THEN 1.2\n                     ELSE 1.5 END\n              WHEN 'IIIT' THEN ? * 0.12 *\n                CASE WHEN ? <= 1000 THEN 0.8\n                     WHEN ? <= 5000 THEN 1.0\n                     WHEN ? <= 20000 THEN 1.2\n                     ELSE 1.5 END\n              ELSE ? * 0.15 *\n                CASE WHEN ? <= 1000 THEN 0.8\n                     WHEN ? <= 5000 THEN 1.0\n                     WHEN ? <= 20000 THEN 1.2\n                     ELSE 1.5 END\n            END\n            \"\"\",\n            c.class, rc.closing_rank, rc.closing_rank, rc.closing_rank, rc.closing_rank,\n            rc.closing_rank, rc.closing_rank, rc.closing_rank, rc.closing_rank,\n            rc.closing_rank, rc.closing_rank, rc.closing_rank, rc.closing_rank,\n            rc.closing_rank, rc.closing_rank, rc.closing_rank, rc.closing_rank\n          ) >= ^number\n    )\n  end\n\n  # NIRF ranking filters\n  def limit_results(query, %{\"nirf\" => \"Top 10\"}) do\n    query |> where([_, _, _, nr], nr.nirf_rank <= 10)\n  end\n\n  def limit_results(query, %{\"nirf\" => \"Top 25\"}) do\n    query |> where([_, _, _, nr], nr.nirf_rank <= 25)\n  end\n\n  def limit_results(query, %{\"nirf\" => \"All Rankings\"}) do\n    query\n  end\n\n  # Dynamic sorting based on user preference\n  def sort_query(query, %{\"sort_by\" => \"NIRF Ranking\"}) do\n    query |> exclude(:order_by) |> order_by([_, _, _, nr], nr.nirf_rank)\n  end\n\n  def sort_query(query, %{\"sort_by\" => \"Name (A-Z)\"}) do\n    query |> exclude(:order_by) |> order_by([c], c.name)\n  end\n\n  def sort_query(query, %{\"sort_by\" => \"Name (Z-A)\"}) do\n    query |> exclude(:order_by) |> order_by([c], desc: c.name)\n  end\n\n  def sort_query(query, _params) do\n    query\n  end\nend\n```","ref":"transformers.html#college-counselling-rank-based-filtering"},{"type":"extras","title":"Complex Search with Rankings - Transformers API Reference","doc":"```elixir\ndef filters do\n  [\n    smart_search: Transformer.new(\"smart_search\", %{\n      query_transformer: &apply_smart_search/2\n    })\n  ]\nend\n\ndefp apply_smart_search(query, %{\"query\" => search_term}) when search_term != \"\" do\n  # Complex search with relevance scoring\n  from p in query,\n    left_join: c in Category, on: c.id == p.category_id,\n    left_join: r in Review, on: r.product_id == p.id,\n    where: \n      ilike(p.name, ^\"%#{search_term}%\") or\n      ilike(p.description, ^\"%#{search_term}%\") or\n      ilike(c.name, ^\"%#{search_term}%\"),\n    group_by: [p.id, p.name, p.description, p.price],\n    order_by: [\n      desc: fragment(\n        \"CASE \n          WHEN ? ILIKE ? THEN 100\n          WHEN ? ILIKE ? THEN 75\n          WHEN ? ILIKE ? THEN 50\n          ELSE 25\n        END\",\n        p.name, ^\"#{search_term}%\",\n        p.name, ^\"%#{search_term}%\", \n        p.description, ^\"%#{search_term}%\"\n      )\n    ],\n    select_merge: %{\n      relevance_score: fragment(\n        \"CASE \n          WHEN ? ILIKE ? THEN 100\n          WHEN ? ILIKE ? THEN 75  \n          WHEN ? ILIKE ? THEN 50\n          ELSE 25\n        END\",\n        p.name, ^\"#{search_term}%\",\n        p.name, ^\"%#{search_term}%\",\n        p.description, ^\"%#{search_term}%\"\n      ),\n      review_count: count(r.id)\n    }\nend\n\ndefp apply_smart_search(query, _), do: query\n```","ref":"transformers.html#complex-search-with-rankings"},{"type":"extras","title":"Conditional Joins Based on User Role - Transformers API Reference","doc":"```elixir\ndef filters do\n  [\n    access_control: Transformer.new(\"access_control\", %{\n      query_transformer: {MyApp.AccessControl, :apply_user_permissions}\n    })\n  ]\nend\n\n# In your context\ndefmodule MyApp.AccessControl do\n  def apply_user_permissions(query, %{\"user_role\" => \"admin\"}) do\n    # Admins see everything with additional data\n    from p in query,\n      left_join: au in AuditLog, on: au.product_id == p.id,\n      group_by: [p.id],\n      select_merge: %{\n        last_modified_by: max(au.user_id),\n        modification_count: count(au.id)\n      }\n  end\n  \n  def apply_user_permissions(query, %{\"user_role\" => \"manager\", \"department_id\" => dept_id}) do\n    # Managers see only their department's products\n    from p in query,\n      join: d in Department, on: d.id == p.department_id,\n      where: d.id == ^dept_id\n  end\n  \n  def apply_user_permissions(query, %{\"user_role\" => \"user\"}) do\n    # Regular users see only active, public products\n    from p in query,\n      where: p.active == true and p.public == true\n  end\n  \n  def apply_user_permissions(query, _), do: query\nend\n```","ref":"transformers.html#conditional-joins-based-on-user-role"},{"type":"extras","title":"State Management - Transformers API Reference","doc":"Transformers maintain state through URL parameters and can handle complex form data:","ref":"transformers.html#state-management"},{"type":"extras","title":"Form Integration with Custom Header - Transformers API Reference","doc":"This example shows how to integrate transformers with a completely custom header component:\n\n```elixir\n# Custom Header Component (custom_header.ex)\ndefmodule CounsellingWeb.CollegeLive.CustomHeader do\n  use Phoenix.Component\n\n  def custom_header(assigns) do\n    ~H\"\"\"\n     \n       \n         \n          <!-- Main rank input with transformer -->\n           \n             \n                \n               \n                Find Colleges for Your Rank\n               \n             \n            \n             \n               \n                <.form for={%{}} phx-debounce={get_in(@table_options, [:search, :debounce])} phx-change=\"sort\">\n                   \n                    Your JEE Rank  (Saved automatically, filters all pages) \n                   \n                   \n                </.form>\n               \n              \n              <!-- Category selector -->\n               \n                 \n                  Category  (Select your reservation category) \n                 \n                 \n                   General \n                   OBC-NCL \n                   SC \n                   ST \n                   EWS \n                 \n               \n             \n           \n\n          <!-- Search and filtering controls -->\n           \n             \n              <.form for={%{}} phx-debounce={get_in(@table_options, [:search, :debounce])} phx-change=\"sort\">\n                 \n                  Search Colleges  (By name or location) \n                 \n                 \n                   \n                   \n                      \n                   \n                 \n              </.form>\n             \n\n            <!-- NIRF Ranking filter (transformer) -->\n             \n              <.form for={%{}} phx-debounce={get_in(@table_options, [:search, :debounce])} phx-change=\"sort\">\n                 \n                  NIRF Ranking\n                 \n                 \n                   All Rankings \n                   Top 10 \n                   Top 25 \n                 \n              </.form>\n             \n\n            <!-- Sort transformer -->\n             \n              <.form for={%{}} phx-debounce={get_in(@table_options, [:search, :debounce])} phx-change=\"sort\">\n                 \n                  Sort By  (Change display order) \n                 \n                 \n                   NIRF Ranking \n                   Name (A-Z) \n                   Name (Z-A) \n                 \n              </.form>\n             \n           \n\n          <!-- Boolean filters for institution types -->\n           \n            <.form for={%{}} phx-debounce={get_in(@table_options, [:search, :debounce])} phx-change=\"sort\">\n               \n                Institution Type  (IIT = Premier institutes, NIT = National institutes, IIIT = Information technology, GFTI = Government funded) \n               \n               \n                <.input\n                  :for={{id, %Boolean{field: :class, options: %{label: label}}} <- @filters}\n                  type=\"checkbox\"\n                  name={\"filters[#{id}]\"}\n                  label={label}\n                  checked={Map.has_key?(@options[\"filters\"], id)}\n                />\n               \n            </.form>\n           \n\n          <!-- Clear filters -->\n           \n            <.link\n              :if={@options[\"filters\"] != %{\"search\" => \"\"}}\n              phx-click=\"sort\"\n              phx-value-clear_filters=\"true\"\n              class=\"my-auto text-sm text-gray-500 hover:text-gray-700 dark:text-gray-400 dark:hover:text-gray-300 transition-colors\"\n            >\n              Clear All Filters\n            </.link>\n           \n         \n       \n     \n    \"\"\"\n  end\nend\n\n# Helper function for accessing transformer values\ndefp get_transformer_value(filters, transformer_key, field_key) do\n  case Map.get(filters, transformer_key) do\n    %LiveTable.Transformer{options: %{applied_data: data}} ->\n      Map.get(data, field_key, \"\")\n    _ ->\n      \"\"\n  end\nend\n```","ref":"transformers.html#form-integration-with-custom-header"},{"type":"extras","title":"Event Handling - Transformers API Reference","doc":"```elixir\ndef handle_event(\"update_transformer\", %{\"sales_performance\" => params}, socket) do\n  # Update transformer state without applying\n  {:noreply, socket}\nend\n\ndef handle_event(\"apply_transformer\", %{\"sales_performance\" => params}, socket) do\n  # Apply transformer by updating URL parameters\n  {:noreply, \n   push_patch(socket, \n     to: ~p\"/products?#{%{\"sales_performance\" => params}}\"\n   )}\nend\n```","ref":"transformers.html#event-handling"},{"type":"extras","title":"Advanced Patterns - Transformers API Reference","doc":"","ref":"transformers.html#advanced-patterns"},{"type":"extras","title":"Chaining Transformers - Transformers API Reference","doc":"```elixir\ndef filters do\n  [\n    # First transformer: Add time-based filtering\n    time_filter: Transformer.new(\"time_filter\", %{\n      query_transformer: &apply_time_filter/2\n    }),\n    \n    # Second transformer: Add performance metrics  \n    performance_metrics: Transformer.new(\"performance\", %{\n      query_transformer: &add_performance_metrics/2\n    }),\n    \n    # Third transformer: Apply sorting/ranking\n    ranking: Transformer.new(\"ranking\", %{\n      query_transformer: &apply_ranking/2\n    })\n  ]\nend\n\n# Transformers are applied in order defined\n```","ref":"transformers.html#chaining-transformers"},{"type":"extras","title":"Conditional Transformer Application - Transformers API Reference","doc":"```elixir\ndefp apply_analytics_transformer(query, filter_data) do\n  # Only apply if user has analytics permission\n  if has_analytics_access?(filter_data) do\n    from p in query,\n      join: a in Analytics, on: a.product_id == p.id,\n      select_merge: %{\n        conversion_rate: a.conversion_rate,\n        bounce_rate: a.bounce_rate,\n        avg_session_duration: a.avg_session_duration\n      }\n  else\n    query\n  end\nend\n\ndefp has_analytics_access?(%{\"user_permissions\" => permissions}) do\n  \"analytics\" in permissions\nend\n\ndefp has_analytics_access?(_), do: false\n```","ref":"transformers.html#conditional-transformer-application"},{"type":"extras","title":"Error Handling - Transformers API Reference","doc":"```elixir\ndefp safe_transformer(query, filter_data) do\n  try do\n    apply_complex_transformation(query, filter_data)\n  rescue\n    e in Ecto.Query.CompileError ->\n      Logger.error(\"Query compilation error in transformer: #{inspect(e)}\")\n      query\n      \n    e ->\n      Logger.error(\"Unexpected error in transformer: #{inspect(e)}\")\n      query\n  end\nend\n```","ref":"transformers.html#error-handling"},{"type":"extras","title":"Performance Considerations - Transformers API Reference","doc":"","ref":"transformers.html#performance-considerations"},{"type":"extras","title":"Efficient Query Building - Transformers API Reference","doc":"```elixir\n# ✅ Good: Build query incrementally\ndefp apply_multi_condition_filter(query, filter_data) do\n  query\n  |> maybe_add_time_filter(filter_data)\n  |> maybe_add_category_filter(filter_data)\n  |> maybe_add_metrics(filter_data)\nend\n\ndefp maybe_add_time_filter(query, %{\"date_range\" => range}) when range != \"\" do\n  # Add time-based filtering\n  query\nend\n\ndefp maybe_add_time_filter(query, _), do: query\n\n# ❌ Avoid: Complex nested conditions in single query\n```","ref":"transformers.html#efficient-query-building"},{"type":"extras","title":"Database Index Optimization - Transformers API Reference","doc":"```sql\n-- Add indexes for transformer queries\nCREATE INDEX idx_sales_product_period ON sales(product_id, period);\nCREATE INDEX idx_orders_date_status ON orders(inserted_at, status);\nCREATE INDEX idx_products_category_active ON products(category_id, active);\n```","ref":"transformers.html#database-index-optimization"},{"type":"extras","title":"Debugging Transformers - Transformers API Reference","doc":"","ref":"transformers.html#debugging-transformers"},{"type":"extras","title":"Query Inspection - Transformers API Reference","doc":"```elixir\ndefp debug_transformer(query, filter_data) do\n  result_query = apply_transformation(query, filter_data)\n  \n  # Log the generated SQL (in development)\n  if Mix.env() == :dev do\n    IO.inspect(Ecto.Adapters.SQL.to_sql(:all, Repo, result_query), label: \"Transformer Query\")\n  end\n  \n  result_query\nend\n```","ref":"transformers.html#query-inspection"},{"type":"extras","title":"State Debugging - Transformers API Reference","doc":"```elixir\ndef handle_event(\"debug_transformers\", _params, socket) do\n  transformers = socket.assigns.filters\n  |> Enum.filter(fn {_, filter} -> \n    match?(%LiveTable.Transformer{}, filter)\n  end)\n  \n  IO.inspect(transformers, label: \"Active Transformers\")\n  {:noreply, socket}\nend\n```","ref":"transformers.html#state-debugging"},{"type":"extras","title":"Best Practices - Transformers API Reference","doc":"","ref":"transformers.html#best-practices"},{"type":"extras","title":"1. Keep Transformers Focused - Transformers API Reference","doc":"```elixir\n# ✅ Good: Single responsibility\nsales_filter: Transformer.new(\"sales\", %{\n  query_transformer: &add_sales_filtering/2\n})\n\nmetrics_calculator: Transformer.new(\"metrics\", %{\n  query_transformer: &add_performance_metrics/2\n})\n\n# ❌ Avoid: Doing everything in one transformer\n```","ref":"transformers.html#1-keep-transformers-focused"},{"type":"extras","title":"2. Handle Empty/Invalid Data - Transformers API Reference","doc":"```elixir\ndefp apply_filter(query, filter_data) do\n  case filter_data do\n    %{\"field\" => value} when value != \"\" and not is_nil(value) ->\n      # Apply transformation\n      transform_query(query, value)\n    _ ->\n      # Return unchanged query for invalid/empty data\n      query\n  end\nend\n```","ref":"transformers.html#2-handle-empty-invalid-data"},{"type":"extras","title":"3. Use Type Safety - Transformers API Reference","doc":"```elixir\ndefp apply_numeric_filter(query, %{\"amount\" => amount_str}) do\n  case Integer.parse(amount_str || \"\") do\n    {amount, \"\"} when amount > 0 ->\n      from p in query, where: p.amount >= ^amount\n    _ ->\n      query\n  end\nend\n```","ref":"transformers.html#3-use-type-safety"},{"type":"extras","title":"4. Document Complex Logic - Transformers API Reference","doc":"```elixir\ndefp apply_complex_business_logic(query, filter_data) do\n  # This transformer implements the \"Premium Customer Analysis\" requirements:\n  # 1. Customers with orders > $1000 in last 6 months\n  # 2. Include loyalty points and tier information  \n  # 3. Calculate lifetime value with predictive scoring\n  \n  case filter_data do\n    %{\"analysis_type\" => \"premium\"} ->\n      # Implementation here...\n  end\nend\n```","ref":"transformers.html#4-document-complex-logic"},{"type":"extras","title":"Troubleshooting - Transformers API Reference","doc":"","ref":"transformers.html#troubleshooting"},{"type":"extras","title":"Common Issues - Transformers API Reference","doc":"**Transformer not applying:**\n- Check that transformer key in URL matches filter definition\n- Verify transformer function exists and is accessible\n- Ensure filter_data is in expected format\n\n**Query compilation errors:**\n- Check that all referenced tables/fields exist\n- Verify join conditions are correct\n- Test transformer functions in isolation\n\n**Performance issues:**\n- Add database indexes for transformer queries\n- Use `explain analyze` to check query execution plans\n- Consider breaking complex transformers into simpler ones\n\n**State not persisting:**\n- Verify transformer key is included in URL parameters\n- Check that form field names match expected data structure\n- Ensure LiveView handle_params processes transformer data\n\nTransformers provide unlimited flexibility for complex filtering scenarios while maintaining the simplicity and URL persistence that makes LiveTable powerful.","ref":"transformers.html#common-issues"},{"type":"extras","title":"Exports API Reference","doc":"# Exports API Reference\n\nLiveTable provides export capabilities for generating CSV and PDF files from your table data. Exports are processed in the background using Oban, ensuring your LiveView remains responsive.","ref":"exports.html"},{"type":"extras","title":"Overview - Exports API Reference","doc":"LiveTable supports two export formats:\n- **CSV** - Comma-separated values for spreadsheet applications\n- **PDF** - Formatted documents using Typst typesetting engine\n\nExports include all table data (not just the current page) and respect active filters. Files are generated asynchronously and served as downloadable links.","ref":"exports.html#overview"},{"type":"extras","title":"Basic Setup - Exports API Reference","doc":"","ref":"exports.html#basic-setup"},{"type":"extras","title":"1. Enable Exports in Table Configuration - Exports API Reference","doc":"```elixir\ndefmodule YourAppWeb.ProductLive.Index do\n  use YourAppWeb, :live_view\n  use LiveTable.LiveResource, schema: YourApp.Product\n\n  def table_options do\n    %{\n      exports: %{\n        enabled: true,\n        formats: [:csv, :pdf]  # Enable both formats\n      }\n    }\n  end\nend\n```","ref":"exports.html#1-enable-exports-in-table-configuration"},{"type":"extras","title":"2. Configure Static File Serving - Exports API Reference","doc":"Add exports to your static paths in `lib/your_app_web.ex`:\n\n```elixir\ndef static_paths do\n  ~w(assets fonts images favicon.ico robots.txt exports)\nend\n```","ref":"exports.html#2-configure-static-file-serving"},{"type":"extras","title":"Export Configuration Options - Exports API Reference","doc":"","ref":"exports.html#export-configuration-options"},{"type":"extras","title":"Table-Level Configuration - Exports API Reference","doc":"```elixir\ndef table_options do\n  %{\n    exports: %{\n      enabled: true,                    # Enable/disable exports\n      formats: [:csv, :pdf]            # Available formats\n    }\n  }\nend\n```","ref":"exports.html#table-level-configuration"},{"type":"extras","title":"Application-Wide Configuration - Exports API Reference","doc":"```elixir\n# config/config.exs\nconfig :live_table,\n  defaults: %{\n    exports: %{\n      enabled: true,\n      formats: [:csv, :pdf]\n    }\n  }\n```","ref":"exports.html#application-wide-configuration"},{"type":"extras","title":"Oban Configuration - Exports API Reference","doc":"```elixir\n# config/config.exs\nconfig :your_app, Oban,\n  repo: YourApp.Repo,\n  engine: Oban.Engines.Basic,\n  notifier: Oban.Notifiers.Postgres,\n  plugins: [Oban.Plugins.Pruner],\n  queues: [\n    exports: 10,        # Export processing queue\n    default: 5          # Other background jobs\n  ]\n```","ref":"exports.html#oban-configuration"},{"type":"extras","title":"CSV Exports - Exports API Reference","doc":"CSV exports use NimbleCSV for fast, memory-efficient processing.","ref":"exports.html#csv-exports"},{"type":"extras","title":"Features - Exports API Reference","doc":"- **Streaming processing** - Handles large datasets without memory issues\n- **Custom headers** - Uses field labels from your configuration\n- **Filter respect** - Only exports filtered/searched data\n- **UTF-8 encoding** - Proper character encoding for international data","ref":"exports.html#features"},{"type":"extras","title":"Example CSV Output - Exports API Reference","doc":"```csv\nID,Product Name,Price,Stock,Category\n1,iPhone 15 Pro,999.99,25,Electronics\n2,MacBook Air,1199.99,8,Electronics\n3,Wireless Mouse,45.99,50,Accessories\n```","ref":"exports.html#example-csv-output"},{"type":"extras","title":"CSV Configuration - Exports API Reference","doc":"```elixir\ndef table_options do\n  %{\n    exports: %{\n      enabled: true,\n      formats: [:csv]\n    }\n  }\nend\n```","ref":"exports.html#csv-configuration"},{"type":"extras","title":"PDF Exports - Exports API Reference","doc":"PDF exports use Typst for professional document formatting.","ref":"exports.html#pdf-exports"},{"type":"extras","title":"Prerequisites - Exports API Reference","doc":"Install Typst on your system:\n\n**macOS:**\n```bash\nbrew install typst\n```\n\n**Ubuntu/Debian:**\n```bash\n# Download latest release\nwget https://github.com/typst/typst/releases/latest/download/typst-x86_64-unknown-linux-musl.tar.xz\ntar -xf typst-x86_64-unknown-linux-musl.tar.xz\nsudo mv typst-x86_64-unknown-linux-musl/typst /usr/local/bin/\n```\n\n**Windows:**\nDownload from [Typst Releases](https://github.com/typst/typst/releases) and add to PATH.\n\n**Verify Installation:**\n```bash\ntypst --version\n```","ref":"exports.html#prerequisites"},{"type":"extras","title":"Features - Exports API Reference","doc":"- **Professional formatting** - Clean, readable table layouts\n- **Large dataset support** - Efficient processing for thousands of records\n- **Responsive tables** - Automatic column width adjustment","ref":"exports.html#features"},{"type":"extras","title":"PDF Configuration - Exports API Reference","doc":"```elixir\ndef table_options do\n  %{\n    exports: %{\n      enabled: true,\n      formats: [:pdf]\n    }\n  }\nend\n```","ref":"exports.html#pdf-configuration"},{"type":"extras","title":"Export Process Flow - Exports API Reference","doc":"","ref":"exports.html#export-process-flow"},{"type":"extras","title":"1. User Initiates Export - Exports API Reference","doc":"When a user clicks an export button, LiveTable:\n1. Captures current table state (filters, search, sort)\n2. Queues a background job with this state\n3. Shows a \"Processing...\" message\n4. Returns immediately (non-blocking)","ref":"exports.html#1-user-initiates-export"},{"type":"extras","title":"2. Background Processing - Exports API Reference","doc":"The Oban worker:\n1. Recreates the exact query from captured state\n2. Streams data in batches\n3. Generates the export file\n4. Saves it to the exports directory\n5. Updates job status","ref":"exports.html#2-background-processing"},{"type":"extras","title":"3. Download Delivery - Exports API Reference","doc":"Once complete:\n1. User receives a download link\n2. File is served via static file handling\n3. Files auto-expire after configured time","ref":"exports.html#3-download-delivery"},{"type":"extras","title":"Troubleshooting - Exports API Reference","doc":"","ref":"exports.html#troubleshooting"},{"type":"extras","title":"Common Issues - Exports API Reference","doc":"**Exports not generating:**\n- Check Oban is running and exports queue is configured\n- Verify static file directory exists and is writable\n- Check for errors in Oban dashboard\n\n**PDF generation failing:**\n- Ensure Typst is installed and in PATH\n- Check file permissions in export directory\n- Verify system has enough disk space\n\n**Download links not working:**\n- Verify static file serving is configured\n- Check exports directory is in static_paths\n- Ensure web server can serve files from exports directory","ref":"exports.html#common-issues"},{"type":"extras","title":"Debug Export Jobs - Exports API Reference","doc":"```elixir\n# Check export job status\niex> Oban.Job |> where([j], j.queue == \"exports\") |> Repo.all()\n\n# Check export files\niex> File.ls!(Path.join([:code.priv_dir(:your_app), \"static\", \"exports\"]))\n```","ref":"exports.html#debug-export-jobs"},{"type":"extras","title":"Examples - Exports API Reference","doc":"","ref":"exports.html#examples"},{"type":"extras","title":"E-commerce Product Export - Exports API Reference","doc":"```elixir\ndef table_options do\n  %{\n    exports: %{\n      enabled: true,\n      formats: [:csv, :pdf]\n    }\n  }\nend\n```","ref":"exports.html#e-commerce-product-export"},{"type":"extras","title":"Customer Data Export - Exports API Reference","doc":"```elixir\ndef table_options do\n  %{\n    exports: %{\n      enabled: true,\n      formats: [:csv]  # CSV only for customer data\n    }\n  }\nend\n```","ref":"exports.html#customer-data-export"},{"type":"extras","title":"Financial Report Export - Exports API Reference","doc":"```elixir\ndef table_options do\n  %{\n    exports: %{\n      enabled: true,\n      formats: [:pdf]  # PDF only for financial reports\n    }\n  }\nend\n```","ref":"exports.html#financial-report-export"},{"type":"extras","title":"Table Options API Reference","doc":"# Table Options API Reference\n\nTable options control the behavior and appearance of your LiveTable. They can be configured at three levels: built-in defaults, application-wide settings, and per-table overrides.","ref":"table-options.html"},{"type":"extras","title":"Configuration Hierarchy - Table Options API Reference","doc":"LiveTable uses a three-tier configuration system where each level overrides the previous:\n\n1. **LiveTable Defaults** - Built-in sensible defaults\n2. **Application Configuration** - Set in `config/config.exs`\n3. **Table Options** - Per-table overrides via `table_options/0`","ref":"table-options.html#configuration-hierarchy"},{"type":"extras","title":"Per-Table Configuration - Table Options API Reference","doc":"Override settings for specific tables by implementing `table_options/0`:\n\n```elixir\ndefmodule YourAppWeb.ProductLive.Index do\n  use YourAppWeb, :live_view\n  use LiveTable.LiveResource, schema: YourApp.Product\n  \n  def table_options do\n    %{\n      pagination: %{\n        enabled: true,\n        sizes: [10, 25, 50]\n      },\n      sorting: %{\n        default_sort: [name: :asc]\n      },\n      mode: :table\n    }\n  end\nend\n```","ref":"table-options.html#per-table-configuration"},{"type":"extras","title":"Application-Wide Configuration - Table Options API Reference","doc":"Set defaults for all tables in your application:\n\n```elixir\n# config/config.exs\nconfig :live_table,\n  defaults: %{\n    pagination: %{\n      enabled: true,\n      sizes: [20, 50, 100]\n    },\n    sorting: %{\n      default_sort: [inserted_at: :desc]\n    },\n    exports: %{\n      enabled: true,\n      formats: [:csv, :pdf]\n    }\n  }\n```","ref":"table-options.html#application-wide-configuration"},{"type":"extras","title":"Available Options - Table Options API Reference","doc":"","ref":"table-options.html#available-options"},{"type":"extras","title":"Pagination Options - Table Options API Reference","doc":"Control pagination behavior and appearance.\n\n```elixir\npagination: %{\n  enabled: true,              # Enable/disable pagination\n  sizes: [10, 25, 50],       # Available page size options\n  default_size: 25           # Default page size\n}\n```\n\n**Options:**\n- `enabled` (boolean) - Enable or disable pagination entirely\n- `sizes` (list) - Available page size options (recommended max: 50)\n- `default_size` (integer) - Default number of records per page\n\n**Examples:**\n\n```elixir\n# Disable pagination (show all records)\npagination: %{enabled: false}\n\n# Custom page sizes for admin interface\npagination: %{\n  enabled: true,\n  sizes: [5, 15, 30, 50],\n  default_size: 15\n}\n\n# Large datasets with conservative pagination\npagination: %{\n  enabled: true,\n  sizes: [10, 20],\n  default_size: 10\n}\n```","ref":"table-options.html#pagination-options"},{"type":"extras","title":"Sorting Options - Table Options API Reference","doc":"Configure sorting behavior and defaults.\n\n```elixir\nsorting: %{\n  enabled: true,                     # Enable/disable sorting\n  default_sort: [id: :asc]          # Default sort order\n}\n```\n\n**Options:**\n- `enabled` (boolean) - Enable or disable column sorting\n- `default_sort` (keyword list) - Default sort columns and directions\n\n**Examples:**\n\n```elixir\n# Default sort order\nsorting: %{\n  enabled: true,\n  default_sort: [name: :asc]\n}\n\n# Disable sorting entirely\nsorting: %{enabled: false}\n```","ref":"table-options.html#sorting-options"},{"type":"extras","title":"Export Options - Table Options API Reference","doc":"Control export functionality and formats.\n\n```elixir\nexports: %{\n  enabled: true,                    # Enable/disable exports\n  formats: [:csv, :pdf]            # Available export formats\n}\n```\n\n**Options:**\n- `enabled` (boolean) - Enable or disable export functionality\n- `formats` (list) - Available formats (`:csv`, `:pdf`, or both)\n\n**Examples:**\n\n```elixir\n# CSV exports only\nexports: %{\n  enabled: true,\n  formats: [:csv]\n}\n\n# PDF reports only\nexports: %{\n  enabled: true,\n  formats: [:pdf]\n}\n\n# Disable exports for sensitive data\nexports: %{enabled: false}\n```","ref":"table-options.html#export-options"},{"type":"extras","title":"Search Options - Table Options API Reference","doc":"Configure global search behavior.\n\n```elixir\nsearch: %{\n  enabled: true,                    # Enable/disable search\n  debounce: 300,                   # Debounce time in milliseconds\n  placeholder: \"Search records...\"  # Search input placeholder\n}\n```\n\n**Options:**\n- `enabled` (boolean) - Enable or disable global text search\n- `debounce` (integer) - Delay before search executes (milliseconds)\n- `placeholder` (string) - Placeholder text for search input\n\n**Examples:**\n\n```elixir\n# Quick search for small datasets\nsearch: %{\n  enabled: true,\n  debounce: 100,\n  placeholder: \"Quick search...\"\n}\n\n# Slower search for large datasets\nsearch: %{\n  enabled: true,\n  debounce: 500,\n  placeholder: \"Search products...\"\n}\n\n# Disable search\nsearch: %{enabled: false}\n```","ref":"table-options.html#search-options"},{"type":"extras","title":"Display Mode Options - Table Options API Reference","doc":"Control table layout and appearance.\n\n```elixir\nmode: :table  # or :card\n```\n\n**Table Mode:**\nTraditional table layout with rows and columns.\n\n```elixir\nmode: :table\n```\n\n**Card Mode:**\nGrid layout using custom card components.\n\n```elixir\nmode: :card,\ncard_component: &product_card/1\n```","ref":"table-options.html#display-mode-options"},{"type":"extras","title":"Custom Components - Table Options API Reference","doc":"Override default LiveTable components.\n\n```elixir\ncomponents: %{\n  header: &custom_header/1,         # Custom table header\n  pagination: &custom_pagination/1,  # Custom pagination controls\n  search: &custom_search/1          # Custom search input\n}\n```","ref":"table-options.html#custom-components"},{"type":"extras","title":"Complete Configuration Examples - Table Options API Reference","doc":"","ref":"table-options.html#complete-configuration-examples"},{"type":"extras","title":"E-commerce Product Table - Table Options API Reference","doc":"```elixir\ndef table_options do\n  %{\n    pagination: %{\n      enabled: true,\n      sizes: [12, 24, 48],\n      default_size: 24\n    },\n    sorting: %{\n      enabled: true,\n      default_sort: [featured: :desc, name: :asc],\n      multi_column: true\n    },\n    exports: %{\n      enabled: true,\n      formats: [:csv]\n    },\n    search: %{\n      enabled: true,\n      debounce: 300,\n      placeholder: \"Search products...\"\n    },\n    mode: :card,\n    card_component: &product_card/1,\n    card_component: &product_card/1\n  }\nend\n\ndefp product_card(assigns) do\n  ~H\"\"\"\n   \n     \n     <%= @record.name %> \n     <%= @record.description %> \n     \n       $<%= @record.price %> \n       <%= @record.stock_quantity %> in stock \n     \n   \n  \"\"\"\nend\n```","ref":"table-options.html#e-commerce-product-table"},{"type":"extras","title":"Admin User Management Table - Table Options API Reference","doc":"```elixir\ndef table_options do\n  %{\n    pagination: %{\n      enabled: true,\n      sizes: [10, 25, 50, 100],\n      default_size: 25\n    },\n    sorting: %{\n      enabled: true,\n      default_sort: [last_sign_in_at: :desc],\n      multi_column: true\n    },\n    exports: %{\n      enabled: true,\n      formats: [:csv, :pdf]\n    },\n    search: %{\n      enabled: true,\n      debounce: 300,\n      placeholder: \"Search by name or email...\"\n    },\n    mode: :table\n  }\nend\n```","ref":"table-options.html#admin-user-management-table"},{"type":"extras","title":"Analytics Dashboard Table - Table Options API Reference","doc":"```elixir\ndef table_options do\n  %{\n    pagination: %{\n      enabled: true,\n      sizes: [50, 100, 200],\n      default_size: 100\n    },\n    sorting: %{\n      enabled: true,\n      default_sort: [date: :desc, revenue: :desc],\n      multi_column: true\n    },\n    exports: %{\n      enabled: true,\n      formats: [:csv, :pdf]\n    },\n    search: %{\n      enabled: false  # Disable search for analytics data\n    },\n    mode: :table\n  }\nend\n```","ref":"table-options.html#analytics-dashboard-table"},{"type":"extras","title":"Minimal Configuration Table - Table Options API Reference","doc":"```elixir\ndef table_options do\n  %{\n    pagination: %{enabled: false},\n    sorting: %{enabled: false},\n    exports: %{enabled: false},\n    search: %{enabled: false}\n  }\nend\n```","ref":"table-options.html#minimal-configuration-table"},{"type":"extras","title":"Environment-Specific Configuration - Table Options API Reference","doc":"Configure different settings per environment:\n\n```elixir\n# config/dev.exs\nconfig :live_table,\n  defaults: %{\n    pagination: %{\n      sizes: [5, 10, 15],    # Smaller pages for development\n      default_size: 5\n    },\n    exports: %{enabled: false}  # Disable exports in development\n  }\n\n# config/test.exs\nconfig :live_table,\n  defaults: %{\n    pagination: %{enabled: false},  # Show all records in tests\n    exports: %{enabled: false},\n    search: %{debounce: 0}         # No debounce in tests\n  }\n\n# config/prod.exs\nconfig :live_table,\n  defaults: %{\n    pagination: %{\n      sizes: [25, 50, 100],\n      default_size: 50\n    },\n    exports: %{\n      enabled: true,\n      formats: [:csv, :pdf]\n    },\n    search: %{debounce: 500}       # Longer debounce for production\n  }\n```","ref":"table-options.html#environment-specific-configuration"},{"type":"extras","title":"Card Mode Configuration - Table Options API Reference","doc":"","ref":"table-options.html#card-mode-configuration"},{"type":"extras","title":"Basic Card Layout - Table Options API Reference","doc":"```elixir\ndef table_options do\n  %{\n    mode: :card,\n    card_component: &simple_card/1\n  }\nend\n\ndefp simple_card(assigns) do\n  ~H\"\"\"\n   \n     <%= @record.name %> \n     <%= @record.description %> \n   \n  \"\"\"\nend\n```","ref":"table-options.html#basic-card-layout"},{"type":"extras","title":"Responsive Card Grid - Table Options API Reference","doc":"```elixir\ndef table_options do\n  %{\n    mode: :card,\n    card_component: &responsive_card/1\n  }\nend\n```","ref":"table-options.html#responsive-card-grid"},{"type":"extras","title":"Advanced Card with Actions - Table Options API Reference","doc":"```elixir\ndefp product_card(assigns) do\n  ~H\"\"\"\n   \n     \n       \n     \n    \n     \n       \n         <%= @record.name %> \n         \n          <%= if @record.active, do: \"Active\", else: \"Inactive\" %>\n         \n       \n      \n       <%= @record.description %> \n      \n       \n         $<%= @record.price %> \n         <%= @record.stock_quantity %> left \n       \n      \n       \n        <.link \n          navigate={~p\"/products/#{@record.id}\"} \n          class=\"flex-1 bg-blue-600 text-white text-center py-2 px-4 rounded text-sm hover:bg-blue-700\"\n        >\n          Edit\n        </.link>\n         \n          <%= if @record.featured, do: \"Unfeature\", else: \"Feature\" %>\n         \n       \n     \n   \n  \"\"\"\nend\n```","ref":"table-options.html#advanced-card-with-actions"},{"type":"extras","title":"Custom Header Components - Table Options API Reference","doc":"Override the default table header:\n\n```elixir\ndef table_options do\n  %{\n    components: %{\n      header: &custom_header/1\n    }\n  }\nend\n\ndefp custom_header(assigns) do\n  ~H\"\"\"\n   \n     \n       Product Inventory \n       \n         \n          Bulk Actions\n         \n         \n          Import\n         \n       \n     \n    \n     \n      Total Products: <%= @total_count %> | \n      Active: <%= @active_count %> | \n      Low Stock: <%= @low_stock_count %>\n     \n   \n  \"\"\"\nend\n```","ref":"table-options.html#custom-header-components"},{"type":"extras","title":"Performance Considerations - Table Options API Reference","doc":"","ref":"table-options.html#performance-considerations"},{"type":"extras","title":"Large Datasets - Table Options API Reference","doc":"```elixir\ndef table_options do\n  %{\n    pagination: %{\n      enabled: true,\n      sizes: [25, 50]        # Smaller page sizes\n    },\n    search: %{\n      debounce: 500          # Longer debounce\n    }\n  }\nend\n```","ref":"table-options.html#large-datasets"},{"type":"extras","title":"Real-time Updates - Table Options API Reference","doc":"```elixir\ndef table_options do\n  %{\n    pagination: %{\n      enabled: true,\n      sizes: [10, 20]        # Smaller pages for faster updates\n    },\n    search: %{\n      debounce: 200          # Quicker search response\n    }\n  }\nend\n```","ref":"table-options.html#real-time-updates"},{"type":"extras","title":"Troubleshooting - Table Options API Reference","doc":"","ref":"table-options.html#troubleshooting"},{"type":"extras","title":"Common Issues - Table Options API Reference","doc":"**Table not respecting options:**\n- Ensure `table_options/0` is properly defined\n- Check for syntax errors in the options map\n- Verify configuration hierarchy (table options override app config)\n\n**Card mode not working:**\n- Confirm `card_component` function is defined\n- Check that the component function accepts proper assigns\n- Verify card grid configuration\n\n**Pagination not showing:**\n- Ensure `pagination.enabled` is `true`\n- Check that you have more records than `default_size`\n- Verify page size options are reasonable\n\n**Exports not available:**\n- Check `exports.enabled` is `true`\n- Verify Oban configuration for background processing\n- Ensure export formats are properly specified","ref":"table-options.html#common-issues"},{"type":"extras","title":"Debugging Configuration - Table Options API Reference","doc":"```elixir\n# Check effective configuration\ndef mount(_params, _session, socket) do\n  effective_config = get_merged_table_options()\n  IO.inspect(effective_config, label: \"Table Config\")\n  {:ok, socket}\nend\n\n# Verify specific options\ndef table_options do\n  config = %{\n    pagination: %{enabled: true, sizes: [10, 25]},\n    sorting: %{default_sort: [name: :asc]}\n  }\n  \n  IO.inspect(config, label: \"Table Options\")\n  config\nend\n```","ref":"table-options.html#debugging-configuration"},{"type":"extras","title":"Simple Table Examples","doc":"# Simple Table Examples\n\nThis guide shows how to create basic LiveTable implementations for common single-schema use cases.","ref":"simple-table.html"},{"type":"extras","title":"Basic Product Table - Simple Table Examples","doc":"A simple product listing with minimal configuration:\n\n```elixir\n# lib/your_app_web/live/product_live/index.ex\ndefmodule YourAppWeb.ProductLive.Index do\n  use YourAppWeb, :live_view\n  use LiveTable.LiveResource, schema: YourApp.Product\n\n  def fields do\n    [\n      id: %{label: \"ID\", sortable: true},\n      name: %{label: \"Product Name\", sortable: true, searchable: true},\n      price: %{label: \"Price\", sortable: true},\n      stock_quantity: %{label: \"Stock\", sortable: true},\n      active: %{label: \"Status\", sortable: true, renderer: &render_status/1}\n    ]\n  end\n\n  def filters do\n    [\n      active: Boolean.new(:active, \"active\", %{\n        label: \"Active Products Only\",\n        condition: dynamic([p], p.active == true)\n      }),\n      \n      price_range: Range.new(:price, \"price_range\", %{\n        type: :number,\n        label: \"Price Range\",\n        min: 0,\n        max: 1000\n      })\n    ]\n  end\n\n  defp render_status(active) do\n    assigns = %{active: active}\n    ~H\"\"\"\n     \n      <%= if @active, do: \"Active\", else: \"Inactive\" %>\n     \n    \"\"\"\n  end\nend\n```\n\n```elixir\n# lib/your_app_web/live/product_live/index.html.heex\n \n   Products \n  \n  <.live_table\n    fields={fields()}\n    filters={filters()}\n    options={@options}\n    streams={@streams}\n  />\n \n```","ref":"simple-table.html#basic-product-table"},{"type":"extras","title":"User Management Table - Simple Table Examples","doc":"A user listing with role-based rendering:\n\n```elixir\n# lib/your_app_web/live/user_live/index.ex\ndefmodule YourAppWeb.UserLive.Index do\n  use YourAppWeb, :live_view\n  use LiveTable.LiveResource, schema: YourApp.User\n\n  def fields do\n    [\n      id: %{label: \"ID\", sortable: true},\n      email: %{label: \"Email\", sortable: true, searchable: true},\n      name: %{label: \"Name\", sortable: true, searchable: true},\n      role: %{label: \"Role\", sortable: true, renderer: &render_role/1},\n      inserted_at: %{label: \"Joined\", sortable: true, renderer: &render_date/1},\n      last_sign_in_at: %{label: \"Last Active\", sortable: true, renderer: &render_last_active/1}\n    ]\n  end\n\n  def filters do\n    [\n      active: Boolean.new(:active, \"active\", %{\n        label: \"Active Users Only\",\n        condition: dynamic([u], u.active == true)\n      }),\n      \n      role: Select.new(:role, \"role\", %{\n        label: \"User Role\",\n        options: [\n          %{label: \"Admin\", value: [\"admin\"]},\n          %{label: \"Manager\", value: [\"manager\"]},\n          %{label: \"User\", value: [\"user\"]}\n        ]\n      }),\n      \n      signup_date: Range.new(:inserted_at, \"signup_range\", %{\n        type: :date,\n        label: \"Registration Date\",\n        min: ~D[2020-01-01],\n        max: Date.utc_today()\n      })\n    ]\n  end\n\n  defp render_role(role) do\n    assigns = %{role: role}\n    ~H\"\"\"\n      \"bg-purple-100 text-purple-700\"\n        \"manager\" -> \"bg-blue-100 text-blue-700\"\n        \"user\" -> \"bg-gray-100 text-gray-700\"\n      end\n    ]}>\n      <%= @role %>\n     \n    \"\"\"\n  end\n\n  defp render_date(date) do\n    assigns = %{date: date}\n    ~H\"\"\"\n     \n      <%= Calendar.strftime(@date, \"%b %d, %Y\") %>\n     \n    \"\"\"\n  end\n\n  defp render_last_active(nil) do\n    assigns = %{}\n    ~H\"\"\"\n     Never \n    \"\"\"\n  end\n\n  defp render_last_active(datetime) do\n    assigns = %{datetime: datetime}\n    ~H\"\"\"\n     \n      <%= Calendar.strftime(@datetime, \"%b %d, %Y at %I:%M %p\") %>\n     \n    \"\"\"\n  end\nend\n```","ref":"simple-table.html#user-management-table"},{"type":"extras","title":"Order History Table - Simple Table Examples","doc":"An order listing with status tracking:\n\n```elixir\n# lib/your_app_web/live/order_live/index.ex\ndefmodule YourAppWeb.OrderLive.Index do\n  use YourAppWeb, :live_view\n  use LiveTable.LiveResource, schema: YourApp.Order\n\n  def fields do\n    [\n      id: %{label: \"Order #\", sortable: true},\n      customer_email: %{label: \"Customer\", sortable: true, searchable: true},\n      total_amount: %{label: \"Total\", sortable: true, renderer: &render_currency/1},\n      status: %{label: \"Status\", sortable: true, renderer: &render_order_status/1},\n      inserted_at: %{label: \"Order Date\", sortable: true, renderer: &render_date/1},\n      actions: %{label: \"Actions\", sortable: false, renderer: &render_actions/2}\n    ]\n  end\n\n  def filters do\n    [\n      status: Select.new(:status, \"status\", %{\n        label: \"Order Status\",\n        options: [\n          %{label: \"Pending\", value: [\"pending\"]},\n          %{label: \"Processing\", value: [\"processing\"]},\n          %{label: \"Shipped\", value: [\"shipped\"]},\n          %{label: \"Delivered\", value: [\"delivered\"]},\n          %{label: \"Cancelled\", value: [\"cancelled\"]}\n        ]\n      }),\n      \n      order_total: Range.new(:total_amount, \"total_range\", %{\n        type: :number,\n        label: \"Order Total\",\n        min: 0,\n        max: 5000\n      }),\n      \n      recent_orders: Boolean.new(:inserted_at, \"recent\", %{\n        label: \"Last 30 Days\",\n        condition: dynamic([o], o.inserted_at >= ago(30, \"day\"))\n      })\n    ]\n  end\n\n  def table_options do\n    %{\n      pagination: %{\n        enabled: true,\n        sizes: [10, 25, 50],\n        default_size: 25\n      },\n      sorting: %{\n        default_sort: [inserted_at: :desc]\n      },\n      exports: %{\n        enabled: true,\n        formats: [:csv, :pdf]\n      }\n    }\n  end\n\n  defp render_currency(amount) do\n    assigns = %{amount: amount}\n    ~H\"\"\"\n     \n      $<%= :erlang.float_to_binary(@amount, decimals: 2) %>\n     \n    \"\"\"\n  end\n\n  defp render_order_status(status) do\n    assigns = %{status: status}\n    ~H\"\"\"\n     \n        \"bg-yellow-400\"\n          \"processing\" -> \"bg-blue-400\"\n          \"shipped\" -> \"bg-purple-400\"\n          \"delivered\" -> \"bg-green-400\"\n          \"cancelled\" -> \"bg-red-400\"\n        end\n      ]}> \n       <%= @status %> \n     \n    \"\"\"\n  end\n\n  defp render_date(date) do\n    assigns = %{date: date}\n    ~H\"\"\"\n     \n      <%= Calendar.strftime(@date, \"%b %d, %Y\") %>\n     \n    \"\"\"\n  end\n\n  defp render_actions(_value, record) do\n    assigns = %{record: record}\n    ~H\"\"\"\n     \n      <.link \n        navigate={~p\"/orders/#{@record.id}\"} \n        class=\"text-blue-600 hover:text-blue-800 text-sm font-medium\"\n      >\n        View\n      </.link>\n      <%= if @record.status in [\"pending\", \"processing\"] do %>\n         \n          Cancel\n         \n      <% end %>\n     \n    \"\"\"\n  end\n\n  def handle_event(\"cancel_order\", %{\"id\" => id}, socket) do\n    order = YourApp.Orders.get_order!(id)\n    case YourApp.Orders.cancel_order(order) do\n      {:ok, _order} ->\n        {:noreply, put_flash(socket, :info, \"Order cancelled successfully\")}\n      {:error, _changeset} ->\n        {:noreply, put_flash(socket, :error, \"Unable to cancel order\")}\n    end\n  end\nend\n```","ref":"simple-table.html#order-history-table"},{"type":"extras","title":"Inventory Management Table - Simple Table Examples","doc":"A product inventory table with stock alerts:\n\n```elixir\n# lib/your_app_web/live/inventory_live/index.ex\ndefmodule YourAppWeb.InventoryLive.Index do\n  use YourAppWeb, :live_view\n  use LiveTable.LiveResource, schema: YourApp.Product\n\n  def fields do\n    [\n      sku: %{label: \"SKU\", sortable: true, searchable: true},\n      name: %{label: \"Product\", sortable: true, searchable: true},\n      stock_quantity: %{label: \"Current Stock\", sortable: true, renderer: &render_stock/2},\n      reorder_point: %{label: \"Reorder Point\", sortable: true},\n      last_restocked: %{label: \"Last Restocked\", sortable: true, renderer: &render_date/1},\n      supplier_name: %{label: \"Supplier\", sortable: true, searchable: true},\n      actions: %{label: \"Actions\", sortable: false, renderer: &render_inventory_actions/2}\n    ]\n  end\n\n  def filters do\n    [\n      low_stock: Boolean.new(:stock_quantity, \"low_stock\", %{\n        label: \"Low Stock Alert\",\n        condition: dynamic([p], p.stock_quantity <= p.reorder_point)\n      }),\n      \n      out_of_stock: Boolean.new(:stock_quantity, \"out_of_stock\", %{\n        label: \"Out of Stock\",\n        condition: dynamic([p], p.stock_quantity == 0)\n      }),\n      \n      stock_range: Range.new(:stock_quantity, \"stock_range\", %{\n        type: :number,\n        label: \"Stock Quantity\",\n        min: 0,\n        max: 1000\n      }),\n      \n      needs_reorder: Boolean.new(:stock_quantity, \"needs_reorder\", %{\n        label: \"Needs Reorder\",\n        condition: dynamic([p], p.stock_quantity <= p.reorder_point and p.stock_quantity > 0)\n      })\n    ]\n  end\n\n  def table_options do\n    %{\n      pagination: %{\n        enabled: true,\n        sizes: [20, 50, 100],\n        default_size: 50\n      },\n      sorting: %{\n        default_sort: [stock_quantity: :asc]  # Show low stock first\n      },\n      exports: %{\n        enabled: true,\n        formats: [:csv]\n      }\n    }\n  end\n\n  defp render_stock(stock_quantity, record) do\n    assigns = %{stock: stock_quantity, record: record}\n    ~H\"\"\"\n     \n        \"text-red-600\"\n          @stock <= @record.reorder_point -> \"text-orange-600\"\n          @stock <= @record.reorder_point * 2 -> \"text-yellow-600\"\n          true -> \"text-green-600\"\n        end\n      ]}>\n        <%= @stock %>\n       \n      \n      <%= cond do %>\n        <% @stock == 0 -> %>\n           OUT \n        <% @stock <= @record.reorder_point -> %>\n           LOW \n        <% @stock <= @record.reorder_point * 2 -> %>\n           WATCH \n        <% true -> %>\n           OK \n      <% end %>\n     \n    \"\"\"\n  end\n\n  defp render_date(nil) do\n    assigns = %{}\n    ~H\"\"\"\n     Never \n    \"\"\"\n  end\n\n  defp render_date(date) do\n    assigns = %{date: date}\n    ~H\"\"\"\n     \n      <%= Calendar.strftime(@date, \"%b %d, %Y\") %>\n     \n    \"\"\"\n  end\n\n  defp render_inventory_actions(_value, record) do\n    assigns = %{record: record}\n    ~H\"\"\"\n     \n       \n        Restock\n       \n      \n      <%= if @record.stock_quantity <= @record.reorder_point do %>\n         \n          Auto Reorder\n         \n      <% end %>\n      \n      <.link \n        navigate={~p\"/inventory/#{@record.id}/adjust\"} \n        class=\"text-gray-600 hover:text-gray-800 text-sm\"\n      >\n        Adjust\n      </.link>\n     \n    \"\"\"\n  end\n\n  def handle_event(\"restock\", %{\"id\" => id}, socket) do\n    # Handle manual restock\n    {:noreply, redirect(socket, to: ~p\"/inventory/#{id}/restock\")}\n  end\n\n  def handle_event(\"auto_reorder\", %{\"id\" => id}, socket) do\n    product = YourApp.Inventory.get_product!(id)\n    case YourApp.Inventory.create_reorder(product) do\n      {:ok, _reorder} ->\n        {:noreply, put_flash(socket, :info, \"Reorder created successfully\")}\n      {:error, _changeset} ->\n        {:noreply, put_flash(socket, :error, \"Failed to create reorder\")}\n    end\n  end\nend\n```","ref":"simple-table.html#inventory-management-table"},{"type":"extras","title":"Simple Read-Only Table - Simple Table Examples","doc":"A minimal table for displaying reference data:\n\n```elixir\n# lib/your_app_web/live/category_live/index.ex\ndefmodule YourAppWeb.CategoryLive.Index do\n  use YourAppWeb, :live_view\n  use LiveTable.LiveResource, schema: YourApp.Category\n\n  def fields do\n    [\n      id: %{label: \"ID\", sortable: true},\n      name: %{label: \"Category Name\", sortable: true, searchable: true},\n      description: %{label: \"Description\", sortable: false, searchable: true},\n      product_count: %{label: \"Products\", sortable: true, renderer: &render_count/1},\n      active: %{label: \"Status\", sortable: true, renderer: &render_active_status/1}\n    ]\n  end\n\n  def filters do\n    [\n      active: Boolean.new(:active, \"active\", %{\n        label: \"Active Categories Only\",\n        condition: dynamic([c], c.active == true)\n      })\n    ]\n  end\n\n  def table_options do\n    %{\n      pagination: %{enabled: false},  # Show all categories\n      sorting: %{default_sort: [name: :asc]},\n      exports: %{enabled: false},     # No exports needed\n      search: %{\n        enabled: true,\n        placeholder: \"Search categories...\"\n      }\n    }\n  end\n\n  defp render_count(count) do\n    assigns = %{count: count}\n    ~H\"\"\"\n     \n      <%= @count %>\n     \n    \"\"\"\n  end\n\n  defp render_active_status(active) do\n    assigns = %{active: active}\n    ~H\"\"\"\n      \n    \"\"\"\n  end\nend\n```","ref":"simple-table.html#simple-read-only-table"},{"type":"extras","title":"Key Patterns for Simple Tables - Simple Table Examples","doc":"","ref":"simple-table.html#key-patterns-for-simple-tables"},{"type":"extras","title":"1. Minimal Setup - Simple Table Examples","doc":"- Use the LiveResource module with `schema:` parameter\n- Define basic fields with labels and sortability\n- Add simple filters for common use cases","ref":"simple-table.html#1-minimal-setup"},{"type":"extras","title":"2. Custom Renderers - Simple Table Examples","doc":"- Use `renderer: &function/1` for simple formatting\n- Use `renderer: &function/2` when you need access to the full record\n- Keep renderers focused and lightweight","ref":"simple-table.html#2-custom-renderers"},{"type":"extras","title":"3. Practical Filters - Simple Table Examples","doc":"- Boolean filters for status toggles\n- Range filters for numeric and date fields\n- Select filters for enumerated values","ref":"simple-table.html#3-practical-filters"},{"type":"extras","title":"4. Sensible Defaults - Simple Table Examples","doc":"- Enable pagination for large datasets\n- Set reasonable default sort orders\n- Configure exports based on user needs","ref":"simple-table.html#4-sensible-defaults"},{"type":"extras","title":"5. Action Columns - Simple Table Examples","doc":"- Use `sortable: false` for action columns\n- Implement event handlers for interactive actions\n- Provide clear user feedback for actions\n\nThese examples demonstrate how LiveTable can handle common table requirements with minimal code while providing rich functionality for users.","ref":"simple-table.html#5-action-columns"},{"type":"extras","title":"Complex Query Examples","doc":"# Complex Query Examples\n\nThis guide demonstrates how to use LiveTable with custom queries, joins, computed fields, and advanced data transformations.","ref":"complex-queries.html"},{"type":"extras","title":"Order Report with Customer Details - Complex Query Examples","doc":"A comprehensive order report joining multiple tables:\n\n```elixir\n# lib/your_app_web/live/order_report_live/index.ex\ndefmodule YourAppWeb.OrderReportLive.Index do\n  use YourAppWeb, :live_view\n  use LiveTable.LiveResource\n\n  def mount(_params, _session, socket) do\n    socket = assign(socket, :data_provider, {YourApp.Reports, :order_details, []})\n    {:ok, socket}\n  end\n\n  def fields do\n    [\n      order_id: %{label: \"Order #\", sortable: true},\n      customer_name: %{label: \"Customer\", sortable: true, searchable: true},\n      customer_email: %{label: \"Email\", sortable: true, searchable: true},\n      product_count: %{label: \"Items\", sortable: true},\n      total_amount: %{label: \"Total\", sortable: true, renderer: &render_currency/1},\n      order_date: %{label: \"Order Date\", sortable: true, renderer: &render_date/1},\n      shipping_method: %{\n        label: \"Shipping\", \n        sortable: true,\n        assoc: {:shipping, :method}\n      },\n      status: %{label: \"Status\", sortable: true, renderer: &render_status/1},\n      days_since_order: %{label: \"Age (Days)\", sortable: true, renderer: &render_days/1}\n    ]\n  end\n\n  def filters do\n    [\n      status: Select.new({:orders, :status}, \"status\", %{\n        label: \"Order Status\",\n        options: [\n          %{label: \"Pending\", value: [\"pending\"]},\n          %{label: \"Processing\", value: [\"processing\"]},\n          %{label: \"Shipped\", value: [\"shipped\"]},\n          %{label: \"Delivered\", value: [\"delivered\"]}\n        ]\n      }),\n      \n      order_value: Range.new(:total_amount, \"order_value\", %{\n        type: :number,\n        label: \"Order Value\",\n        min: 0,\n        max: 10000,\n        step: 100\n      }),\n      \n      customer_type: Select.new({:customers, :type}, \"customer_type\", %{\n        label: \"Customer Type\",\n        options: [\n          %{label: \"Individual\", value: [\"individual\"]},\n          %{label: \"Business\", value: [\"business\"]}\n        ]\n      }),\n      \n      recent_orders: Boolean.new(:order_date, \"recent\", %{\n        label: \"Last 30 Days\",\n        condition: dynamic([o, customers: c, shipping: s], o.inserted_at >= ago(30, \"day\"))\n      })\n    ]\n  end\n\n  defp render_currency(amount) do\n    assigns = %{amount: amount}\n    ~H\"\"\"\n     $<%= :erlang.float_to_binary(@amount, decimals: 2) %> \n    \"\"\"\n  end\n\n  defp render_date(date) do\n    assigns = %{date: date}\n    ~H\"\"\"\n     <%= Calendar.strftime(@date, \"%b %d, %Y\") %> \n    \"\"\"\n  end\n\n  defp render_status(status) do\n    assigns = %{status: status}\n    ~H\"\"\"\n      \"bg-yellow-100 text-yellow-700\"\n        \"processing\" -> \"bg-blue-100 text-blue-700\"\n        \"shipped\" -> \"bg-purple-100 text-purple-700\"\n        \"delivered\" -> \"bg-green-100 text-green-700\"\n      end\n    ]}>\n      <%= String.capitalize(@status) %>\n     \n    \"\"\"\n  end\n\n  defp render_days(days) do\n    assigns = %{days: days}\n    ~H\"\"\"\n      30, do: \"text-red-600\", else: \"text-gray-600\")\n    ]}>\n      <%= @days %> days\n     \n    \"\"\"\n  end\nend\n```\n\nThe corresponding context function:\n\n```elixir\n# lib/your_app/reports.ex\ndefmodule YourApp.Reports do\n  import Ecto.Query\n  alias YourApp.Repo\n\n  def order_details do\n    from o in YourApp.Order,\n      join: c in YourApp.Customer, on: o.customer_id == c.id, as: :customers,\n      join: s in YourApp.ShippingMethod, on: o.shipping_method_id == s.id, as: :shipping,\n      left_join: oi in YourApp.OrderItem, on: oi.order_id == o.id,\n      group_by: [o.id, c.id, s.id],\n      select: %{\n        order_id: o.id,\n        customer_name: fragment(\"? || ' ' || ?\", c.first_name, c.last_name),\n        customer_email: c.email,\n        product_count: count(oi.id),\n        total_amount: o.total_amount,\n        order_date: o.inserted_at,\n        shipping_method: s.name,\n        status: o.status,\n        days_since_order: fragment(\"DATE_PART('day', NOW() - ?)\", o.inserted_at)\n      }\n  end\nend\n```","ref":"complex-queries.html#order-report-with-customer-details"},{"type":"extras","title":"Product Analytics with Sales Data - Complex Query Examples","doc":"Advanced product performance analytics:\n\n```elixir\n# lib/your_app_web/live/product_analytics_live/index.ex\ndefmodule YourAppWeb.ProductAnalyticsLive.Index do\n  use YourAppWeb, :live_view\n  use LiveTable.LiveResource\n\n  def mount(%{\"period\" => period}, _session, socket) do\n    data_provider = {YourApp.Analytics, :product_performance, [period]}\n    socket = assign(socket, :data_provider, data_provider)\n    {:ok, socket}\n  end\n\n  def mount(_params, _session, socket) do\n    mount(%{\"period\" => \"30\"}, nil, socket)\n  end\n\n  def fields do\n    [\n      product_name: %{label: \"Product\", sortable: true, searchable: true},\n      sku: %{label: \"SKU\", sortable: true, searchable: true},\n      category_name: %{\n        label: \"Category\", \n        sortable: true,\n        assoc: {:categories, :name}\n      },\n      units_sold: %{label: \"Units Sold\", sortable: true},\n      revenue: %{label: \"Revenue\", sortable: true, renderer: &render_currency/1},\n      avg_order_value: %{label: \"AOV\", sortable: true, renderer: &render_currency/1},\n      profit_margin: %{label: \"Margin %\", sortable: true, renderer: &render_percentage/1},\n      conversion_rate: %{label: \"Conv. Rate\", sortable: true, renderer: &render_percentage/1},\n      stock_turns: %{label: \"Stock Turns\", sortable: true, renderer: &render_decimal/1},\n      performance_score: %{label: \"Score\", sortable: true, renderer: &render_score/1}\n    ]\n  end\n\n  def filters do\n    [\n      category: Select.new({:categories, :name}, \"category\", %{\n        label: \"Category\",\n        options_source: {YourApp.Catalog, :list_categories, []}\n      }),\n      \n      revenue_range: Range.new(:revenue, \"revenue_range\", %{\n        type: :number,\n        label: \"Revenue Range\",\n        min: 0,\n        max: 100000,\n        step: 1000\n      }),\n      \n      high_performers: Boolean.new(:performance_score, \"high_performers\", %{\n        label: \"High Performers (Score > 80)\",\n        condition: dynamic([p, categories: c], p.performance_score > 80)\n      }),\n      \n      low_margin: Boolean.new(:profit_margin, \"low_margin\", %{\n        label: \"Low Margin (< 20%)\",\n        condition: dynamic([p, categories: c], p.profit_margin < 20)\n      })\n    ]\n  end\n\n  def table_options do\n    %{\n      pagination: %{\n        enabled: true,\n        sizes: [25, 50, 100],\n        default_size: 50\n      },\n      sorting: %{\n        default_sort: [performance_score: :desc, revenue: :desc]\n      },\n      exports: %{\n        enabled: true,\n        formats: [:csv, :pdf]\n      }\n    }\n  end\n\n  defp render_currency(amount) do\n    assigns = %{amount: amount}\n    ~H\"\"\"\n     $<%= :erlang.float_to_binary(@amount, decimals: 2) %> \n    \"\"\"\n  end\n\n  defp render_percentage(value) do\n    assigns = %{value: value}\n    ~H\"\"\"\n     = 20, do: \"text-green-600\", else: \"text-red-600\")\n    ]}>\n      <%= :erlang.float_to_binary(@value, decimals: 1) %>%\n     \n    \"\"\"\n  end\n\n  defp render_decimal(value) do\n    assigns = %{value: value}\n    ~H\"\"\"\n     <%= :erlang.float_to_binary(@value, decimals: 2) %> \n    \"\"\"\n  end\n\n  defp render_score(score) do\n    assigns = %{score: score}\n    ~H\"\"\"\n     \n       \n         = 80 -> \"bg-green-500\"\n              @score >= 60 -> \"bg-yellow-500\"\n              true -> \"bg-red-500\"\n            end\n          ]}\n          style={\"width: #{@score}%\"}\n        > \n       \n       <%= round(@score) %> \n     \n    \"\"\"\n  end\nend\n```\n\nComplex analytics context:\n\n```elixir\n# lib/your_app/analytics.ex\ndefmodule YourApp.Analytics do\n  import Ecto.Query\n  alias YourApp.Repo\n\n  def product_performance(days \\\\ 30) do\n    cutoff_date = Date.add(Date.utc_today(), -days)\n    \n    from p in YourApp.Product,\n      join: c in YourApp.Category, on: p.category_id == c.id, as: :categories,\n      left_join: oi in YourApp.OrderItem, on: oi.product_id == p.id,\n      left_join: o in YourApp.Order, on: oi.order_id == o.id and o.inserted_at >= ^cutoff_date,\n      left_join: v in YourApp.ProductView, on: v.product_id == p.id and v.inserted_at >= ^cutoff_date,\n      group_by: [p.id, c.id],\n      select: %{\n        product_name: p.name,\n        sku: p.sku,\n        category_name: c.name,\n        units_sold: coalesce(sum(oi.quantity), 0),\n        revenue: coalesce(sum(fragment(\"? * ?\", oi.quantity, oi.unit_price)), 0.0),\n        avg_order_value: fragment(\"CASE WHEN COUNT(DISTINCT ?) > 0 THEN ? / COUNT(DISTINCT ?) ELSE 0 END\", \n                                 o.id, coalesce(sum(fragment(\"? * ?\", oi.quantity, oi.unit_price)), 0.0), o.id),\n        profit_margin: fragment(\"CASE WHEN ? > 0 THEN ((? - ?) / ?) * 100 ELSE 0 END\",\n                               p.price, p.price, p.cost_price, p.price),\n        conversion_rate: fragment(\"CASE WHEN COUNT(?) > 0 THEN (COUNT(DISTINCT ?) * 100.0 / COUNT(?)) ELSE 0 END\",\n                                 v.id, o.id, v.id),\n        stock_turns: fragment(\"CASE WHEN ? > 0 THEN ? / ? ELSE 0 END\",\n                             p.stock_quantity, coalesce(sum(oi.quantity), 0), p.stock_quantity),\n        performance_score: fragment(\"\"\"\n          LEAST(100, GREATEST(0, \n            (CASE WHEN ? > 0 THEN ? / ? * 20 ELSE 0 END) +\n            (CASE WHEN ? > 0 THEN LEAST(20, ? / 1000 * 20) ELSE 0 END) +\n            (CASE WHEN ? > 0 THEN ? / 50 * 20 ELSE 0 END) +\n            (CASE WHEN ? > 0 THEN ? / 5 * 20 ELSE 0 END) +\n            (CASE WHEN ? > 10 THEN 20 ELSE ? / 10 * 20 END)\n          ))\n        \"\"\",\n        # Stock turns score (0-20)\n        p.stock_quantity, coalesce(sum(oi.quantity), 0), p.stock_quantity,\n        # Revenue score (0-20)\n        coalesce(sum(fragment(\"? * ?\", oi.quantity, oi.unit_price)), 0.0),\n        coalesce(sum(fragment(\"? * ?\", oi.quantity, oi.unit_price)), 0.0),\n        # Margin score (0-20)\n        fragment(\"((? - ?) / ?) * 100\", p.price, p.cost_price, p.price),\n        fragment(\"((? - ?) / ?) * 100\", p.price, p.cost_price, p.price),\n        # Conversion score (0-20)\n        fragment(\"CASE WHEN COUNT(?) > 0 THEN (COUNT(DISTINCT ?) * 100.0 / COUNT(?)) ELSE 0 END\",\n                 v.id, o.id, v.id),\n        fragment(\"CASE WHEN COUNT(?) > 0 THEN (COUNT(DISTINCT ?) * 100.0 / COUNT(?)) ELSE 0 END\",\n                 v.id, o.id, v.id),\n        # Units sold score (0-20)\n        coalesce(sum(oi.quantity), 0), coalesce(sum(oi.quantity), 0))\n      }\n  end\nend\n```","ref":"complex-queries.html#product-analytics-with-sales-data"},{"type":"extras","title":"Multi-Table Inventory Report - Complex Query Examples","doc":"Comprehensive inventory analysis across suppliers and warehouses:\n\n```elixir\n# lib/your_app_web/live/inventory_report_live/index.ex\ndefmodule YourAppWeb.InventoryReportLive.Index do\n  use YourAppWeb, :live_view\n  use LiveTable.LiveResource\n\n  def mount(_params, _session, socket) do\n    socket = assign(socket, :data_provider, {YourApp.Inventory, :comprehensive_report, []})\n    {:ok, socket}\n  end\n\n  def fields do\n    [\n      product_name: %{label: \"Product\", sortable: true, searchable: true},\n      sku: %{label: \"SKU\", sortable: true, searchable: true},\n      supplier_name: %{\n        label: \"Supplier\", \n        sortable: true,\n        assoc: {:suppliers, :name}\n      },\n      warehouse_location: %{\n        label: \"Warehouse\", \n        sortable: true,\n        assoc: {:warehouses, :location}\n      },\n      current_stock: %{label: \"Current Stock\", sortable: true, renderer: &render_stock/2},\n      reserved_stock: %{label: \"Reserved\", sortable: true},\n      available_stock: %{label: \"Available\", sortable: true, renderer: &render_available/1},\n      reorder_point: %{label: \"Reorder Point\", sortable: true},\n      max_stock: %{label: \"Max Stock\", sortable: true},\n      avg_daily_usage: %{label: \"Daily Usage\", sortable: true, renderer: &render_usage/1},\n      days_of_supply: %{label: \"Days Supply\", sortable: true, renderer: &render_days_supply/2},\n      last_movement: %{label: \"Last Movement\", sortable: true, renderer: &render_date/1},\n      status: %{label: \"Status\", sortable: false, renderer: &render_inventory_status/2}\n    ]\n  end\n\n  def filters do\n    [\n      supplier: Select.new({:suppliers, :name}, \"supplier\", %{\n        label: \"Supplier\",\n        options_source: {YourApp.Suppliers, :list_active, []}\n      }),\n      \n      warehouse: Select.new({:warehouses, :location}, \"warehouse\", %{\n        label: \"Warehouse\",\n        options_source: {YourApp.Warehouses, :list_locations, []}\n      }),\n      \n      stock_status: Select.new(:current_stock, \"stock_status\", %{\n        label: \"Stock Status\",\n        options: [\n          %{label: \"Out of Stock\", value: [\"out_of_stock\"]},\n          %{label: \"Low Stock\", value: [\"low_stock\"]},\n          %{label: \"Normal\", value: [\"normal\"]},\n          %{label: \"Overstock\", value: [\"overstock\"]}\n        ]\n      }),\n      \n      needs_reorder: Boolean.new(:current_stock, \"needs_reorder\", %{\n        label: \"Needs Reorder\",\n        condition: dynamic([p, suppliers: s, warehouses: w], \n          p.current_stock <= p.reorder_point and p.current_stock > 0)\n      }),\n      \n      slow_moving: Boolean.new(:avg_daily_usage, \"slow_moving\", %{\n        label: \"Slow Moving (< 1/day)\",\n        condition: dynamic([p, suppliers: s, warehouses: w], p.avg_daily_usage < 1.0)\n      })\n    ]\n  end\n\n  def table_options do\n    %{\n      pagination: %{\n        enabled: true,\n        sizes: [50, 100, 200],\n        default_size: 100\n      },\n      sorting: %{\n        default_sort: [days_of_supply: :asc, current_stock: :asc]\n      },\n      exports: %{\n        enabled: true,\n        formats: [:csv, :pdf]\n      }\n    }\n  end\n\n  defp render_stock(stock, record) do\n    assigns = %{stock: stock, record: record}\n    ~H\"\"\"\n     \n        \"text-red-600\"\n          @stock <= @record.reorder_point -> \"text-orange-600\"\n          @stock >= @record.max_stock -> \"text-purple-600\"\n          true -> \"text-green-600\"\n        end\n      ]}>\n        <%= @stock %>\n       \n     \n    \"\"\"\n  end\n\n  defp render_available(available) do\n    assigns = %{available: available}\n    ~H\"\"\"\n     \n      <%= @available %>\n     \n    \"\"\"\n  end\n\n  defp render_usage(usage) do\n    assigns = %{usage: usage}\n    ~H\"\"\"\n     \n      <%= :erlang.float_to_binary(@usage, decimals: 1) %>/day\n     \n    \"\"\"\n  end\n\n  defp render_days_supply(days, record) do\n    assigns = %{days: days, record: record}\n    ~H\"\"\"\n      \"text-red-600\"\n        @days <= 14 -> \"text-orange-600\"\n        @days >= 90 -> \"text-purple-600\"\n        true -> \"text-green-600\"\n      end\n    ]}>\n      <%= round(@days) %> days\n     \n    \"\"\"\n  end\n\n  defp render_date(nil) do\n    assigns = %{}\n    ~H\"\"\"\n     No movement \n    \"\"\"\n  end\n\n  defp render_date(date) do\n    assigns = %{date: date}\n    ~H\"\"\"\n     \n      <%= Calendar.strftime(@date, \"%b %d\") %>\n     \n    \"\"\"\n  end\n\n  defp render_inventory_status(_value, record) do\n    assigns = %{record: record}\n    ~H\"\"\"\n     \n      <%= cond do %>\n        <% @record.current_stock == 0 -> %>\n           OUT OF STOCK \n        <% @record.current_stock <= @record.reorder_point -> %>\n           LOW STOCK \n        <% @record.current_stock >= @record.max_stock -> %>\n           OVERSTOCK \n        <% @record.days_of_supply <= 7 -> %>\n           URGENT \n        <% true -> %>\n           OK \n      <% end %>\n      \n      <%= if @record.avg_daily_usage < 1.0 do %>\n         SLOW \n      <% end %>\n     \n    \"\"\"\n  end\nend\n```\n\nThe comprehensive inventory context:\n\n```elixir\n# lib/your_app/inventory.ex\ndefmodule YourApp.Inventory do\n  import Ecto.Query\n  alias YourApp.Repo\n\n  def comprehensive_report do\n    from p in YourApp.Product,\n      join: s in YourApp.Supplier, on: p.supplier_id == s.id, as: :suppliers,\n      join: w in YourApp.Warehouse, on: p.warehouse_id == w.id, as: :warehouses,\n      left_join: im in YourApp.InventoryMovement, \n        on: im.product_id == p.id and im.inserted_at >= ago(30, \"day\"),\n      group_by: [p.id, s.id, w.id],\n      select: %{\n        product_name: p.name,\n        sku: p.sku,\n        supplier_name: s.name,\n        warehouse_location: w.location,\n        current_stock: p.stock_quantity,\n        reserved_stock: coalesce(p.reserved_quantity, 0),\n        available_stock: p.stock_quantity - coalesce(p.reserved_quantity, 0),\n        reorder_point: p.reorder_point,\n        max_stock: p.max_stock_level,\n        avg_daily_usage: fragment(\"\"\"\n          CASE \n            WHEN COUNT(?) > 0 THEN ABS(SUM(CASE WHEN ?.movement_type = 'out' THEN ?.quantity ELSE 0 END)) / 30.0\n            ELSE 0 \n          END\n        \"\"\", im.id, im, im),\n        days_of_supply: fragment(\"\"\"\n          CASE \n            WHEN COUNT(?) > 0 AND SUM(CASE WHEN ?.movement_type = 'out' THEN ?.quantity ELSE 0 END) > 0 \n            THEN ? / (ABS(SUM(CASE WHEN ?.movement_type = 'out' THEN ?.quantity ELSE 0 END)) / 30.0)\n            ELSE 999\n          END\n        \"\"\", im.id, im, im, p.stock_quantity, im, im),\n        last_movement: max(im.inserted_at)\n      }\n  end\nend\n```","ref":"complex-queries.html#multi-table-inventory-report"},{"type":"extras","title":"Customer Lifetime Value Analysis - Complex Query Examples","doc":"Complex customer analytics with cohort analysis:\n\n```elixir\n# lib/your_app_web/live/customer_ltv_live/index.ex\ndefmodule YourAppWeb.CustomerLtvLive.Index do\n  use YourAppWeb, :live_view\n  use LiveTable.LiveResource\n\n  def mount(%{\"cohort\" => cohort}, _session, socket) do\n    data_provider = {YourApp.Analytics, :customer_ltv_analysis, [cohort]}\n    socket = assign(socket, :data_provider, data_provider)\n    {:ok, socket}\n  end\n\n  def mount(_params, _session, socket) do\n    mount(%{\"cohort\" => \"2024\"}, nil, socket)\n  end\n\n  def fields do\n    [\n      customer_name: %{label: \"Customer\", sortable: true, searchable: true},\n      email: %{label: \"Email\", sortable: true, searchable: true},\n      first_order_date: %{label: \"First Order\", sortable: true, renderer: &render_date/1},\n      customer_segment: %{\n        label: \"Segment\", \n        sortable: true,\n        assoc: {:segments, :name}\n      },\n      total_orders: %{label: \"Orders\", sortable: true},\n      total_spent: %{label: \"Total Spent\", sortable: true, renderer: &render_currency/1},\n      avg_order_value: %{label: \"AOV\", sortable: true, renderer: &render_currency/1},\n      order_frequency: %{label: \"Frequency (days)\", sortable: true, renderer: &render_frequency/1},\n      ltv_score: %{label: \"LTV Score\", sortable: true, renderer: &render_ltv_score/1},\n      predicted_ltv: %{label: \"Predicted LTV\", sortable: true, renderer: &render_currency/1},\n      churn_risk: %{label: \"Churn Risk\", sortable: true, renderer: &render_churn_risk/1},\n      last_order_date: %{label: \"Last Order\", sortable: true, renderer: &render_last_order/1}\n    ]\n  end\n\n  def filters do\n    [\n      segment: Select.new({:segments, :name}, \"segment\", %{\n        label: \"Customer Segment\",\n        options: [\n          %{label: \"VIP\", value: [\"vip\"]},\n          %{label: \"Regular\", value: [\"regular\"]},\n          %{label: \"New\", value: [\"new\"]},\n          %{label: \"At Risk\", value: [\"at_risk\"]}\n        ]\n      }),\n      \n      ltv_range: Range.new(:predicted_ltv, \"ltv_range\", %{\n        type: :number,\n        label: \"Predicted LTV\",\n        min: 0,\n        max: 50000,\n        step: 1000\n      }),\n      \n      high_value: Boolean.new(:total_spent, \"high_value\", %{\n        label: \"High Value (> $1000)\",\n        condition: dynamic([c, segments: s], c.total_spent > 1000)\n      }),\n      \n      churn_risk: Boolean.new(:churn_risk, \"churn_risk\", %{\n        label: \"High Churn Risk (> 70%)\",\n        condition: dynamic([c, segments: s], c.churn_risk > 70)\n      }),\n      \n      active_customers: Boolean.new(:last_order_date, \"active\", %{\n        label: \"Active (Last 90 Days)\",\n        condition: dynamic([c, segments: s], c.last_order_date >= ago(90, \"day\"))\n      })\n    ]\n  end\n\n  def table_options do\n    %{\n      pagination: %{\n        enabled: true,\n        sizes: [25, 50, 100],\n        default_size: 50\n      },\n      sorting: %{\n        default_sort: [predicted_ltv: :desc, total_spent: :desc]\n      },\n      exports: %{\n        enabled: true,\n        formats: [:csv, :pdf]\n      }\n    }\n  end\n\n  defp render_currency(amount) do\n    assigns = %{amount: amount}\n    ~H\"\"\"\n     $<%= :erlang.float_to_binary(@amount, decimals: 2) %> \n    \"\"\"\n  end\n\n  defp render_date(date) do\n    assigns = %{date: date}\n    ~H\"\"\"\n     <%= Calendar.strftime(@date, \"%b %d, %Y\") %> \n    \"\"\"\n  end\n\n  defp render_frequency(days) do\n    assigns = %{days: days}\n    ~H\"\"\"\n     <%= round(@days) %> days \n    \"\"\"\n  end\n\n  defp render_ltv_score(score) do\n    assigns = %{score: score}\n    ~H\"\"\"\n     \n       \n         = 80 -> \"bg-green-500\"\n              @score >= 60 -> \"bg-yellow-500\"\n              @score >= 40 -> \"bg-orange-500\"\n              true -> \"bg-red-500\"\n            end\n          ]}\n          style={\"width: #{@score}%\"}\n        > \n       \n       <%= round(@score) %> \n     \n    \"\"\"\n  end\n\n  defp render_churn_risk(risk) do\n    assigns = %{risk: risk}\n    ~H\"\"\"\n     = 80 -> \"bg-red-100 text-red-700\"\n        @risk >= 60 -> \"bg-orange-100 text-orange-700\"\n        @risk >= 40 -> \"bg-yellow-100 text-yellow-700\"\n        true -> \"bg-green-100 text-green-700\"\n      end\n    ]}>\n      <%= round(@risk) %>%\n     \n    \"\"\"\n  end\n\n  defp render_last_order(date) do\n    assigns = %{date: date, days_ago: Date.diff(Date.utc_today(), date)}\n    ~H\"\"\"\n     \n       <%= Calendar.strftime(@date, \"%b %d\") %> \n        \"text-green-600\"\n          @days_ago <= 90 -> \"text-yellow-600\"\n          true -> \"text-red-600\"\n        end\n      ]}>\n        <%= @days_ago %> days ago\n       \n     \n    \"\"\"\n  end\nend\n```\n\nAdvanced customer analytics context:\n\n```elixir\n# lib/your_app/analytics.ex (addition to existing module)\ndef customer_ltv_analysis(cohort_year) do\n  cohort_start = Date.new!(String.to_integer(cohort_year), 1, 1)\n  cohort_end = Date.new!(String.to_integer(cohort_year), 12, 31)\n  \n  from c in YourApp.Customer,\n    join: s in YourApp.CustomerSegment, on: c.segment_id == s.id, as: :segments,\n    left_join: o in YourApp.Order, on: o.customer_id == c.id,\n    where: c.inserted_at >= ^cohort_start and c.inserted_at <= ^cohort_end,\n    group_by: [c.id, s.id],\n    select: %{\n      customer_name: fragment(\"? || ' ' || ?\", c.first_name, c.last_name),\n      email: c.email,\n      first_order_date: min(o.inserted_at),\n      customer_segment: s.name,\n      total_orders: count(o.id),\n      total_spent: coalesce(sum(o.total_amount), 0.0),\n      avg_order_value: fragment(\"CASE WHEN COUNT(?) > 0 THEN ? / COUNT(?) ELSE 0 END\",\n                               o.id, coalesce(sum(o.total_amount), 0.0), o.id),\n      order_frequency: fragment(\"\"\"\n        CASE \n          WHEN COUNT(?) > 1 THEN \n            DATE_PART('day', MAX(?) - MIN(?)) / (COUNT(?) - 1)\n          ELSE 0 \n        END\n      \"\"\", o.id, o.inserted_at, o.inserted_at, o.id),\n      ltv_score: fragment(\"\"\"\n        LEAST(100, GREATEST(0,\n          (CASE WHEN ? > 0 THEN LEAST(25, ? / 2000 * 25) ELSE 0 END) +\n          (CASE WHEN COUNT(?) > 0 THEN LEAST(25, COUNT(?) / 10 * 25) ELSE 0 END) +\n          (CASE WHEN ? > 0 THEN LEAST(25, ? / 500 * 25) ELSE 0 END) +\n          (CASE WHEN ? <= 60 THEN 25 ELSE GREATEST(0, 25 - (? - 60) / 10) END)\n        ))\n      \"\"\",\n      # Total spent score (0-25)\n      coalesce(sum(o.total_amount), 0.0), coalesce(sum(o.total_amount), 0.0),\n      # Order frequency score (0-25)\n      o.id, o.id,\n      # AOV score (0-25)\n      fragment(\"CASE WHEN COUNT(?) > 0 THEN ? / COUNT(?) ELSE 0 END\",\n               o.id, coalesce(sum(o.total_amount), 0.0), o.id),\n      fragment(\"CASE WHEN COUNT(?) > 0 THEN ? / COUNT(?) ELSE 0 END\",\n               o.id, coalesce(sum(o.total_amount), 0.0), o.id),\n      # Recency score (0-25, based on days since last order)\n      fragment(\"DATE_PART('day', NOW() - MAX(?))\", o.inserted_at),\n      fragment(\"DATE_PART('day', NOW() - MAX(?))\", o.inserted_at)),\n      predicted_ltv: fragment(\"\"\"\n        CASE \n          WHEN COUNT(?) > 0 THEN \n            ? * (? / ?) * \n            CASE \n              WHEN ? <= 30 THEN 5.0\n              WHEN ? <= 90 THEN 3.0\n              WHEN ? <= 180 THEN 2.0\n              ELSE 1.5\n            END\n          ELSE 0 \n        END\n      \"\"\", \n      o.id, \n      coalesce(sum(o.total_amount), 0.0),\n      fragment(\"CASE WHEN COUNT(?) > 0 THEN ? / COUNT(?) ELSE 0 END\",\n               o.id, coalesce(sum(o.total_amount), 0.0), o.id),\n      fragment(\"CASE WHEN COUNT(?) > 1 THEN DATE_PART('day', MAX(?) - MIN(?)) / (COUNT(?) - 1) ELSE 365 END\",\n               o.id, o.inserted_at, o.inserted_at, o.id),\n      fragment(\"DATE_PART('day', NOW() - MAX(?))\", o.inserted_at),\n      fragment(\"DATE_PART('day', NOW() - MAX(?))\", o.inserted_at),\n      fragment(\"DATE_PART('day', NOW() - MAX(?))\", o.inserted_at)),\n      churn_risk: fragment(\"\"\"\n        CASE \n          WHEN COUNT(?) = 0 THEN 95\n          WHEN ? > 365 THEN 90\n          WHEN ? > 180 THEN 70\n          WHEN ? > 90 THEN 50\n          WHEN ? > 60 THEN 30\n          WHEN ? > 30 THEN 20\n          ELSE 10\n        END\n      \"\"\", \n      o.id,\n      fragment(\"DATE_PART('day', NOW() - MAX(?))\", o.inserted_at),\n      fragment(\"DATE_PART('day', NOW() - MAX(?))\", o.inserted_at),\n      fragment(\"DATE_PART('day', NOW() - MAX(?))\", o.inserted_at),\n      fragment(\"DATE_PART('day', NOW() - MAX(?))\", o.inserted_at),\n      fragment(\"DATE_PART('day', NOW() - MAX(?))\", o.inserted_at)),\n      last_order_date: max(o.inserted_at)\n    }\nend\n```","ref":"complex-queries.html#customer-lifetime-value-analysis"},{"type":"extras","title":"Key Patterns for Complex Queries - Complex Query Examples","doc":"","ref":"complex-queries.html#key-patterns-for-complex-queries"},{"type":"extras","title":"1. Data Provider Assignment - Complex Query Examples","doc":"- Assign `:data_provider` in `mount/3` or `handle_params/3`\n- Use `{Module, :function, [args]}` tuple format\n- Pass parameters dynamically based on route or user input","ref":"complex-queries.html#1-data-provider-assignment"},{"type":"extras","title":"2. Query Design - Complex Query Examples","doc":"- Use meaningful aliases with `as:` keyword\n- Group by all non-aggregated fields\n- Use `fragment/2` for complex calculations\n- Leverage window functions for advanced analytics","ref":"complex-queries.html#2-query-design"},{"type":"extras","title":"3. Field References - Complex Query Examples","doc":"- Use `assoc: {:alias_name, :field}` for joined fields\n- Ensure field keys match query select keys exactly\n- Use computed fields sparingly for performance","ref":"complex-queries.html#3-field-references"},{"type":"extras","title":"4. Complex Renderers - Complex Query Examples","doc":"- Use `renderer: &function/2` to access full record\n- Implement conditional rendering based on multiple fields\n- Create reusable renderer functions for common patterns","ref":"complex-queries.html#4-complex-renderers"},{"type":"extras","title":"5. Advanced Filters - Complex Query Examples","doc":"- Reference table aliases in dynamic queries\n- Use complex conditions with multiple criteria\n- Combine filters with AND logic automatically\n\nThese examples demonstrate how LiveTable handles sophisticated reporting requirements while maintaining clean, maintainable code.","ref":"complex-queries.html#5-advanced-filters"},{"type":"extras","title":"Troubleshooting Guide","doc":"# Troubleshooting Guide\n\nThis guide covers common issues, error messages, and solutions when working with LiveTable.","ref":"troubleshooting.html"},{"type":"extras","title":"Common Setup Issues - Troubleshooting Guide","doc":"","ref":"troubleshooting.html#common-setup-issues"},{"type":"extras","title":"LiveTable Components Not Rendering - Troubleshooting Guide","doc":"**Problem:** Table doesn't appear or shows blank content.\n\n**Symptoms:**\n- Empty div where table should be\n- No error messages in logs\n- LiveView mounts successfully but no table content\n\n**Solutions:**\n\n1. **Check LiveResource Setup**\n```elixir\n# ✅ Correct setup\ndefmodule YourAppWeb.ProductLive.Index do\n  use YourAppWeb, :live_view\n  use LiveTable.LiveResource, schema: YourApp.Product\n  \n  def fields do\n    [name: %{label: \"Name\"}]\n  end\nend\n\n# ❌ Missing LiveResource\ndefmodule YourAppWeb.ProductLive.Index do\n  use YourAppWeb, :live_view\n  # Missing: use LiveTable.LiveResource\nend\n```\n\n2. **Verify Template Usage**\n```elixir\n# ✅ Correct template\n~H\"\"\"\n<.live_table\n  fields={fields()}\n  filters={filters()}\n  options={@options}\n  streams={@streams}\n/>\n\"\"\"\n\n# ❌ Missing required assigns\n~H\"\"\"\n<.live_table fields={fields()} />\n\"\"\"\n```\n\n3. **Check JavaScript Assets**\n```javascript\n// assets/js/app.js - ensure LiveTable hooks are imported\nimport {LiveTableHooks} from \"../deps/live_table/priv/static/live-table\"\nlet liveSocket = new LiveSocket(\"/live\", Socket, {\n  hooks: {...LiveTableHooks}\n})\n```","ref":"troubleshooting.html#livetable-components-not-rendering"},{"type":"extras","title":"CSS Styles Not Applied - Troubleshooting Guide","doc":"**Problem:** Table appears unstyled or with broken layout.\n\n**Solutions:**\n\n1. **Import CSS Files**\n```css\n/* assets/css/app.css */\n@import \"../../deps/live_table/priv/static/live-table.css\";\n```\n\n2. **Verify Tailwind Configuration**\n```javascript\n// tailwind.config.js\nmodule.exports = {\n  content: [\n    \"./js/**/*.js\",\n    \"../lib/*_web.ex\",\n    \"../lib/*_web/**/*.*ex\",\n    \"../deps/live_table/**/*.ex\"  // Add this line\n  ],\n}\n```\n\n3. **Check Build Process**\n```bash\n# Rebuild assets\ncd assets && npm run build\n# Or for development\ncd assets && npm run dev\n```","ref":"troubleshooting.html#css-styles-not-applied"},{"type":"extras","title":"Query and Data Issues - Troubleshooting Guide","doc":"","ref":"troubleshooting.html#query-and-data-issues"},{"type":"extras","title":"Empty Table Despite Having Data - Troubleshooting Guide","doc":"**Problem:** Table shows \"No records found\" but database has data.\n\n**Debugging Steps:**\n\n1. **Check Schema Configuration**\n```elixir\n# Verify your schema is correct\nuse LiveTable.LiveResource, schema: YourApp.Product\n\n# Check if schema exists and has data\niex> YourApp.Repo.all(YourApp.Product) |> length()\n```\n\n2. **Test Custom Data Provider**\n```elixir\n# If using custom query, test it directly\ndef mount(_params, _session, socket) do\n  socket = assign(socket, :data_provider, {YourApp.Products, :list_products, []})\n  {:ok, socket}\nend\n\n# Test the function\niex> YourApp.Products.list_products() |> YourApp.Repo.all()\n```\n\n3. **Debug Field Configuration**\n```elixir\n# Verify fields exist in your schema/query\ndef fields do\n  [\n    id: %{label: \"ID\"},\n    name: %{label: \"Name\"},  # Ensure this field exists\n    nonexistent_field: %{label: \"Bad\"}  # This will cause issues\n  ]\nend\n```","ref":"troubleshooting.html#empty-table-despite-having-data"},{"type":"extras","title":"Sorting Not Working - Troubleshooting Guide","doc":"**Problem:** Clicking column headers doesn't sort the table.\n\n**Solutions:**\n\n1. **Enable Sortable Fields**\n```elixir\ndef fields do\n  [\n    name: %{label: \"Name\", sortable: true},  # ✅ Sortable\n    description: %{label: \"Description\"}    # ❌ Not sortable\n  ]\nend\n```\n\n2. **Check Field References for Custom Queries**\n```elixir\n# For joined fields, use assoc option\ndef fields do\n  [\n    category_name: %{\n      label: \"Category\",\n      sortable: true,\n      assoc: {:categories, :name}  # Must match query alias\n    }\n  ]\nend\n\n# Ensure your query has the right alias\ndef custom_query do\n  from p in Product,\n    join: c in Category, on: p.category_id == c.id, as: :categories  # Must match assoc\nend\n```\n\n3. **Verify Table Options**\n```elixir\ndef table_options do\n  %{\n    sorting: %{\n      enabled: true  # Ensure sorting is enabled\n    }\n  }\nend\n```","ref":"troubleshooting.html#sorting-not-working"},{"type":"extras","title":"Filters Not Applying - Troubleshooting Guide","doc":"**Problem:** Filter widgets appear but don't filter data.\n\n**Common Causes & Solutions:**\n\n1. **Boolean Filter Dynamic Query Issues**\n```elixir\n# ❌ Incorrect dynamic syntax\ncondition: dynamic([p], p.active = true)  # Wrong: single =\n\n# ✅ Correct dynamic syntax\ncondition: dynamic([p], p.active == true)  # Right: double ==\n```\n\n2. **Select Filter Value Format**\n```elixir\n# ✅ Correct option format\noptions: [\n  %{label: \"Active\", value: [\"active\"]},    # Value must be array\n  %{label: \"Inactive\", value: [\"inactive\"]} # Even for single values\n]\n\n# ❌ Incorrect format\noptions: [\n  %{label: \"Active\", value: \"active\"}       # String won't work\n]\n```\n\n3. **Range Filter Type Mismatch**\n```elixir\n# ✅ Match field type\nprice_range: Range.new(:price, \"price_range\", %{\n  type: :number,  # Field is numeric\n  min: 0,\n  max: 1000\n})\n\n# ❌ Wrong type for field\ncreated_range: Range.new(:inserted_at, \"created_range\", %{\n  type: :number,  # Wrong: inserted_at is datetime\n  min: 0,\n  max: 100\n})\n```","ref":"troubleshooting.html#filters-not-applying"},{"type":"extras","title":"Performance Issues - Troubleshooting Guide","doc":"","ref":"troubleshooting.html#performance-issues"},{"type":"extras","title":"Slow Table Loading - Troubleshooting Guide","doc":"**Problem:** Table takes a long time to load or times out.\n\n**Diagnostic Steps:**\n\n1. **Check Query Performance**\n```elixir\n# Add query logging to see what's being executed\nconfig :logger, level: :debug\n\n# Or inspect queries directly\niex> query = YourApp.Products.list_products()\niex> IO.inspect(Ecto.Adapters.SQL.to_sql(:all, YourApp.Repo, query))\n```\n\n2. **Profile Large Datasets**\n```elixir\n# Check record count\niex> YourApp.Repo.aggregate(YourApp.Product, :count)\n\n# Test with smaller page sizes\ndef table_options do\n  %{\n    pagination: %{\n      enabled: true,\n      sizes: [10, 25],        # Smaller pages\n      default_size: 10\n    }\n  }\nend\n```\n\n3. **Optimize Queries**\n```sql\n-- Add indexes for sorted/filtered columns\nCREATE INDEX idx_products_name ON products(name);\nCREATE INDEX idx_products_active ON products(active);\nCREATE INDEX idx_products_price ON products(price);\n\n-- For joined queries\nCREATE INDEX idx_products_category_id ON products(category_id);\n```\n\n4. **Limit Eager Loading**\n```elixir\n# ❌ Loading too much associated data\nfrom p in Product,\n  preload: [:category, :supplier, :reviews, :orders]  # Too much\n\n# ✅ Load only what you need\nfrom p in Product,\n  join: c in Category, on: p.category_id == c.id,\n  select: %{id: p.id, name: p.name, category_name: c.name}\n```","ref":"troubleshooting.html#slow-table-loading"},{"type":"extras","title":"Memory Issues with Large Exports - Troubleshooting Guide","doc":"**Problem:** Export jobs fail or consume too much memory.\n\n**Solutions:**\n\n1. **Configure Oban Limits**\n```elixir\nconfig :your_app, Oban,\n  queues: [\n    exports: [limit: 2, paused: false],  # Limit concurrent exports\n    default: 5\n  ]\n```\n\n2. **Increase Timeout**\n```elixir\nconfig :live_table,\n  export_timeout: 600_000  # 10 minutes\n```\n\n3. **Monitor Export File Sizes**\n```bash\n# Check export directory size\ndu -sh priv/static/exports/\n\n# Clean up old exports\nfind priv/static/exports/ -name \"*.csv\" -mtime +7 -delete\n```","ref":"troubleshooting.html#memory-issues-with-large-exports"},{"type":"extras","title":"Runtime Errors - Troubleshooting Guide","doc":"### \"Schema not found\" Errors\n\n**Error:** `ArgumentError: schema YourApp.Product is not an Ecto schema`\n\n**Solutions:**\n\n1. **Verify Schema Definition**\n```elixir\n# Ensure your schema is properly defined\ndefmodule YourApp.Product do\n  use Ecto.Schema\n  \n  schema \"products\" do\n    field :name, :string\n    # ... other fields\n  end\nend\n```\n\n2. **Check Module Compilation**\n```bash\n# Recompile the schema module\nmix compile --force\n```\n\n3. **Verify Import/Alias**\n```elixir\n# Make sure schema is accessible\ndefmodule YourAppWeb.ProductLive.Index do\n  use YourAppWeb, :live_view\n  use LiveTable.LiveResource, schema: YourApp.Product  # Full module name\nend\n```\n\n### \"Function not exported\" Errors\n\n**Error:** `UndefinedFunctionError: function YourApp.Products.list_products/0 is undefined`\n\n**Solutions:**\n\n1. **Verify Data Provider Function**\n```elixir\n# Ensure function exists and is exported\ndefmodule YourApp.Products do\n  def list_products do  # Make sure it's public\n    # Your query here\n  end\n  \n  # If it needs to accept parameters\n  def list_products(_params \\\\ []) do\n    # Your query here\n  end\nend\n```\n\n2. **Check Function Arity**\n```elixir\n# Data provider should match expected signature\nassign(socket, :data_provider, {YourApp.Products, :list_products, []})\n\n# Function should accept right number of arguments\ndef list_products(params \\\\ []), do: query\n```","ref":"troubleshooting.html#runtime-errors"},{"type":"extras","title":"Dynamic Query Errors - Troubleshooting Guide","doc":"**Error:** `Ecto.Query.CompileError: unbound variable`\n\n**Common Issues:**\n\n1. **Binding Mismatch**\n```elixir\n# ❌ Incorrect binding\ncondition: dynamic([product], product.active == true)  # Wrong variable name\n\n# ✅ Correct binding  \ncondition: dynamic([p], p.active == true)  # Matches query binding\n```\n\n2. **Missing Join References**\n```elixir\n# ❌ Referencing non-existent join\ncondition: dynamic([p, category: c], c.active == true)  # No category join\n\n# ✅ Ensure join exists in query\ndef custom_query do\n  from p in Product,\n    join: c in Category, on: p.category_id == c.id, as: :category\nend\n```","ref":"troubleshooting.html#dynamic-query-errors"},{"type":"extras","title":"Export Issues - Troubleshooting Guide","doc":"","ref":"troubleshooting.html#export-issues"},{"type":"extras","title":"PDF Generation Failing - Troubleshooting Guide","doc":"**Error:** `Export failed: Typst not found`\n\n**Solutions:**\n\n1. **Install Typst**\n```bash\n# macOS\nbrew install typst\n\n# Ubuntu/Debian\nwget https://github.com/typst/typst/releases/latest/download/typst-x86_64-unknown-linux-musl.tar.xz\ntar -xf typst-x86_64-unknown-linux-musl.tar.xz\nsudo mv typst-x86_64-unknown-linux-musl/typst /usr/local/bin/\n\n# Verify installation\ntypst --version\n```\n\n2. **Check System PATH**\n```bash\n# Ensure Typst is in PATH\nwhich typst\necho $PATH\n```\n\n3. **Verify File Permissions**\n```bash\n# Check export directory permissions\nls -la priv/static/exports/\nchmod 755 priv/static/exports/\n```","ref":"troubleshooting.html#pdf-generation-failing"},{"type":"extras","title":"CSV Export Character Encoding Issues - Troubleshooting Guide","doc":"**Problem:** Special characters appear corrupted in exported CSV.\n\n**Solutions:**\n\n1. **Configure UTF-8 Encoding**\n```elixir\ndef table_options do\n  %{\n    exports: %{\n      enabled: true,\n      formats: [:csv],\n      csv: %{\n        encoding: :utf8,      # Explicit UTF-8\n        delimiter: \",\"\n      }\n    }\n  }\nend\n```\n\n2. **Test with Different Applications**\n```bash\n# Test CSV in terminal\nfile exports/products_2024-01-01.csv\ncat exports/products_2024-01-01.csv | head -5\n```","ref":"troubleshooting.html#csv-export-character-encoding-issues"},{"type":"extras","title":"LiveView Integration Issues - Troubleshooting Guide","doc":"","ref":"troubleshooting.html#liveview-integration-issues"},{"type":"extras","title":"Handle Params Not Working - Troubleshooting Guide","doc":"**Problem:** URL parameters not updating table state.\n\n**Solutions:**\n\n1. **Implement handle_params**\n```elixir\ndef handle_params(params, _url, socket) do\n  # LiveTable needs this to handle URL parameters\n  {:noreply, apply_action(socket, socket.assigns.live_action, params)}\nend\n\ndef apply_action(socket, :index, _params) do\n  socket\n  |> assign(:page_title, \"Products\")\nend\n```\n\n2. **Check Router Configuration**\n```elixir\n# Ensure live route is properly configured\nlive \"/products\", ProductLive.Index, :index\n```","ref":"troubleshooting.html#handle-params-not-working"},{"type":"extras","title":"Event Handling Conflicts - Troubleshooting Guide","doc":"**Problem:** Custom LiveView events conflict with LiveTable.\n\n**Solutions:**\n\n1. **Use Unique Event Names**\n```elixir\n# ❌ Generic event names that might conflict\ndef handle_event(\"update\", params, socket)\n\n# ✅ Specific event names\ndef handle_event(\"update_product\", params, socket)\ndef handle_event(\"toggle_featured\", params, socket)\n```\n\n2. **Check Event Propagation**\n```elixir\n# Use phx-click-away or stop propagation if needed\n~H\"\"\"\n \n  Action\n \n\"\"\"\n```","ref":"troubleshooting.html#event-handling-conflicts"},{"type":"extras","title":"Debugging Techniques - Troubleshooting Guide","doc":"","ref":"troubleshooting.html#debugging-techniques"},{"type":"extras","title":"Enable Debug Logging - Troubleshooting Guide","doc":"```elixir\n# config/dev.exs\nconfig :logger, :console,\n  level: :debug,\n  format: \"[$level] $message\\n\"\n\n# See LiveTable internal operations\nconfig :live_table,\n  debug: true\n```","ref":"troubleshooting.html#enable-debug-logging"},{"type":"extras","title":"Inspect Socket State - Troubleshooting Guide","doc":"```elixir\ndef handle_params(params, _url, socket) do\n  IO.inspect(socket.assigns, label: \"Socket Assigns\")\n  IO.inspect(params, label: \"URL Params\")\n  {:noreply, socket}\nend\n```","ref":"troubleshooting.html#inspect-socket-state"},{"type":"extras","title":"Test Queries in IEx - Troubleshooting Guide","doc":"```elixir\n# Test your queries directly\niex> alias YourApp.{Repo, Product}\niex> query = from p in Product, where: p.active == true\niex> Repo.all(query) |> length()\n\n# Test custom data providers\niex> YourApp.Products.list_products() |> Repo.all()\n```","ref":"troubleshooting.html#test-queries-in-iex"},{"type":"extras","title":"Check Component State - Troubleshooting Guide","doc":"```elixir\n# Add temporary debug output to templates\n~H\"\"\"\n \n  Debug: <%= inspect(@streams) %>\n \n<.live_table ... />\n\"\"\"\n```","ref":"troubleshooting.html#check-component-state"},{"type":"extras","title":"Monitor Background Jobs - Troubleshooting Guide","doc":"```elixir\n# Check Oban job status\niex> Oban.Job |> YourApp.Repo.all()\n\n# Check specific export jobs\niex> \nOban.Job \n|> where([j], j.queue == \"exports\") \n|> order_by([j], desc: j.inserted_at)\n|> limit(10)\n|> YourApp.Repo.all()\n```","ref":"troubleshooting.html#monitor-background-jobs"},{"type":"extras","title":"Getting Help - Troubleshooting Guide","doc":"","ref":"troubleshooting.html#getting-help"},{"type":"extras","title":"Check Documentation - Troubleshooting Guide","doc":"- Review API documentation for specific modules\n- Look at example implementations\n- Check configuration options","ref":"troubleshooting.html#check-documentation"},{"type":"extras","title":"Common Debug Commands - Troubleshooting Guide","doc":"```bash\n# Check LiveTable version\nmix deps | grep live_table\n\n# Recompile everything\nmix deps.clean live_table\nmix deps.get\nmix compile\n\n# Check for compilation errors\nmix compile --warnings-as-errors\n\n# Run tests to verify setup\nmix test\n```","ref":"troubleshooting.html#common-debug-commands"},{"type":"extras","title":"Community Resources - Troubleshooting Guide","doc":"- Check GitHub issues for similar problems\n- Look at LiveTable examples in the repository\n- Search for error messages in issue tracker","ref":"troubleshooting.html#community-resources"},{"type":"extras","title":"Performance Profiling - Troubleshooting Guide","doc":"```elixir\n# Profile memory usage\n:observer.start()\n\n# Profile specific operations\n:fprof.apply(YourApp.Products, :list_products, [])\n:fprof.profile()\n:fprof.analyse()\n```\n\nRemember: Most LiveTable issues stem from incorrect configuration, missing dependencies, or query problems. Always verify your basic setup before investigating complex issues.","ref":"troubleshooting.html#performance-profiling"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}